/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: DataCenterResourceUsageEvent.java 36 2011-06-20 14:38:41Z dangtran $
 *
 */

package org.ow2.sirocco.cloudmanager.admin.event;

import java.io.Serializable;

import org.ow2.sirocco.cloudmanager.clustermanager.api.Domain;

/**
 * Cloud administration event that indicates that the resource usage (CPU,
 * memory, storage) of the datacenter has changed.
 */
public class DataCenterResourceUsageEvent extends CloudAdminEvent implements Serializable {
    private static final long serialVersionUID = -8422306583562523537L;

    private Domain rootDomain;

    /**
     * Constructs a DataCenterResourceUsageEvent object
     */
    public DataCenterResourceUsageEvent() {
        super();
    }

    /**
     * Constructs a DataCenterResourceUsageEvent object with a top-level domain
     * 
     * @param rootDomain the top-level domain value object of the datacenter
     */
    public DataCenterResourceUsageEvent(final Domain rootDomain) {
        this.rootDomain = rootDomain;
    }

    /**
     * Returns the top-level domain value object of the datacenter
     * 
     * @return top-level domain value object of the datacenter
     */
    public Domain getRootDomain() {
        return this.rootDomain;
    }

    /**
     * Sets the top-level domain value object of the datacenter
     * 
     * @param rootDomain top-level domain value object of the datacenter
     */
    public void setRootDomain(final Domain rootDomain) {
        this.rootDomain = rootDomain;
    }

}
