/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: ProjectVO.java 202 2011-07-26 16:21:46Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.admin.vo;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.logging.Logger;

import org.ow2.sirocco.cloudmanager.business.Project;
import org.ow2.sirocco.cloudmanager.business.User;
import org.ow2.sirocco.cloudmanager.common.jndilocator.ServiceLocator;
import org.ow2.sirocco.cloudmanager.service.IRemoteUserManager;

/**
 * Business object representing a project
 */
public class ProjectVO implements Serializable {
    private static final long serialVersionUID = 7820619916406005211L;

    private String projectId;

    private String name;

    private String description;

    private ResourceQuotaVO quotaVo;

    private Date createDate;

    private List<UserVO> userVoList;

    private String owner;

    private static Logger logger = Logger.getLogger(ProjectVO.class.getName());

    private transient IRemoteUserManager userManager;

    public ProjectVO() {
        this.initialize();

    }

    private void initialize() {
        try {
            this.userManager = (IRemoteUserManager) ServiceLocator.getInstance().getRemoteObject("UserManagerBean");
        } catch (Exception ex) {
            ProjectVO.logger.severe(ex.getMessage());
        }
    }

    /**
     * Builds a new ProjectVO from its business object counterpart
     * 
     * @param from the Project business object
     * @param withUsersList TODO
     * @return a new ProjectVO
     */
    public static ProjectVO from(final Project from, final Boolean withUsersList) {
        ProjectVO projectVo = new ProjectVO();
        projectVo.projectId = from.getProjectId();
        projectVo.createDate = from.getCreateDate();
        projectVo.description = from.getDescription();
        projectVo.name = from.getName();
        ResourceQuotaVO rqVo = new ResourceQuotaVO();
        rqVo.setCpuQuota(from.getResourceQuota().getCpuQuota());
        rqVo.setDiskQuotaInMB(from.getResourceQuota().getDiskQuotaInMB());
        rqVo.setMaxNumberOfVMs(from.getResourceQuota().getVmQuota());
        rqVo.setRamQuotaInMB(from.getResourceQuota().getRamQuotaInMB());
        projectVo.setQuotaVo(rqVo);
        if (withUsersList) {
            projectVo.setUserVoList(from.getProjectId());
        }
        projectVo.setOwner(from.getOwner().getUsername());
        return projectVo;
    }

    public String getName() {
        return this.name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    public String getDescription() {
        return this.description;
    }

    public void setDescription(final String description) {
        this.description = description;
    }

    public Date getCreateDate() {
        return this.createDate;
    }

    public void setCreateDate(final Date createDate) {
        this.createDate = createDate;
    }

    public String getProjectId() {
        return this.projectId;
    }

    public void setProjectId(final String projectId) {
        this.projectId = projectId;
    }

    public synchronized void setUserVoList(final String projectId) {
        this.userVoList = new ArrayList<UserVO>();
        List<User> userList = new ArrayList<User>();
        userList = this.userManager.getUsersByProjectId(projectId);
        for (User user : userList) {
            this.userVoList.add(new UserVO().from(user, projectId, false));
        }
    }

    public List<UserVO> getUserVoList() {
        return this.userVoList;
    }

    public void setUserVoList(final List<UserVO> userVoList) {
        this.userVoList = userVoList;
    }

    public void setOwner(final String owner) {
        this.owner = owner;
    }

    public String getOwner() {
        return this.owner;
    }

    public void setQuotaVo(final ResourceQuotaVO quotaVo) {
        this.quotaVo = quotaVo;
    }

    public ResourceQuotaVO getQuotaVo() {
        return this.quotaVo;
    }

}
