/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: CloudProvider.java 265 2011-08-18 08:01:55Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.business;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;

/**
 * A CloudProvider can provision compute resources (virtual machines along with
 * their network and storage resources). A CloudProvider can represent a private
 * cloud (e.g. an OpenStack instance available within a corporate network) or a
 * public cloud (e.g. Amazon EC2).
 */
@Entity
public class CloudProvider {

    private Integer id;

    private String cloudProviderType;

    private List<CloudProviderAccount> cloudProviderAccounts;

    // TODO ComputeCapabilities, StorageCapabilities, NetworkCapabilities.

    public CloudProvider() {
        // Let's initialize associations' attributs.
        this.cloudProviderAccounts = new ArrayList<CloudProviderAccount>();
    }

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    public Integer getId() {
        return this.id;
    }

    public void setId(final Integer id) {
        this.id = id;
    }

    /**
     * XXX Cf. org.ow2.sirocco.multicloud.api.MultiCloudConstants.
     * CLOUD_PROVIDER_TYPE, CLOUD_PROVIDER_TYPE_DUMMY,
     * CLOUD_PROVIDER_TYPE_OPENSTACK_NOVA, ...
     * 
     * @return
     */
    public String getCloudProviderType() {
        return this.cloudProviderType;
    }

    public void setCloudProviderType(final String cloudProviderType) {
        this.cloudProviderType = cloudProviderType;
    }

    @OneToMany(mappedBy = "cloudProvider", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
    public List<CloudProviderAccount> getCloudProviderAccount() {
        return this.cloudProviderAccounts;
    }

    public void setCloudProviderAccount(final List<CloudProviderAccount> cloudProviderAccounts) {
        this.cloudProviderAccounts = cloudProviderAccounts;
    }

    @Override
    public String toString() {
        String s = this.getClass().getSimpleName() + "=[id=" + this.getId() + ", cloudProviderType="
            + this.getCloudProviderType() + "]";
        return s;
    }
}
