/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: CloudProviderAccount.java 372 2011-09-30 14:49:32Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.business;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

@Entity
public class CloudProviderAccount {

    private Integer id;

    // TODO Need to have a union of all attributes describing different cloud
    // accounts...
    private String login;

    private String password;

    // XXX creationTime, startTime, terminationTime ==> cf. MultiCloud
    // Controller Spec (they were attributs of removed
    // ProjectCloudProviderAccountAssociation class).

    private Date creationTime;

    private Date startTime;

    private Date terminationTime;

    private Collection<Project> projects;

    // TODO Antonin -> Frederic: We don't have any Network entity/class that
    // works without any SystemTemplate or SystemInstance==> How should we
    // handle the UML Network class?
    // private Collection<Network> networks;

    private Collection<VirtualMachine> virtualMachines;

    private Collection<Volume> volumes;

    private List<VMImage> vmImages;

    private CloudProvider cloudProvider;

    public CloudProviderAccount() {
        this.projects = new ArrayList<Project>();
        // this.networks = new ArrayList<Network>();
        this.virtualMachines = new ArrayList<VirtualMachine>();
        this.volumes = new ArrayList<Volume>();
        this.vmImages = new ArrayList<VMImage>();
    }

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    public Integer getId() {
        return this.id;
    }

    public void setId(final Integer id) {
        this.id = id;
    }

    public String getLogin() {
        return this.login;
    }

    public void setLogin(final String login) {
        this.login = login;
    }

    public String getPassword() {
        return this.password;
    }

    public void setPassword(final String password) {
        this.password = password;
    }

    @Temporal(TemporalType.TIMESTAMP)
    public Date getCreationTime() {
        return this.creationTime;
    }

    public void setCreationTime(final Date creationTime) {
        this.creationTime = creationTime;
    }

    @Temporal(TemporalType.TIMESTAMP)
    public Date getStartTime() {
        return this.startTime;
    }

    public void setStartTime(final Date startTime) {
        this.startTime = startTime;
    }

    @Temporal(TemporalType.TIMESTAMP)
    public Date getTerminationTime() {
        return this.terminationTime;
    }

    public void setTerminationTime(final Date terminationTime) {
        this.terminationTime = terminationTime;
    }

    @ManyToMany(mappedBy = "cloudProviderAccounts")
    public Collection<Project> getProjects() {
        return this.projects;
    }

    public void setProjects(final List<Project> projects) {
        this.projects = projects;
    }

    @OneToMany(mappedBy = "cloudProviderAccount", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
    public Collection<VirtualMachine> getVirtualMachines() {
        return this.virtualMachines;
    }

    public void setVirtualMachines(final List<VirtualMachine> virtualMachines) {
        this.virtualMachines = virtualMachines;
    }

    @OneToMany(mappedBy = "cloudProviderAccount", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
    public Collection<Volume> getVolumes() {
        return this.volumes;
    }

    public void setVolumes(final List<Volume> volumes) {
        this.volumes = volumes;
    }

    /**
     * CloudProviderAccount is the OWNER, we must use it (instead of VMImage) in
     * order to associate, persist, ...
     * 
     * @return the associated vmImages.
     */
    @ManyToMany
    @JoinTable(name = "CLOUDPROVIDERACCOUNT_VMIMAGE")
    public Collection<VMImage> getVmImages() {
        return this.vmImages;
    }

    public void setVmImages(final List<VMImage> vmImages) {
        this.vmImages = vmImages;
    }

    @ManyToOne
    @JoinColumn(name = "CloudProvider_id")
    public CloudProvider getCloudProvider() {
        return this.cloudProvider;
    }

    public void setCloudProvider(final CloudProvider cloudProvider) {
        this.cloudProvider = cloudProvider;
    }

    @Override
    public String toString() {
        // In order to avoid a "loop" pb, project is not present in this
        // toString() method.
        String s = this.getClass().getSimpleName() + "=[id=" + this.getId() + ", login=" + this.getLogin() + ", password="
            + this.getPassword() + ", creationTime=" + this.getCreationTime() + ", startTime=" + this.getStartTime()
            + ", terminationTime=" + this.getTerminationTime() + ", cloudProvider=" + this.getCloudProvider() + "]";
        return s;
    }

}
