/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: NetworkInstance.java 197 2011-07-26 10:19:46Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.business;

import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;

@Entity
public class NetworkInstance {

    public static enum NetworkInstanceStatus {
        CREATED, DESTROYED
    }

    // Warning "key" can't be an attribut --> this is a reserved attribut !

    /**
     * This is the networkId
     */
    private Integer id;

    private NetworkInstanceStatus status;

    private SystemInstance systemInstance;

    private NetworkTemplate networkTemplate;

    public NetworkInstance() {
        // This contructor is required by JPA when using non-empty
        // contructor(s).
    }

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    public Integer getId() {
        return this.id;
    }

    public void setId(final Integer id) {
        this.id = id;
    }

    @Enumerated(EnumType.STRING)
    public NetworkInstanceStatus getStatus() {
        return this.status;
    }

    public void setStatus(final NetworkInstanceStatus status) {
        this.status = status;
    }

    @ManyToOne
    // @JoinColumn(name = "System_id", nullable = false)
    @JoinColumn(name = "SystemInstance_id")
    public SystemInstance getSystemInstance() {
        return this.systemInstance;
    }

    public void setSystemInstance(final SystemInstance systemInstance) {
        this.systemInstance = systemInstance;
    }

    @ManyToOne
    // @JoinColumn(name = "NetworkTemplate_id", nullable = false)
    @JoinColumn(name = "NetworkTemplate_id")
    public NetworkTemplate getNetworkTemplate() {
        return this.networkTemplate;
    }

    public void setNetworkTemplate(final NetworkTemplate networkTemplate) {
        this.networkTemplate = networkTemplate;
    }

    @Override
    public String toString() {
        String s = this.getClass().getSimpleName() + "=[id=" + this.getId() + ", status=" + this.getStatus()
            + ", systemInstance=" + this.getSystemInstance() + ", networkTemplate=" + this.getNetworkTemplate() + "]";
        return s;
    }
}
