/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: NetworkTemplate.java 219 2011-08-01 12:33:08Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.business;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;

@SuppressWarnings("serial")
@Entity
public class NetworkTemplate implements Serializable {

    // Warning "key" can't be an attribut --> this is a reserved attribut !

    /**
     * This is the networkTemplateId
     */
    private Integer id;

    private String name;

    private SystemTemplate systemTemplate;

    private Collection<NetworkInterfaceTemplate> networkInterfaceTemplates;

    private Collection<NetworkInstance> networks;

    public NetworkTemplate() {
        // This contructor is required by JPA when using non-empty
        // contructor(s).

        // Let's initialize associations' attributs.
        this.networkInterfaceTemplates = new ArrayList<NetworkInterfaceTemplate>();
        this.networks = new ArrayList<NetworkInstance>();
    }

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    public Integer getId() {
        return this.id;
    }

    public void setId(final Integer id) {
        this.id = id;
    }

    public String getName() {
        return this.name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    @ManyToOne
    // @JoinColumn(name = "SystemTemplate_id", nullable = false)
    @JoinColumn(name = "SystemTemplate_id")
    public SystemTemplate getSystemTemplate() {
        return this.systemTemplate;
    }

    public void setSystemTemplate(final SystemTemplate systemTemplate) {
        this.systemTemplate = systemTemplate;
    }

    @OneToMany(mappedBy = "networkTemplate", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
    public Collection<NetworkInterfaceTemplate> getNetworkInterfaceTemplates() {
        return this.networkInterfaceTemplates;
    }

    public void setNetworkInterfaceTemplates(final Collection<NetworkInterfaceTemplate> networkInterfaceTemplates) {
        this.networkInterfaceTemplates = networkInterfaceTemplates;
    }

    @OneToMany(mappedBy = "networkTemplate", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
    public Collection<NetworkInstance> getNetworks() {
        return this.networks;
    }

    public void setNetworks(final Collection<NetworkInstance> networks) {
        this.networks = networks;
    }

    @Override
    public String toString() {
        String s = this.getClass().getSimpleName() + "=[id=" + this.getId() + ", name=" + this.getName() + ", systemTemplate="
            + this.getSystemTemplate() + "]";
        return s;
    }
}
