/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: Project.java 289 2011-08-31 10:01:02Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.business;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

@Entity
public class Project implements Serializable {

    private static final long serialVersionUID = -416537505440187795L;

    private Integer id;

    private String projectId;

    private String name;

    private String description;

    private ResourceQuota resourceQuota;

    private Date createDate;

    private Collection<VirtualMachine> virtualMachines;

    private Collection<Volume> volumes;

    private Collection<VMImage> vmImages;

    private Collection<RoleAssignment> assignments;

    private User owner;

    private Collection<SystemTemplate> systemTemplates;

    private Collection<SystemInstance> systemInstances;

    private Collection<CloudProviderAccount> cloudProviderAccounts;

    public Project() {
        this.assignments = new ArrayList<RoleAssignment>();
        this.systemTemplates = new ArrayList<SystemTemplate>();
        this.systemInstances = new ArrayList<SystemInstance>();
        this.cloudProviderAccounts = new ArrayList<CloudProviderAccount>();
    }

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    public Integer getId() {
        return this.id;
    }

    @OneToMany(mappedBy = "project")
    public Collection<RoleAssignment> getAssignments() {
        return this.assignments;
    }

    @OneToMany(mappedBy = "project")
    public Collection<VMImage> getVmImages() {
        return this.vmImages;
    }

    @OneToMany(mappedBy = "project")
    public Collection<VirtualMachine> getVirtualMachines() {
        return this.virtualMachines;
    }

    @OneToMany(mappedBy = "project")
    public Collection<Volume> getVolumes() {
        return this.volumes;
    }

    @OneToOne(cascade = CascadeType.ALL)
    public ResourceQuota getResourceQuota() {
        return this.resourceQuota;
    }

    @OneToOne
    public User getOwner() {
        return this.owner;
    }

    @Temporal(TemporalType.TIMESTAMP)
    public Date getCreateDate() {
        return this.createDate;
    }

    public String getDescription() {
        return this.description;
    }

    public String getName() {
        return this.name;
    }

    public void setCreateDate(final Date createDate) {
        this.createDate = createDate;
    }

    public void setDescription(final String description) {
        this.description = description;
    }

    public void setId(final Integer id) {
        this.id = id;
    }

    public void setName(final String name) {
        this.name = name;
    }

    public void setAssignments(final Collection<RoleAssignment> assignments) {
        this.assignments = assignments;
    }

    public void setVmImages(final Collection<VMImage> vmImages) {
        this.vmImages = vmImages;
    }

    public void setVirtualMachines(final Collection<VirtualMachine> virtualMachines) {
        this.virtualMachines = virtualMachines;
    }

    public void setVolumes(final Collection<Volume> volumes) {
        this.volumes = volumes;
    }

    public void setResourceQuota(final ResourceQuota resourceQuota) {
        this.resourceQuota = resourceQuota;
    }

    public void setOwner(final User owner) {
        this.owner = owner;
    }

    @Column(unique = true)
    public String getProjectId() {
        return this.projectId;
    }

    public void setProjectId(final String projectId) {
        this.projectId = projectId;
    }

    @OneToMany(mappedBy = "project", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
    public Collection<SystemTemplate> getSystemTemplates() {
        return this.systemTemplates;
    }

    public void setSystemTemplates(final Collection<SystemTemplate> systemTemplates) {
        this.systemTemplates = systemTemplates;
    }

    @OneToMany(mappedBy = "project", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
    public Collection<SystemInstance> getSystemInstances() {
        return this.systemInstances;
    }

    public void setSystemInstances(final Collection<SystemInstance> systemInstances) {
        this.systemInstances = systemInstances;
    }

    @ManyToMany
    @JoinTable(name = "PROJECT_CLOUDPROVIDERACCOUNT")
    /**
     * Project is the OWNER, we must use it (instead of CloudProviderAccount) in order to associate, persist, ...
     */
    public Collection<CloudProviderAccount> getCloudProviderAccounts() {
        return this.cloudProviderAccounts;
    }

    public void setCloudProviderAccounts(final Collection<CloudProviderAccount> cloudProviderAccounts) {
        this.cloudProviderAccounts = cloudProviderAccounts;
    }

}
