/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: SystemInstance.java 219 2011-08-01 12:33:08Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.business;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;

@SuppressWarnings("serial")
@Entity
public class SystemInstance implements Serializable {

    public static enum SystemInstanceStatus {
        CREATING, CREATED, STARTING, STARTED, STOPPING, STOPPED, DESTROYING, DESTROYED
    }

    // Warning "key" can't be an attribut --> this is a reserved attribut !

    /**
     * This is the systemInstanceId
     */
    private Integer id;

    private SystemInstanceStatus status;

    private User user;

    private Project project;

    private SystemTemplate systemTemplate;

    private Collection<VirtualMachine> virtualMachines;

    private Collection<Volume> volumes;

    private Collection<NetworkInstance> networks;

    public SystemInstance() {
        // This contructor is required by JPA when using non-empty
        // contructor(s).

        // Let's initialize associations' attributs.
        this.virtualMachines = new ArrayList<VirtualMachine>();
        this.volumes = new ArrayList<Volume>();
        this.networks = new ArrayList<NetworkInstance>();
    }

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    public Integer getId() {
        return this.id;
    }

    public void setId(final Integer id) {
        this.id = id;
    }

    @Enumerated(EnumType.STRING)
    public SystemInstanceStatus getStatus() {
        return this.status;
    }

    public void setStatus(final SystemInstanceStatus status) {
        this.status = status;
    }

    @ManyToOne
    // @JoinColumn(name = "User_id", nullable = false)
    @JoinColumn(name = "User_id")
    public User getUser() {
        return this.user;
    }

    public void setUser(final User user) {
        this.user = user;
    }

    @ManyToOne
    // @JoinColumn(name = "Project_id", nullable = false)
    @JoinColumn(name = "Project_id")
    public Project getProject() {
        return this.project;
    }

    public void setProject(final Project project) {
        this.project = project;
    }

    @ManyToOne
    // @JoinColumn(name = "SystemTemplate_id", nullable = false)
    @JoinColumn(name = "SystemTemplate_id")
    public SystemTemplate getSystemTemplate() {
        return this.systemTemplate;
    }

    public void setSystemTemplate(final SystemTemplate systemTemplate) {
        this.systemTemplate = systemTemplate;
    }

    @OneToMany(mappedBy = "systemInstance", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
    public Collection<VirtualMachine> getVirtualMachines() {
        return this.virtualMachines;
    }

    public void setVirtualMachines(final Collection<VirtualMachine> virtualMachines) {
        this.virtualMachines = virtualMachines;
    }

    @ManyToMany
    @JoinTable(name = "SYSTEMINSTANCE_VOLUME")
    /**
     * SystemInstance est le OWNER, il faut passer par lui pour associer
     * et persister. (SystemInstance is the OWNER, we must use it (instead of Volume) in order to associate, and persist).
     */
    public Collection<Volume> getVolumes() {
        return this.volumes;
    }

    public void setVolumes(final Collection<Volume> volumes) {
        this.volumes = volumes;
    }

    @OneToMany(mappedBy = "systemInstance", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
    public Collection<NetworkInstance> getNetworks() {
        return this.networks;
    }

    public void setNetworks(final Collection<NetworkInstance> networks) {
        this.networks = networks;
    }

    @Override
    public String toString() {
        String s = this.getClass().getSimpleName() + "=[id=" + this.getId() + ", status=" + this.getStatus() + ", user="
            + this.getUser() + ", project=" + this.getProject() + ", systemTemplate=" + this.getSystemTemplate() + "]";
        return s;
    }
}
