/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: SystemTemplate.java 219 2011-08-01 12:33:08Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.business;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;

@SuppressWarnings("serial")
@Entity
public class SystemTemplate implements Serializable {

    public static enum SystemTemplateStatus {
        IN_CONFIGURATION, AVAILABLE, DESTROYED
    }

    // Warning "key" can't be an attribut --> this is a reserved attribut !

    /**
     * This is the systemTemplateId
     */
    private Integer id;

    /**
     * Unique.
     */
    private String name;

    private SystemTemplateStatus status;

    private User user;

    private Project project;

    private List<VirtualMachineTemplate> virtualMachineTemplates;

    private Collection<NetworkTemplate> networkTemplates;

    private Collection<VolumeTemplate> volumeTemplates;

    private Collection<SystemInstance> systemInstances;

    private Collection<CustomizationProperties> customizationProperties;

    public SystemTemplate() {
        // This contructor is required by JPA when using non-empty
        // contructor(s).

        // Let's initialize associations' attributs.
        this.virtualMachineTemplates = new ArrayList<VirtualMachineTemplate>();
        this.networkTemplates = new ArrayList<NetworkTemplate>();
        this.volumeTemplates = new ArrayList<VolumeTemplate>();
        this.systemInstances = new ArrayList<SystemInstance>();
        this.customizationProperties = new ArrayList<CustomizationProperties>();

    }

    // /**
    // * this.id is automatically generated.
    // */
    // public SystemTemplate(final String name, final String status,
    // final CustomizationProperties customizationProperties) {
    // super();
    //
    // // this.id is automatically generated.
    //
    // this.setName(name);
    // this.setStatus(status);
    // this.setCustomizationProperties(customizationProperties);
    // }

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    public Integer getId() {
        return this.id;
    }

    public void setId(final Integer id) {
        this.id = id;
    }

    @Column(unique = true)
    public String getName() {
        return this.name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    @Enumerated(EnumType.STRING)
    public SystemTemplateStatus getStatus() {
        return this.status;
    }

    public void setStatus(final SystemTemplateStatus status) {
        this.status = status;
    }

    @ManyToOne
    // @JoinColumn(name = "User_id", nullable = false)
    @JoinColumn(name = "User_id")
    public User getUser() {
        return this.user;
    }

    public void setUser(final User user) {
        this.user = user;
    }

    @ManyToOne
    // @JoinColumn(name = "Project_id", nullable = false)
    @JoinColumn(name = "Project_id")
    public Project getProject() {
        return this.project;
    }

    public void setProject(final Project project) {
        this.project = project;
    }

    /**
     * Be careful, the order you add the virtualMachineTemplates defines the
     * order in which the VM will be started (it corresponds to the OVF's
     * StartupSection).
     */
    @OneToMany(mappedBy = "systemTemplate", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
    public List<VirtualMachineTemplate> getVirtualMachineTemplates() {
        return this.virtualMachineTemplates;
    }

    public void setVirtualMachineTemplates(final List<VirtualMachineTemplate> virtualMachineTemplates) {
        this.virtualMachineTemplates = virtualMachineTemplates;
    }

    @OneToMany(mappedBy = "systemTemplate", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
    public Collection<NetworkTemplate> getNetworkTemplates() {
        return this.networkTemplates;
    }

    public void setNetworkTemplates(final Collection<NetworkTemplate> networkTemplates) {
        this.networkTemplates = networkTemplates;
    }

    @OneToMany(mappedBy = "systemTemplate", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
    public Collection<VolumeTemplate> getVolumeTemplates() {
        return this.volumeTemplates;
    }

    public void setVolumeTemplates(final Collection<VolumeTemplate> volumeTemplates) {
        this.volumeTemplates = volumeTemplates;
    }

    @OneToMany(mappedBy = "systemTemplate", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
    public Collection<SystemInstance> getSystemInstances() {
        return this.systemInstances;
    }

    public void setSystemInstances(final Collection<SystemInstance> systemInstances) {
        this.systemInstances = systemInstances;
    }

    @OneToMany(mappedBy = "systemTemplate", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
    public Collection<CustomizationProperties> getCustomizationProperties() {
        return this.customizationProperties;
    }

    public void setCustomizationProperties(final Collection<CustomizationProperties> customizationProperties) {
        this.customizationProperties = customizationProperties;
    }

    @Override
    public String toString() {
        // Adding, e.g., this.getCustomizationProperties() to s, leads to a
        // (silent) error, because CustomizationProperties' toString() itself
        // calls SystemTemplate toString Method.
        String s = this.getClass().getSimpleName() + "=[id=" + this.getId() + ", name=" + this.getName() + ", status="
            + this.getStatus() + ", user=" + this.getUser() + ", project=" + this.getProject() + "]";
        return s;
    }
}
