/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: VirtualMachineTemplate.java 220 2011-08-01 13:13:33Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.business;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;

@SuppressWarnings("serial")
@Entity
public class VirtualMachineTemplate implements Serializable {

    // Warning "key" can't be an attribut --> this is a reserved attribut !

    /**
     * This is the virtualMachineTemplateId
     */
    private Integer id;

    private String name;

    // private VirtualMachine.Status status;

    private Integer ramSizeMB;

    private Integer nbCpus;

    private Collection<VolumeTemplate> volumeTemplates;

    private Collection<NetworkInterfaceTemplate> networkInterfaceTemplates;

    private Collection<CustomizationProperties> customizationProperties;

    private SystemTemplate systemTemplate;

    private Collection<VirtualMachine> virtualMachines;

    private Integer oVF_order;

    private Integer oVF_startDelay;

    private Integer oVF_stopDelay;

    public VirtualMachineTemplate() {
        // This contructor is required by JPA when using non-empty
        // contructor(s).

        // Let's initialize associations' attributs.
        this.volumeTemplates = new ArrayList<VolumeTemplate>();
        this.networkInterfaceTemplates = new ArrayList<NetworkInterfaceTemplate>();
        this.customizationProperties = new ArrayList<CustomizationProperties>();
        this.virtualMachines = new ArrayList<VirtualMachine>();
    }

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    public Integer getId() {
        return this.id;
    }

    public void setId(final Integer id) {
        this.id = id;
    }

    public String getName() {
        return this.name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    // @Enumerated(EnumType.STRING)
    // public VirtualMachine.Status getStatus() {
    // return this.status;
    // }
    //
    // public void setStatus(final VirtualMachine.Status status) {
    // this.status = status;
    // }

    public Integer getRamSizeMB() {
        return this.ramSizeMB;
    }

    public void setRamSizeMB(final Integer ramSizeMB) {
        this.ramSizeMB = ramSizeMB;
    }

    public Integer getNbCpus() {
        return this.nbCpus;
    }

    public void setNbCpus(final Integer nbCpus) {
        this.nbCpus = nbCpus;
    }

    @ManyToMany
    @JoinTable(name = "VIRTUALMACHINETEMPLATE_VOLUMETEMPLATE")
    /**
     * VirtualMachineTemplate est le OWNER, il faut passer par lui pour associer
     * et persister. (VirtualMachineTemplate is the OWNER, we must use it (instead of VolumeTemplate) in order to associate, and persist).
     */
    public Collection<VolumeTemplate> getVolumeTemplates() {
        return this.volumeTemplates;
    }

    public void setVolumeTemplates(final Collection<VolumeTemplate> volumeTemplates) {
        this.volumeTemplates = volumeTemplates;
    }

    @OneToMany(mappedBy = "virtualMachineTemplate", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
    public Collection<NetworkInterfaceTemplate> getNetworkInterfaceTemplates() {
        return this.networkInterfaceTemplates;
    }

    public void setNetworkInterfaceTemplates(final Collection<NetworkInterfaceTemplate> networkInterfaceTemplates) {
        this.networkInterfaceTemplates = networkInterfaceTemplates;
    }

    @OneToMany(mappedBy = "virtualMachineTemplate", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
    public Collection<CustomizationProperties> getCustomizationProperties() {
        return this.customizationProperties;
    }

    public void setCustomizationProperties(final Collection<CustomizationProperties> customizationProperties) {
        this.customizationProperties = customizationProperties;
    }

    @ManyToOne
    @JoinColumn(name = "SystemTemplate_id")
    public SystemTemplate getSystemTemplate() {
        return this.systemTemplate;
    }

    public void setSystemTemplate(final SystemTemplate systemTemplate) {
        this.systemTemplate = systemTemplate;
    }

    @OneToMany(mappedBy = "virtualMachineTemplate", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
    public Collection<VirtualMachine> getVirtualMachines() {
        return this.virtualMachines;
    }

    public void setVirtualMachines(final Collection<VirtualMachine> virtualMachines) {
        this.virtualMachines = virtualMachines;
    }

    public Integer getoVF_order() {
        return this.oVF_order;
    }

    public void setoVF_order(final Integer oVF_order) {
        this.oVF_order = oVF_order;
    }

    public Integer getoVF_startDelay() {
        return this.oVF_startDelay;
    }

    public void setoVF_startDelay(final Integer oVF_startDelay) {
        this.oVF_startDelay = oVF_startDelay;
    }

    public Integer getoVF_stopDelay() {
        return this.oVF_stopDelay;
    }

    public void setoVF_stopDelay(final Integer oVF_stopDelay) {
        this.oVF_stopDelay = oVF_stopDelay;
    }

    @Override
    public String toString() {
        String s = this.getClass().getSimpleName() + "=[id=" + this.getId() + ", name=" + this.getName() + ", ramSizeMB="
            + this.getRamSizeMB() + ", nbCpus=" + this.getNbCpus() + ", systemTemplate=" + this.getSystemTemplate()
            + ", oVF_order=" + this.getoVF_order() + ", oVF_startDelay=" + this.getoVF_startDelay() + ", oVF_stopDelay="
            + this.getoVF_stopDelay() + "]";
        return s;
    }
}
