/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: VolumeTemplate.java 220 2011-08-01 13:13:33Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.business;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;

@SuppressWarnings("serial")
@Entity
public class VolumeTemplate implements Serializable {

    // Warning "key" can't be an attribut --> this is a reserved attribut !

    /**
     * This is the volumeTemplateId from Sirocco's (technical) point of view.
     */
    private Integer id;

    /**
     * This is the volumeTemplateId inside the SystemTemplate point of view.
     * Unique.
     */
    private String name;

    /**
     * Storage size in MB/Mo. There is a wanted duplication of the attribut
     * capacityInMB: it appears both in VolumeTemplate, and in Volume.
     */
    private Long capacityInMB;

    /**
     * VolumeTemplate's system format. The format's value comes from the
     * associated VMImage, a part for .iso.
     */
    private String volumeFormat;

    private SystemTemplate systemTemplate;

    private Collection<VirtualMachineTemplate> virtualMachineTemplates;

    private Collection<Volume> volumes;

    private VMImage virtualMachineImage;

    private String fileOVFHRef;

    private String virtualDiskDescType_oVFFileRef;

    public VolumeTemplate() {
        // This contructor is required by JPA when using non-empty
        // contructor(s).

        // Let's initialize associations' attributs.
        this.virtualMachineTemplates = new ArrayList<VirtualMachineTemplate>();
        this.volumes = new ArrayList<Volume>();
    }

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    public Integer getId() {
        return this.id;
    }

    public void setId(final Integer id) {
        this.id = id;
    }

    @Column(unique = true)
    public String getName() {
        return this.name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    public Long getCapacityInMB() {
        return this.capacityInMB;
    }

    public void setCapacityInMB(final Long capacityInMB) {
        this.capacityInMB = capacityInMB;
    }

    public String getVolumeFormat() {
        return this.volumeFormat;
    }

    public void setVolumeFormat(final String volumeFormat) {
        this.volumeFormat = volumeFormat;
    }

    @ManyToOne
    @JoinColumn(name = "SystemTemplate_id")
    public SystemTemplate getSystemTemplate() {
        return this.systemTemplate;
    }

    public void setSystemTemplate(final SystemTemplate systemTemplate) {
        this.systemTemplate = systemTemplate;
    }

    @ManyToMany(mappedBy = "volumeTemplates")
    public Collection<VirtualMachineTemplate> getVirtualMachineTemplates() {
        return this.virtualMachineTemplates;
    }

    public void setVirtualMachineTemplates(final Collection<VirtualMachineTemplate> virtualMachineTemplates) {
        this.virtualMachineTemplates = virtualMachineTemplates;
    }

    @OneToMany(mappedBy = "volumeTemplate", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
    public Collection<Volume> getVolumes() {
        return this.volumes;
    }

    public void setVolumes(final Collection<Volume> volumes) {
        this.volumes = volumes;
    }

    @ManyToOne
    @JoinColumn(name = "VMImage_id")
    public VMImage getVirtualMachineImage() {
        return this.virtualMachineImage;
    }

    public void setVirtualMachineImage(final VMImage virtualMachineImage) {
        this.virtualMachineImage = virtualMachineImage;
    }

    /**
     * XXX fileOVFHRef is a temporary attribut. It will be deleted in the
     * future. The association amongst VolumeTemplate, and VMImage should then
     * be used.
     */
    public String getFileOVFHRef() {
        return this.fileOVFHRef;
    }

    public void setFileOVFHRef(final String fileOVFHRef) {
        this.fileOVFHRef = fileOVFHRef;
    }

    /**
     * XXX virtualDiskDescType_oVFFileRef is a temporary attribut (it
     * corresponds to the VMImage's name. It will be deleted in the future. The
     * association amongst VolumeTemplate, and VMImage should then be used.
     */
    public String getVirtualDiskDescType_oVFFileRef() {
        return this.virtualDiskDescType_oVFFileRef;
    }

    public void setVirtualDiskDescType_oVFFileRef(final String virtualDiskDescType_oVFFileRef) {
        this.virtualDiskDescType_oVFFileRef = virtualDiskDescType_oVFFileRef;
    }

    @Override
    public String toString() {
        String s = this.getClass().getSimpleName() + "=[id=" + this.getId() + ", name=" + this.getName() + ", capacityInMB="
            + this.getCapacityInMB() + ", volumeFormat=" + this.getVolumeFormat() + ", systemTemplate="
            + this.getSystemTemplate() + ", virtualMachineImage=" + this.getVirtualMachineImage() + ", fileOVFHRef="
            + this.getFileOVFHRef() + ", virtualDiskDescType_oVFFileRef=" + this.virtualDiskDescType_oVFFileRef + "]";
        return s;
    }
}
