/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: DbParameters.java 107 2011-07-04 17:04:24Z dangtran $
 *
 */

package org.ow2.sirocco.cloudmanager.common;

import java.util.logging.Logger;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.ow2.sirocco.cloudmanager.admin.exception.CloudAdminException;
import org.ow2.sirocco.cloudmanager.service.ICloudSettings;
import org.ow2.sirocco.cloudmanager.service.reservation.api.ReservationException;
import org.ow2.sirocco.cloudmanager.service.reservation.api.TimeUnitGranularityType;

public class DbParameters {

    /**
     * Singleton class used to initialize database settings of application
     */

    private static DbParameters instance;

    private static Logger logger = Logger.getLogger(DbParameters.class.getName());

    private ICloudSettings cloudSettings;

    private InitialContext context;

    private Object obj;

    public static DbParameters getInstance() {
        if (null == DbParameters.instance) {
            DbParameters.instance = new DbParameters();
        }
        return DbParameters.instance;
    }

    public String ADMIN_MAIL;

    public String VM_EXPIRED_MAIL1;

    public String VM_EXPIRED_MAIL2;

    public String SERVICE_NAME;

    public String SERVICE_URL;

    public String VM_CREATED;

    public String VM_ROOT_PWD;

    public String LDAP_SERVER_IP;

    public String LDAP_BASENAME;

    public String MAIL_HOST;

    public String MAIL_PORT;

    public String VNC_PROXY_URL;

    public String HOST_SSH_KEY_FILE;

    public int DEFAULT_DISK_QUOTA_IN_MB;

    public int DEFAULT_RAM_QUOTA_IN_MB;

    public int DEFAULT_CPU_QUOTA;

    public int DEFAULT_VM_QUOTA;

    public int PURGE_STATS_MONTH;

    public int STATS_COLLECT_INTERVAL_MS;

    public String TARGET_AUDIENCE;

    public boolean AUTO_CREATE_ACCOUNT;

    public boolean SEND_EMAIL_UPON_VM_CREATION;

    public boolean USE_LDAP_SERVER_FOR_AUTHENTIFICATION;

    public int VM_REFRESH_PERIOD_SECONDS;

    // #Reservation module
    // #TimeUnitGranularityType.HOUR, TimeUnitGranularityType.DAY,
    // TimeUnitGranularityType.WEEK
    public TimeUnitGranularityType TEMPORAL_GRANULARITY_TYPE;

    public int BOOKING_PERIOD_EXPRESSED_IN_TIME_UNIT;

    private DbParameters() {
        try {
            this.context = new InitialContext();
            this.obj = this.context.lookup("CloudSettingsBean");
            this.cloudSettings = (ICloudSettings) this.obj;

            this.ADMIN_MAIL = this.cloudSettings.getValue("ADMIN_MAIL");
            this.VM_EXPIRED_MAIL1 = this.cloudSettings.getValue("VM_EXPIRED_MAIL1");
            this.VM_EXPIRED_MAIL2 = this.cloudSettings.getValue("VM_EXPIRED_MAIL2");
            this.SERVICE_NAME = this.cloudSettings.getValue("SERVICE_NAME");
            this.SERVICE_URL = this.cloudSettings.getValue("SERVICE_URL");
            this.VM_CREATED = this.cloudSettings.getValue("VM_CREATED");
            this.VM_ROOT_PWD = this.cloudSettings.getValue("VM_ROOT_PWD");
            this.LDAP_SERVER_IP = this.cloudSettings.getValue("LDAP_SERVER_IP");
            this.LDAP_BASENAME = this.cloudSettings.getValue("LDAP_BASENAME");
            this.MAIL_HOST = this.cloudSettings.getValue("MAIL_HOST");
            this.MAIL_PORT = this.cloudSettings.getValue("MAIL_PORT");
            this.VNC_PROXY_URL = this.cloudSettings.getValue("VNC_PROXY_URL");
            this.HOST_SSH_KEY_FILE = this.cloudSettings.getValue("HOST_SSH_KEY_FILE");
            this.DEFAULT_DISK_QUOTA_IN_MB = Integer.parseInt(this.cloudSettings.getValue("DEFAULT_DISK_QUOTA_IN_MB"));
            this.DEFAULT_RAM_QUOTA_IN_MB = Integer.parseInt(this.cloudSettings.getValue("DEFAULT_RAM_QUOTA_IN_MB"));
            this.DEFAULT_CPU_QUOTA = Integer.parseInt(this.cloudSettings.getValue("DEFAULT_CPU_QUOTA"));
            this.DEFAULT_VM_QUOTA = Integer.parseInt(this.cloudSettings.getValue("DEFAULT_VM_QUOTA"));
            this.PURGE_STATS_MONTH = Integer.parseInt(this.cloudSettings.getValue("PURGE_STATS_MONTH"));
            this.STATS_COLLECT_INTERVAL_MS = Integer.parseInt(this.cloudSettings.getValue("STATS_COLLECT_INTERVAL_MS"));
            this.TARGET_AUDIENCE = this.cloudSettings.getValue("TARGET_AUDIENCE");
            if ("TimeUnitGranularityType.HOUR".equals(this.cloudSettings.getValue("TEMPORAL_GRANULARITY_TYPE"))) {
                this.TEMPORAL_GRANULARITY_TYPE = TimeUnitGranularityType.HOUR;
            } else if ("TimeUnitGranularityType.DAY".equals(this.cloudSettings.getValue("TEMPORAL_GRANULARITY_TYPE"))) {
                this.TEMPORAL_GRANULARITY_TYPE = TimeUnitGranularityType.DAY;
            } else if ("TimeUnitGranularityType.WEEK".equals(this.cloudSettings.getValue("TEMPORAL_GRANULARITY_TYPE"))) {
                this.TEMPORAL_GRANULARITY_TYPE = TimeUnitGranularityType.WEEK;
            } else {
                throw new ReservationException("The given Parameters.TEMPORAL_GRANULARITY_TYPE_KEY ("
                    + this.cloudSettings.getValue("TEMPORAL_GRANULARITY_TYPE") + ") is unknown.");
            }
            this.BOOKING_PERIOD_EXPRESSED_IN_TIME_UNIT = Integer.parseInt(this.cloudSettings
                .getValue("BOOKING_PERIOD_EXPRESSED_IN_TIME_UNIT"));
            this.AUTO_CREATE_ACCOUNT = Boolean.parseBoolean(this.cloudSettings.getValue("AUTO_CREATE_ACCOUNT", "false"));
            this.USE_LDAP_SERVER_FOR_AUTHENTIFICATION = Boolean.parseBoolean(this.cloudSettings.getValue(
                "USE_LDAP_SERVER_FOR_AUTHENTIFICATION", "false"));
            this.SEND_EMAIL_UPON_VM_CREATION = Boolean.parseBoolean(this.cloudSettings.getValue("SEND_EMAIL_UPON_VM_CREATION",
                "false"));
            this.VM_REFRESH_PERIOD_SECONDS = Integer.parseInt(this.cloudSettings.getValue("VM_REFRESH_PERIOD_SECONDS", "60"));

        } catch (CloudAdminException e) {
            DbParameters.logger.severe(e.getMessage());
            e.printStackTrace();
        } catch (NamingException e) {
            DbParameters.logger.severe(e.getMessage());
            e.printStackTrace();
        }
    }
}
