/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: CloudAdminBean.java 371 2011-09-30 14:21:35Z chou $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.io.Serializable;
import java.net.InetAddress;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.jms.JMSException;
import javax.jms.ObjectMessage;
import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicPublisher;
import javax.jms.TopicSession;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.Query;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.ow2.easybeans.osgi.annotation.OSGiResource;
import org.ow2.sirocco.cloudmanager.admin.event.DelVMImageEvent;
import org.ow2.sirocco.cloudmanager.admin.event.DelVirtualMachineEvent;
import org.ow2.sirocco.cloudmanager.admin.event.NewVMImageEvent;
import org.ow2.sirocco.cloudmanager.admin.event.NewVirtualMachineEvent;
import org.ow2.sirocco.cloudmanager.admin.event.VMImageUpdateEvent;
import org.ow2.sirocco.cloudmanager.admin.event.VirtualMachineStatusChangeEvent;
import org.ow2.sirocco.cloudmanager.admin.exception.CloudAdminException;
import org.ow2.sirocco.cloudmanager.admin.exception.DuplicateNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InsufficientResourceException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidArgumentException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidHostIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidPowerStateException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidProjectIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidProjectNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidUsernameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVMImageException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVMSizeException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVirtualMachineIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.NoRelevantResourcesPoolException;
import org.ow2.sirocco.cloudmanager.admin.exception.PermissionDeniedException;
import org.ow2.sirocco.cloudmanager.admin.exception.ResourceQuotaExceededException;
import org.ow2.sirocco.cloudmanager.admin.exception.VMImageInUseException;
import org.ow2.sirocco.cloudmanager.admin.vo.VMImageVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VMSizeVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineExtendedInfo;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineSpec;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VolumeVO;
import org.ow2.sirocco.cloudmanager.business.Event;
import org.ow2.sirocco.cloudmanager.business.Project;
import org.ow2.sirocco.cloudmanager.business.User;
import org.ow2.sirocco.cloudmanager.business.VMImage;
import org.ow2.sirocco.cloudmanager.business.VMSize;
import org.ow2.sirocco.cloudmanager.business.VirtualMachine;
import org.ow2.sirocco.cloudmanager.business.Volume;
import org.ow2.sirocco.cloudmanager.business.VolumeTemplate;
import org.ow2.sirocco.cloudmanager.clustermanager.api.AllocationMode;
import org.ow2.sirocco.cloudmanager.clustermanager.api.IClusterManager;
import org.ow2.sirocco.cloudmanager.clustermanager.api.PerfMetricInfo;
import org.ow2.sirocco.cloudmanager.clustermanager.api.VirtualMachineCreationSpec;
import org.ow2.sirocco.cloudmanager.common.DbParameters;
import org.ow2.sirocco.cloudmanager.service.reservation.api.IResourcesReservation;
import org.ow2.sirocco.cloudmanager.service.reservation.api.ReservationException;
import org.ow2.sirocco.cloudmanager.service.reservation.impl.ResourcesReservation;
import org.ow2.sirocco.cloudmanager.sshtunnel.api.ISSHTunnelManager;
import org.ow2.sirocco.cloudmanager.utils.PermissionChecker;
import org.ow2.sirocco.vmm.api.CustomizationSpec;
import org.ow2.sirocco.vmm.api.GuestInfo;
import org.ow2.sirocco.vmm.api.VMMException;
import org.ow2.sirocco.vmm.api.VirtualMachineMXBean;
import org.ow2.sirocco.vmm.api.VnicIPSettings;
import org.ow2.sirocco.vmm.api.VnicIPSettings.IpAssignmentMode;

@Stateless(name = "CloudAdminBean", mappedName = "CloudAdminBean")
@Remote(IRemoteCloudAdmin.class)
@Local(ICloudAdmin.class)
public class CloudAdminBean implements ICloudAdmin, IRemoteCloudAdmin {

    private static Logger log = Logger.getLogger(CloudAdminBean.class.getName());

    @PersistenceContext(unitName = "persistence-unit/main", type = PersistenceContextType.TRANSACTION)
    protected EntityManager em;

    @Resource(mappedName = "JTCF")
    private TopicConnectionFactory topicConnectionFactory;

    @Resource(mappedName = "CloudAdmin")
    private Topic cloudAdminTopic;

    @OSGiResource
    private BundleContext context;

    @OSGiResource
    private IClusterManager clusterManager;

    @EJB(mappedName = "UserManagerBean")
    private IUserManager userManager;

    @EJB(mappedName = "ProjectManagerBean")
    private IProjectManager projectManager;

    @EJB(mappedName = "EventManagerBean")
    private IEventManager eventManager;

    @EJB(mappedName = "ResourcesReservationBean")
    private IResourcesReservation resourcesReservationManager;

    @EJB(mappedName = "CloudAdminBean")
    private ICloudAdmin cloudAdmin;

    @EJB(mappedName = "VolumeManagerBean")
    private IVolumeManager volumeManager;

    @SuppressWarnings("unused")
    private ServiceReference sshTunnelManagerRef;

    private ISSHTunnelManager sshTunnelManager;

    public boolean checkIfVmNameIsCorrect(final String vmLabel) {
        if (vmLabel == null || vmLabel.equals("")) {
            return false;
        }
        Pattern pattern = Pattern.compile("^[a-zA-Z0-9_.-]*$");
        Matcher matcher = pattern.matcher(vmLabel);
        return matcher.find();
    }

    private boolean checkIfTemplateNameExistForProjectId(final String name, final String projectId) {
        @SuppressWarnings("unchecked")
        List<VMImage> list = this.em
            .createQuery("FROM VMImage v WHERE v.name=:name AND v.project.projectId=:projectId AND v.status<>'DESTROYED'")
            .setParameter("name", name).setParameter("projectId", projectId).getResultList();
        if (null == list || list.size() == 0) {
            return false; // don't exist
        }
        return true; // already exists
    }

    public boolean checkIfVMNameExistForProjectId(final String vmName, final String projectId) {
        @SuppressWarnings("unchecked")
        List<VirtualMachine> list = this.em
            .createQuery(
                "FROM VirtualMachine v WHERE v.name=:name AND v.project.projectId=:projectId AND v.status<>'DESTROYED'")
            .setParameter("name", vmName).setParameter("projectId", projectId).getResultList();
        if (null == list || list.size() == 0) {
            return false; // don't exist
        }
        return true; // already exists
    }

    @Override
    public String createVirtualMachineWithVolumes(final String projectId, final String userName,
        final VirtualMachineSpec vmSpec, final List<Integer> volumesId) throws InvalidUsernameException, InvalidNameException,
        InvalidVMImageException, ResourceQuotaExceededException, DuplicateNameException, InvalidVMSizeException,
        InvalidProjectIdException, PermissionDeniedException, ReservationException, CloudAdminException {

        if (volumesId == null) {
            throw new InvalidArgumentException("The given volumesId is: " + volumesId + ".");
        } else if (volumesId.isEmpty()) {
            throw new InvalidArgumentException("The given volumesId is: empty.");
        } else {
            // XXX here, only the main/first volume is considered.
            Integer idOfTheFirstVolume = volumesId.iterator().next();

            // Let's find the volume's volumeTemplate.
            Volume theFirstVolumeItself = null;
            theFirstVolumeItself = this.volumeManager.getVolumeById(idOfTheFirstVolume);
            if (theFirstVolumeItself == null) {
                throw new InvalidArgumentException("The given volume's id: " + idOfTheFirstVolume + " is unknown.");
            } else {
                // nothing to do.
            }
            VolumeTemplate volumeTemplateOfTheFirstVolume = theFirstVolumeItself.getVolumeTemplate();

            // VMImage vMImage = this.getVMImageByImageId(...);
            VMImage vMImageOfTheFirstVolume = volumeTemplateOfTheFirstVolume.getVirtualMachineImage();

            if (vMImageOfTheFirstVolume == null) {
                throw new InvalidVMImageException("The VMImage is null here, but it can't be.");
            } else {
                vmSpec.setImageId(vMImageOfTheFirstVolume.getId());
                // XXX hardcoded.
                boolean startVM = false;
                return this.cloudAdmin.createAndStartVirtualMachine(projectId, userName, vmSpec, startVM);
            }
        }
    }

    @Override
    public String createAndStartVirtualMachine(final String projectId, final String username, final VirtualMachineSpec vmSpec,
        final boolean startVM) throws InvalidUsernameException, InvalidNameException, InvalidVMImageException,
        ResourceQuotaExceededException, DuplicateNameException, InvalidVMSizeException, InvalidProjectIdException,
        PermissionDeniedException, ReservationException, InvalidProjectNameException, InvalidArgumentException,
        InsufficientResourceException, NoRelevantResourcesPoolException, InvalidVirtualMachineIdException {

        // createVirtualMachine --> create and start a VM

        CloudAdminBean.log.info("Creating Virtual Machine for user " + username + " vmSpec=" + vmSpec);

        User user = this.userManager.getUserByUsername(username);
        if (user == null) {
            throw new InvalidUsernameException("Unknown user " + username);
        }

        Project project = this.projectManager.getProjectByProjectId(projectId);
        if (project == null) {
            throw new InvalidProjectIdException("Unknown project " + projectId);
        }

        // check project authorization
        if (!new PermissionChecker().canCreateVM(user, project)) {
            throw new PermissionDeniedException("Permission denied for user " + username + " in the projectId " + projectId);
        }

        // check VM name validity
        if (!this.checkIfVmNameIsCorrect(vmSpec.getVmName())) {
            throw new InvalidNameException("Invalid VM name " + vmSpec.getVmName());
        }

        // check if VM name already taken
        if (this.checkIfVMNameExistForProjectId(vmSpec.getVmName(), projectId)) {
            throw new DuplicateNameException("VM name " + vmSpec.getVmName() + " already exists in this project !");
        }

        if (vmSpec.getSizeName() != null && !vmSpec.getSizeName().equals("")) {
            VMSize size = this.lookUpVMSizeByName(vmSpec.getSizeName());
            if (size == null) {
                throw new InvalidVMSizeException("size " + vmSpec.getSizeName() + " unknown");
            }
            vmSpec.setMemorySizeMB((int) size.getMemorySizeMB());
            vmSpec.setNumCPUs(size.getNumCPUs());
        }

        VMImage image = this.getVMImageByImageId(vmSpec.getImageId());
        if (image == null) {
            throw new InvalidVMImageException("Invalid image ID " + vmSpec.getImageId());
        }

        // the disk size passed as argument is ignored
        // the size of the disk is forced to the size of the image
        vmSpec.setDiskSizeMB(image.getDiskSizeMB());

        // quota validation
        if (!this
            .validateVirtualMachineCreation(project, vmSpec.getNumCPUs(), vmSpec.getMemorySizeMB(), vmSpec.getDiskSizeMB())) {
            throw new ResourceQuotaExceededException("Quota exceeded");
        }

        // create server in db with status BEING_CREATED
        int vmId = this.cloudAdmin.newVirtualMachine(project, user, vmSpec);

        VirtualMachine newServer = this.cloudAdmin.getVirtualMachineById(vmId);
        if (newServer == null) {
            throw new InvalidVirtualMachineIdException("Internal Error: The given vmId: " + vmId + " is unknown.");
        } else {
            // nothing to do.
        }

        VirtualMachineVO serverVo = VirtualMachineVO.from(newServer);

        // notify virtual server creation
        this.emitTopicMessage(new NewVirtualMachineEvent(serverVo), "projectId", projectId);

        // this.emitCommandQueueMessage(new CreateVMCommand(newServer.getId(),
        // vmSpec));

        String sshKey = newServer.getUser().getPublicKey();

        VirtualMachineCreationSpec spec = new VirtualMachineCreationSpec();
        spec.setName(vmSpec.getVmName() + "-" + newServer.getId());
        spec.setDiskCapacityMB(vmSpec.getDiskSizeMB());
        spec.setVmImageId(image.getProviderId());
        spec.setMemorySizeMB(vmSpec.getMemorySizeMB());
        spec.setNumVCPUs(vmSpec.getNumCPUs());

        CustomizationSpec customizationSpec = new CustomizationSpec();
        spec.setCustomizationSpec(customizationSpec);
        customizationSpec.setGuestOsHostName(vmSpec.getDNSName());

        Map<String, String> guestOSparams = vmSpec.getUserData();
        if (guestOSparams == null) {
            guestOSparams = new HashMap<String, String>();
        }

        if (sshKey != null && sshKey.length() > 1) {
            guestOSparams.put("sshkey", sshKey);
        }
        customizationSpec.setGuestOsCustomizationParams(guestOSparams);

        Map<String, String> networkParams = vmSpec.getNetworkParameters();
        if (networkParams != null && networkParams.get("ipAssignmentMode") != null) {
            if (networkParams.get("ipAssignmentMode").equals("FIXED")) {
                VnicIPSettings ipSettings = new VnicIPSettings();
                ipSettings.setIpAssignmentMode(IpAssignmentMode.FIXED);
                ipSettings.setIpAddress(networkParams.get("ip"));
                ipSettings.setSubnetMask(networkParams.get("mask"));
                ipSettings.setGateway(networkParams.get("gateway"));
                if (networkParams.get("domain") != null) {
                    customizationSpec.setGuestOsDomain(networkParams.get("domain"));
                }
                if (networkParams.get("dns1") != null) {
                    List<String> dnsServerList = new LinkedList<String>();
                    dnsServerList.add(networkParams.get("dns1"));
                    if (networkParams.get("dns2") != null) {
                        dnsServerList.add(networkParams.get("dns2"));
                    }
                    customizationSpec.setGuestOsDnsServerList(dnsServerList);
                }
                CloudAdminBean.log.log(Level.FINE, "Network config for VM " + vmSpec.getVmName() + " " + ipSettings);
                customizationSpec.setVnicIpSettingsList(Collections.singletonList(ipSettings));
            }
        }

        HashMap<String, String> constraints = new HashMap<String, String>();
        if (vmSpec.getLocation() != null && !vmSpec.getLocation().equals("") & !vmSpec.getLocation().equals("anywhere")) {
            constraints.put("location", vmSpec.getLocation());
        }
        constraints.put("hypervisor", image.getHypervisor());

        // Create the VM on the right resourcePool (i.e., reservation (or
        // not), hypervisor type, and location).
        if (vmSpec.getReservationId() == null) {
            // on-demand allocation

            // The hypervisor type, and location have already been retrieved
            // (and forwarded to HashMap<String, String> constraints).

            try {
                this.clusterManager.createVirtualMachine(spec, constraints, AllocationMode.ON_DEMAND, startVM);
            } catch (VMMException ex) {
                CloudAdminBean.log.log(Level.SEVERE, "VM provision failure", ex);
                // if creation fails set status to error and fire event
                this.updateVirtualMachineStatusById(newServer.getId(), VirtualMachine.Status.ERROR);
                this.emitTopicMessage(new VirtualMachineStatusChangeEvent(VirtualMachineVO.from(newServer)), "projectId",
                    newServer.getProject().getProjectId());
            }

        } else {
            // on-reservation allocation
            try {
                if (this.resourcesReservationManager.createAVMUsingReservedResources(vmSpec.getReservationId(), vmSpec)) {

                    // Let's retrieve the reservation location (the hypervisor
                    // type has already been retrieved, and forwarded to
                    // HashMap<String, String> constraints).
                    Query persistedResourcesReservation_list_query = this.em.createQuery(
                        "FROM " + ResourcesReservation.class.getSimpleName() + " p WHERE p.id=:reservationId").setParameter(
                        "reservationId", vmSpec.getReservationId());
                    if (persistedResourcesReservation_list_query == null) {
                        throw new InvalidArgumentException("The given reservationId (" + vmSpec.getReservationId()
                            + ") is unknown to Sirocco.");
                    } else {
                        @SuppressWarnings("unchecked")
                        List<ResourcesReservation> persistedResourcesReservation_list = persistedResourcesReservation_list_query
                            .getResultList();
                        if (persistedResourcesReservation_list.size() == 0) {
                            throw new InvalidArgumentException("The given reservationId (" + vmSpec.getReservationId()
                                + ") is unknown to Sirocco.");
                        } else {
                            constraints.put("location", persistedResourcesReservation_list.get(0).getLocation());
                        }
                    }

                    try {
                        this.clusterManager.createVirtualMachine(spec, constraints, AllocationMode.ADVANCED_RESERVATION,
                            startVM);
                    } catch (VMMException ex) {
                        CloudAdminBean.log.log(Level.SEVERE, "VM provision failure", ex);
                        // if creation fails set status to error and fire event
                        this.updateVirtualMachineStatusById(newServer.getId(), VirtualMachine.Status.ERROR);
                        this.emitTopicMessage(new VirtualMachineStatusChangeEvent(VirtualMachineVO.from(newServer)),
                            "projectId", newServer.getProject().getProjectId());

                        this.em.remove(newServer);
                        this.em.flush();

                        throw new ReservationException("Virtual machine creation failed.");
                    }

                } else {
                    this.em.remove(newServer);
                    this.em.flush();
                    throw new ReservationException(
                        "createAVMUsingReservedResources did not authorized this VM creation on the given reservation.");
                }
            } catch (ReservationException e) {
                this.em.remove(newServer);
                this.em.flush();
                throw e;
            } catch (InvalidUsernameException e) {
                this.em.remove(newServer);
                this.em.flush();
                throw e;
            } catch (InvalidNameException e) {
                this.em.remove(newServer);
                this.em.flush();
                throw e;
            } catch (InvalidVMImageException e) {
                this.em.remove(newServer);
                this.em.flush();
                throw e;
            } catch (ResourceQuotaExceededException e) {
                this.em.remove(newServer);
                this.em.flush();
                throw e;
            } catch (DuplicateNameException e) {
                this.em.remove(newServer);
                this.em.flush();
                throw e;
            } catch (InvalidVMSizeException e) {
                this.em.remove(newServer);
                this.em.flush();
                throw e;
            } catch (InvalidProjectNameException e) {
                this.em.remove(newServer);
                this.em.flush();
                throw e;
            }

        }
        return newServer.getId().toString();
    }

    public void createVMImage(final String projectId, final VMImage.Visibility visibility, final String vmId,
        final String name, final String description, final String fromUsername) throws CloudAdminException,
        InvalidVirtualMachineIdException, InvalidNameException, DuplicateNameException, InvalidProjectIdException {

        if (name.equals("")) {
            throw new InvalidNameException("Image name must be specified");
        }
        if (this.checkIfTemplateNameExistForProjectId(name, projectId)) {
            throw new DuplicateNameException("Image name " + name + " already used");
        }

        VirtualMachine vs = this.getVirtualMachineByVmId(vmId);
        if (vs == null) {
            throw new InvalidVirtualMachineIdException("vmId " + vmId + " unknown");
        }
        Project proj = this.projectManager.getProjectByProjectId(projectId);
        if (proj == null) {
            throw new InvalidProjectIdException("projectId" + projectId + " unknown");
        }

        VMImage image = new VMImage();

        image.setStatus(VMImage.Status.IN_CONSTRUCTION);
        image.setDescription(description);
        image.setName(name);
        image.setHypervisor(vs.getVmImage().getHypervisor());
        image.setUser(this.userManager.getUserByUsername(fromUsername));
        image.setProject(proj);
        image.setArchitecture(vs.getVmImage().getArchitecture());
        image.setDiskSizeMB(vs.getVmImage().getDiskSizeMB());
        image.setImageSizeMB(vs.getVmImage().getImageSizeMB());
        image.setOsType(vs.getVmImage().getOsType());
        image.setVisibility(visibility);

        image.setId(this.cloudAdmin.createVMImageRecord(image));

        try {
            this.clusterManager.createImageFromVirtualMachine(vs.getProviderId(), image.getId(), name, description);
        } catch (VMMException ex) {
            CloudAdminBean.log.log(Level.SEVERE, "Cannot create image", ex);
            this.em.remove(image);
            throw new CloudAdminException("Internal error");
        }

        vs.setStatus(VirtualMachine.Status.BEING_TEMPLATED);
        this.em.persist(vs);
        this.em.flush();

        this.emitTopicMessage(new VirtualMachineStatusChangeEvent(VirtualMachineVO.from(vs)), "projectId", vs.getProject()
            .getProjectId());
        this.emitTopicMessage(new NewVMImageEvent(VMImageVO.from(image)), "projectId", vs.getProject().getProjectId());
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
    public int createVMImageRecord(final VMImage image) {
        this.em.persist(image);
        this.em.flush();
        return image.getId();
    }

    public void destroyVirtualMachine(final String vmId, final String fromUsername) throws InvalidVirtualMachineIdException,
        PermissionDeniedException, InvalidPowerStateException, InvalidArgumentException, NoRelevantResourcesPoolException,
        ReservationException {

        VirtualMachineVO vmVo = this.cloudAdmin.destroyVirtualMachineRecord(vmId, fromUsername);

        this.emitTopicMessage(new VirtualMachineStatusChangeEvent(vmVo), "projectId", vmVo.getProjectId());

        // notify virtual server deletion
        this.emitTopicMessage(new DelVirtualMachineEvent(vmId, vmVo.getMachine()), "projectId", vmVo.getProjectId());
        this.eventManager.newEvent(Event.Level.INFO, "Virtual Machine " + vmVo.getName() + " deleted", vmVo.getProjectId(),
            vmVo.getId(), Event.ITypes.VM_DELETION, null);
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
    public VirtualMachineVO destroyVirtualMachineRecord(final String vmId, final String fromUsername)
        throws InvalidVirtualMachineIdException, PermissionDeniedException, InvalidPowerStateException,
        InvalidArgumentException, NoRelevantResourcesPoolException, ReservationException {
        VirtualMachineVO result = null;

        CloudAdminBean.log.info("vmId: " + vmId);
        @SuppressWarnings("unchecked")
        List<VirtualMachine> list = this.em.createQuery("FROM VirtualMachine v WHERE v.id=:vmId")
            .setParameter("vmId", Integer.parseInt(vmId)).getResultList();

        VirtualMachine vm = null;
        if (list.size() > 0) {
            vm = list.get(0);
        } else {
            throw new InvalidVirtualMachineIdException("vmId " + vmId + " unknown");
        }

        // check project authorization
        if (!new PermissionChecker().canDestroyVM(this.userManager.getUserByUsername(fromUsername), vm)) {
            throw new PermissionDeniedException("Permission denied!");
        }

        if (this.resourcesReservationManager.destroyAVMThatUsesReservedResources(vmId)) {
            // nothing to do. The VM corresponding to id has actually been
            // created on reserved resources.
        } else {
            // nothing to do. The VM corresponding to id has actually been
            // created on non reserved resources.
        }

        if (vm.getStatus() != VirtualMachine.Status.ERROR) {
            this.destroyVirtualMachineVNCSSHTunnel(vm);
            CloudAdminBean.log.info("DESTROYING VM with providerId=" + vm.getProviderId());
            try {
                this.clusterManager.destroyVirtualMachine(vm.getProviderId());
            } catch (VMMException ex) {
                CloudAdminBean.log.log(Level.SEVERE, "Error while destroying VM", ex);
            }
        }

        vm.setStatus(VirtualMachine.Status.DESTROYED);
        vm.setDeleteDate(new Date());

        // detach all volumes attached to the VM
        // vm.getVolumes().clear();
        List<Volume> volumeToBeDeleted = new ArrayList<Volume>();
        for (Volume v : vm.getVolumes()) {
            volumeToBeDeleted.add(v);
        }
        for (Volume v : volumeToBeDeleted) {
            vm.getVolumes().remove(v);
            v.getVirtualMachines().remove(v);
        }

        this.em.merge(vm);
        this.em.flush();

        result = VirtualMachineVO.from(vm);
        return result;
    }

    private void destroyVirtualMachineVNCSSHTunnel(final VirtualMachine vs) {
        String consoleDesc;
        try {
            consoleDesc = this.clusterManager.getVirtualMachineConsole(vs.getProviderId());
        } catch (VMMException ex) {
            CloudAdminBean.log.log(Level.SEVERE, "getVMConsole", ex);
            return;
        } catch (UnsupportedOperationException ex) {
            return;
        }
        if (consoleDesc != null && consoleDesc.length() > 0) {
            URI vncURI = null;
            try {
                vncURI = new URI(consoleDesc);
            } catch (URISyntaxException ex) {
                return;
            }
            String vncHost = vncURI.getHost();
            int vncPort = vncURI.getPort();
            try {
                this.sshTunnelManager.destroySSHTunnel(vncHost, vncPort);
            } catch (Exception ex) {
                CloudAdminBean.log.log(Level.SEVERE, "destroySSHTunnel", ex);
            }

        }
    }

    public void destroyVMImage(final Integer imageId, final String fromUsername) throws InvalidVMImageException,
        VMImageInUseException, PermissionDeniedException {

        VMImage vmi = (VMImage) this.em.createQuery("FROM VMImage v WHERE v.id=:vmiId").setParameter("vmiId", imageId)
            .getSingleResult();
        if (vmi == null) {
            throw new InvalidVMImageException("Invalid image id " + imageId);
        }

        // check authorization
        if (!new PermissionChecker().canDestroyVMImage(this.userManager.getUserByUsername(fromUsername), vmi)) {
            throw new PermissionDeniedException("Permission denied!");
        }

        Query query = this.em.createQuery("FROM VirtualMachine v WHERE v.vmImage.id=:vmiId AND v.status<>'DESTROYED'")
            .setParameter("vmiId", imageId);
        if (query.getResultList().size() > 0) {
            throw new VMImageInUseException("Image id " + imageId + " still in used");
        }
        String providerId = vmi.getProviderId();

        CloudAdminBean.log.info("Removing VM image Id=" + providerId);
        try {
            this.clusterManager.destroyVMImage(providerId);
        } catch (VMMException ex) {
            CloudAdminBean.log.log(Level.SEVERE, "Cannot destroy Image " + providerId, ex);
        }
        vmi.setStatus(VMImage.Status.DESTROYED);
        this.em.merge(vmi);
        this.emitTopicMessage(new DelVMImageEvent(imageId), "projectId", vmi.getProject().getProjectId());
    }

    public void emitTopicMessage(final Serializable obj, final String property, final String value) {
        try {
            TopicConnection connection = this.topicConnectionFactory.createTopicConnection();
            TopicSession session = connection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
            TopicPublisher topicPublisher = session.createPublisher(this.cloudAdminTopic);
            ObjectMessage message = session.createObjectMessage();
            message.setObject(obj);
            message.setStringProperty(property, value);
            topicPublisher.publish(message);
            CloudAdminBean.log.info("EMITTED EVENT " + obj + " on CloudAdmin topic");
            topicPublisher.close();
            session.close();
            connection.close();
        } catch (JMSException ex) {
            CloudAdminBean.log.severe(ex.getMessage());
        }
    }

    public List<VirtualMachineVO> getAllVirtualMachinesVo() {
        List<VirtualMachineVO> resultVo = new ArrayList<VirtualMachineVO>();
        @SuppressWarnings("unchecked")
        List<VirtualMachine> list = this.em.createQuery(
            "FROM VirtualMachine v WHERE v.status<>'DESTROYED' ORDER BY user.username,name").getResultList();
        for (VirtualMachine vs : list) {
            resultVo.add(VirtualMachineVO.from(vs));
        }
        return resultVo;
    }

    @Override
    public List<VolumeVO> getAllVolumeVo() {
        List<VolumeVO> resultVo = new ArrayList<VolumeVO>();
        @SuppressWarnings("unchecked")
        List<Volume> list = this.em.createQuery("FROM Volume v WHERE v.status<>'DESTROYED'").getResultList();
        for (Volume vol : list) {
            resultVo.add(VolumeVO.from(vol));
        }
        return resultVo;
    }

    public List<VMImageVO> getAllVMImagesVo() {
        @SuppressWarnings("unchecked")
        List<VMImage> list = this.em.createQuery("FROM VMImage v WHERE v.status<>'DESTROYED'").getResultList();
        List<VMImageVO> resultVo = new ArrayList<VMImageVO>();
        for (VMImage vmImage : list) {
            resultVo.add(VMImageVO.from(vmImage));
        }
        return resultVo;
    }

    public VirtualMachine getVirtualMachineById(final Integer id) {
        return this.em.find(VirtualMachine.class, id);
    }

    public VirtualMachine getVirtualMachineByProviderId(final String providerId) {
        @SuppressWarnings("unchecked")
        List<VirtualMachine> list = this.em
            .createQuery("FROM VirtualMachine v WHERE v.providerId=:providerId AND v.status<>'DESTROYED'")
            .setParameter("providerId", providerId).getResultList();
        Iterator<VirtualMachine> iter = list.iterator();
        VirtualMachine element = null;
        if (iter.hasNext()) {
            element = (iter.next());
        }
        return element;
    }

    public VirtualMachine getVirtualMachineByVmId(final String vmId) {
        return this.em.find(VirtualMachine.class, Integer.parseInt(vmId));
    }

    public String getVirtualMachineConsole(final String vmId) throws InvalidVirtualMachineIdException, CloudAdminException {
        if (this.getVirtualMachineVNCConsoleLocalPort(vmId) != -1) {
            return DbParameters.getInstance().VNC_PROXY_URL + "/vnc?id=" + vmId;
        }
        return null;
    }

    public VirtualMachineExtendedInfo getVirtualMachineExtendedInfo(final String vmId) throws InvalidVirtualMachineIdException {
        VirtualMachine vs = this.getVirtualMachineByVmId(vmId);
        if (vs == null) {
            throw new InvalidVirtualMachineIdException("vmId " + vmId + " unknown");
        }
        VirtualMachineExtendedInfo info = new VirtualMachineExtendedInfo();
        Map<String, Long> schedParams;
        schedParams = this.clusterManager.getVirtualMachineSchedulingParams(vs.getProviderId());
        Long cap = schedParams.get("cap");
        Long weight = schedParams.get("weight");
        if (cap != null) {
            info.setSchedulingCap(cap.intValue());
        }
        if (weight != null) {
            info.setSchedulingWeight(weight.intValue());
        }
        return info;
    }

    public float[] getVirtualMachinesCPULoad(final String[] vmIds) throws InvalidVirtualMachineIdException {
        float cpuLoads[] = new float[vmIds.length];
        for (int i = 0; i < vmIds.length; i++) {
            VirtualMachine vs = this.getVirtualMachineByVmId(vmIds[i]);
            if (vs == null) {
                throw new InvalidVirtualMachineIdException("id " + vmIds[i] + " unknown");
            }
            if (vs.getProviderId() != null) {
                cpuLoads[i] = this.clusterManager.getVirtualMachineCPULoad(vs.getProviderId());
            }
        }
        return cpuLoads;
    }

    public long[] getVirtualMachinesMemoryUsedMB(final String[] vmIds) throws InvalidVirtualMachineIdException {
        long memoryUsedMB[] = new long[vmIds.length];
        for (int i = 0; i < vmIds.length; i++) {
            VirtualMachine vs = this.getVirtualMachineByVmId(vmIds[i]);
            if (vs == null) {
                throw new InvalidVirtualMachineIdException("id " + vmIds[i] + " unknown");
            }
            if (vs.getProviderId() != null) {
                memoryUsedMB[i] = this.clusterManager.getVirtualMachineMemoryUsedMB(vs.getProviderId());
            }
        }
        return memoryUsedMB;
    }

    public List<VirtualMachineVO> getVirtualMachinesVoByHostId(final String hostId) {
        String uuids[] = this.clusterManager.getVirtualMachineRefByHost(hostId);
        List<VirtualMachineVO> result = new ArrayList<VirtualMachineVO>();
        for (String providerId : uuids) {
            VirtualMachine vs = this.getVirtualMachineByProviderId(providerId);
            if (vs != null) {
                result.add(VirtualMachineVO.from(vs));
            }
        }
        return result;
    }

    public int getVirtualMachineVNCConsoleLocalPort(final String vmId) throws InvalidVirtualMachineIdException {
        VirtualMachine vs = this.getVirtualMachineByVmId(vmId);
        if (vs == null) {
            throw new InvalidVirtualMachineIdException("id " + vmId + " unknown");
        }
        try {
            String consoleDesc = this.clusterManager.getVirtualMachineConsole(vs.getProviderId());
            if (consoleDesc != null && consoleDesc.length() > 0) {
                URI vncURI;
                try {
                    vncURI = new URI(consoleDesc);
                    String vncHost = vncURI.getHost();
                    int vncPort = vncURI.getPort();
                    CloudAdminBean.log.info("Getting SSH tunnel to host " + vncHost + " port=" + vncPort);
                    int localPort = this.sshTunnelManager.getSSHTunnel(vncHost, vncPort);
                    if (localPort == -1) {
                        try {
                            CloudAdminBean.log.info("Creating SSH tunnel to host " + vncHost + " port=" + vncPort);
                            String keyFile = DbParameters.getInstance().HOST_SSH_KEY_FILE;
                            localPort = this.sshTunnelManager.createSSHTunnel(vncHost, vncPort, "root", null, keyFile);
                        } catch (Exception ex) {
                            CloudAdminBean.log.log(Level.SEVERE, "Failed to create SSH tunnel", ex);
                            return -1;
                        }
                    }
                    CloudAdminBean.log.info("SSH tunnel found, forwarded local port=" + localPort);
                    return localPort;
                } catch (URISyntaxException ex) {
                    CloudAdminBean.log.log(Level.SEVERE, "Unable to parse VNC URL " + consoleDesc, ex);
                    return -1;
                }
            } else {
                return -1;
            }
        } catch (VMMException ex) {
            CloudAdminBean.log.log(Level.SEVERE, "Failed to get VM VNC console ", ex);
            return -1;
        } catch (UnsupportedOperationException ex) {
            return -1;
        }
    }

    public VirtualMachineVO getVirtualMachineVoByVmId(final String vmId) throws InvalidVirtualMachineIdException {
        @SuppressWarnings("unchecked")
        List<VirtualMachine> list = this.em.createQuery("FROM VirtualMachine v WHERE v.id=:vmId AND v.status<>'DESTROYED'")
            .setParameter("vmId", Integer.parseInt(vmId)).getResultList();

        if (list.size() > 0) {
            return VirtualMachineVO.from(list.get(0));
        } else {
            throw new InvalidVirtualMachineIdException("vmId " + vmId + " unknown");
        }
    }

    public VMImage getVMImageByImageId(final Integer vmiId) {
        return this.em.find(VMImage.class, vmiId);
    }

    public String getVMSizeName(final String vmId) throws InvalidVirtualMachineIdException {
        VirtualMachine vs = this.getVirtualMachineByVmId(vmId);
        if (vs == null) {
            throw new InvalidVirtualMachineIdException("vmId " + vmId + " unknown");
        }
        return this.getVMSizeName(vs);
    }

    public String getVMSizeName(final VirtualMachine vm) {
        List<VMSize> sizes = this.getVMSizes();
        for (VMSize size : sizes) {
            if (size.getNumCPUs() == vm.getNbCpus() && size.getMemorySizeMB() == vm.getRamSizeMB()) {
                return size.getName();
            }
        }
        return "custom";
    }

    private List<VMSize> getVMSizes() {
        @SuppressWarnings("unchecked")
        List<VMSize> list = this.em.createQuery("FROM VMSize").getResultList();
        return list;
    }

    public List<VMSizeVO> getVMSizesVo() {
        List<VMSize> list = this.getVMSizes();
        List<VMSizeVO> result = new ArrayList<VMSizeVO>();
        for (Object object : list) {
            result.add(VMSizeVO.from((VMSize) object));
        }
        return result;
    }

    public VMSize lookUpVMSizeByName(final String name) {
        @SuppressWarnings("unchecked")
        List<VMSize> list = this.em.createQuery("FROM VMSize s where s.name=:name").setParameter("name", name).getResultList();
        if (list != null & list.size() > 0) {
            return list.get(0);
        }
        return null;
    }

    public void migrateVirtualMachine(final String vmId, final String destinationHostId)
        throws InvalidVirtualMachineIdException, InvalidHostIdException, CloudAdminException {
        VirtualMachine vs = this.getVirtualMachineByVmId(vmId);
        if (vs == null) {
            throw new InvalidVirtualMachineIdException("vmId " + vmId + " unknown");
        }
        try {
            this.clusterManager.migrateVirtualMachine(vs.getProviderId(), destinationHostId);
        } catch (VMMException ex) {
            throw new CloudAdminException(ex.getMessage());
        }
    }

    public int newVirtualMachine(final Project project, final User user, final VirtualMachineSpec spec)
        throws InvalidVMImageException {
        VirtualMachine vm = new VirtualMachine();
        vm.setName(spec.getVmName());
        vm.setProject(project);
        vm.setUser(user);
        vm.setStatus(VirtualMachine.Status.IN_CONFIGURATION);
        vm.setDiskSizeMB(spec.getDiskSizeMB());
        vm.setNbCpus(spec.getNumCPUs());
        vm.setRamSizeMB(spec.getMemorySizeMB());
        vm.setLocation(spec.getLocation());
        vm.setExpirationDate(spec.getExpirationDate());
        vm.setAdditionalProperties(spec.getAdditionalProperties());
        VMImage vmImage = this.getVMImageByImageId(spec.getImageId());
        if (null != vmImage) {
            vm.setVmImage(vmImage);
        } else {
            throw new InvalidVMImageException("Virtual machine image " + spec.getImageId() + " unknown");
        }
        vm.setReservationId(spec.getReservationId());
        this.em.persist(vm);
        this.em.flush();
        return vm.getId();
    }

    public void pauseVirtualMachine(final String vmId, final String fromUsername) throws InvalidVirtualMachineIdException,
        InvalidPowerStateException, PermissionDeniedException {
        VirtualMachine vm = this.getVirtualMachineByVmId(vmId);
        if (vm == null) {
            throw new InvalidVirtualMachineIdException("vmId " + vmId + " unknown");
        }

        // check authorization
        if (!new PermissionChecker().canPauseVM(this.userManager.getUserByUsername(fromUsername), vm)) {
            throw new PermissionDeniedException("Permission denied!");
        }

        try {
            this.clusterManager.pauseVirtualMachine(vm.getProviderId());
        } catch (VMMException ex) {
            CloudAdminBean.log.log(Level.SEVERE, "Cannot pause VM " + vm.getProviderId(), ex);
        }
    }

    public void rebootVirtualMachine(final String vmId, final String fromUsername) throws InvalidVirtualMachineIdException,
        InvalidPowerStateException, PermissionDeniedException {

        VirtualMachine vm = this.getVirtualMachineByVmId(vmId);

        if (vm == null) {
            throw new InvalidVirtualMachineIdException("vmId " + vmId + " unknown");
        }

        // check authorization
        if (!new PermissionChecker().canRebootVM(this.userManager.getUserByUsername(fromUsername), vm)) {
            throw new PermissionDeniedException("Permission denied!");
        }

        try {
            this.clusterManager.rebootVirtualMachine(vm.getProviderId());
        } catch (VMMException ex) {
            CloudAdminBean.log.log(Level.SEVERE, "Cannot reboot VM " + vm.getProviderId(), ex);
        }
    }

    public void updateVirtualMachineExpirationDate(final String vmId, final Date expirationDate, final String fromUsername)
        throws InvalidVirtualMachineIdException, PermissionDeniedException {

        VirtualMachine vm = this.getVirtualMachineByVmId(vmId);

        if (vm == null) {
            throw new InvalidVirtualMachineIdException("vmId " + vmId + " unknown");
        }

        // check authorization
        if (!new PermissionChecker()
            .canUpdateVirtualMachineExpirationDate(this.userManager.getUserByUsername(fromUsername), vm)) {
            throw new PermissionDeniedException("Permission denied!");
        }

        vm.setExpirationDate(expirationDate);
        this.em.merge(vm);
        this.em.flush();

    }

    @PostConstruct
    public void start() {
        ServiceReference reference = this.context.getServiceReference(ISSHTunnelManager.class.getName());
        if (reference != null) {
            this.sshTunnelManager = (ISSHTunnelManager) this.context.getService(reference);
            this.sshTunnelManagerRef = reference;
        }
        CloudAdminBean.log.info("CloudAdminBean started");
    }

    public void startVirtualMachine(final String vmId) throws InvalidVirtualMachineIdException, InvalidPowerStateException {
        VirtualMachine vm = this.getVirtualMachineByVmId(vmId);
        if (vm == null) {
            throw new InvalidVirtualMachineIdException("vmId " + vmId + " unknown");
        }
        vm.setStatus(VirtualMachine.Status.STARTING);
        this.em.persist(vm);
        this.em.flush();
        try {
            this.clusterManager.startVirtualMachine(vm.getProviderId());
        } catch (VMMException ex) {
            CloudAdminBean.log.log(Level.SEVERE, "Cannot start VM " + vm.getProviderId(), ex);
        }
    }

    @PreDestroy
    public void stop() {
    }

    public void stopVirtualMachine(final String vmId, final String fromUsername) throws InvalidVirtualMachineIdException,
        InvalidPowerStateException, PermissionDeniedException {
        VirtualMachine vm = this.getVirtualMachineByVmId(vmId);
        if (vm == null) {
            throw new InvalidVirtualMachineIdException("vmId " + vmId + " unknown");
        }

        // check authorization
        if (!new PermissionChecker().canStopVm(this.userManager.getUserByUsername(fromUsername), vm)) {
            throw new PermissionDeniedException("Permission denied!");
        }

        vm.setStatus(VirtualMachine.Status.STOPPING);
        this.em.persist(vm);
        this.em.flush();

        this.emitTopicMessage(new VirtualMachineStatusChangeEvent(VirtualMachineVO.from(vm)), "projectId", vm.getProject()
            .getProjectId());
        try {
            this.clusterManager.stopVirtualMachine(vm.getProviderId());
        } catch (VMMException ex) {
            CloudAdminBean.log.log(Level.SEVERE, "Cannot stop VM " + vm.getProviderId(), ex);
        }
    }

    public void unpauseVirtualMachine(final String vmId) throws InvalidVirtualMachineIdException, InvalidPowerStateException {
        VirtualMachine vs = this.getVirtualMachineByVmId(vmId);
        if (vs == null) {
            throw new InvalidVirtualMachineIdException("vmId " + vmId + " unknown");
        }
        try {
            this.clusterManager.unpauseVirtualMachine(vs.getProviderId());
        } catch (VMMException ex) {
            CloudAdminBean.log.log(Level.SEVERE, "Cannot unpause VM " + vs.getProviderId(), ex);
        }
    }

    public void updateVirtualMachineStatusById(final int vmId, final VirtualMachine.Status status) {
        VirtualMachine vs = this.getVirtualMachineById(vmId);
        vs.setStatus(status);
        this.em.persist(vs);
        this.em.flush();
    }

    public void updateVMImage(final VMImageVO from) throws InvalidVMImageException {
        @SuppressWarnings("unchecked")
        List<VMImage> list = this.em.createQuery("FROM VMImage i WHERE i.id=:vmiId").setParameter("vmiId", from.getId())
            .getResultList();
        if (list.size() == 0) {
            throw new InvalidVMImageException("VM image not found");
        }
        VMImage image = list.get(0);
        image.setName(from.getName());
        image.setDescription(from.getDescription());
        image.setOsType(from.getOsType());
        this.em.merge(image);
        this.em.persist(image);
        this.em.flush();
        this.emitTopicMessage(new VMImageUpdateEvent(VMImageVO.from(image)), "projectId", image.getProject().getProjectId());
    }

    public boolean validateVirtualMachineCreation(final Project project, final int numCPUs, final int memorySizeMB,
        final int diskSizeMB) {
        int cpuCoreUsed = this.projectManager.countCPUsByProject(project.getProjectId()) + numCPUs;
        int memoryUsedInMB = this.projectManager.countRAMByProject(project.getProjectId()) + memorySizeMB;
        int diskUsedInMB = this.projectManager.countDiskByProject(project.getProjectId()) + diskSizeMB;
        int vmUsed = this.projectManager.countVMByProject(project.getProjectId()) + 1;

        if (project.getResourceQuota().getCpuQuota() != -1 && cpuCoreUsed > project.getResourceQuota().getCpuQuota()) {
            return false;
        }
        if (project.getResourceQuota().getRamQuotaInMB() != -1 && memoryUsedInMB > project.getResourceQuota().getRamQuotaInMB()) {
            return false;
        }
        if (project.getResourceQuota().getDiskQuotaInMB() != -1 && diskUsedInMB > project.getResourceQuota().getDiskQuotaInMB()) {
            return false;
        }
        if (project.getResourceQuota().getVmQuota() != -1 && vmUsed > project.getResourceQuota().getVmQuota()) {
            return false;
        }
        return true;
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
    @Override
    public boolean doesThisVmExist(final String vmId) {
        boolean result;
        VirtualMachine vm = this.em.find(VirtualMachine.class, Integer.parseInt(vmId));
        if (null == vm) {
            result = false;
        } else {
            result = true;
        }
        return result;
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
    @Override
    public VirtualMachineVO onVirtualMachineCreatedSecondPart(final String vmId, final String vmName, final String vmProviderId) {
        VirtualMachine vm = this.em.find(VirtualMachine.class, Integer.parseInt(vmId));

        VirtualMachineMXBean vmBean = this.clusterManager.getVirtualMachineMBean(vmProviderId);
        try {
            vm.setProviderId(vmProviderId);
            vm.setMac(vmBean.getMacAddress());
            GuestInfo guestInfo = vmBean.getGuestInfo();
            if (guestInfo != null && guestInfo.getIpAddresses() != null && guestInfo.getIpAddresses().size() > 0) {
                String vmIpAddress = guestInfo.getIpAddresses().get(0);
                vm.setIp(vmIpAddress);
                InetAddress addr = null;
                try {
                    addr = InetAddress.getByName(vmIpAddress);
                } catch (UnknownHostException e) {
                }
                if (addr != null) {
                    vm.setDnsName(addr.getHostName());
                }
            }
            vm.setMachine(vmBean.getHostMBean().getObjectName().toString());
            CloudAdminBean.log.info("VM " + vmName + " ready, IP=" + vm.getIp() + " hostname=" + vm.getDnsName());
        } catch (VMMException ex) {
            CloudAdminBean.log.log(Level.SEVERE, ex.getMessage(), ex);
        }

        vm.setCreateDate(new Date());
        vm.setStatus(VirtualMachine.Status.STOPPED);

        try {
            this.em.merge(vm);
            this.em.flush();
        } catch (Exception ex) {
            CloudAdminBean.log.log(Level.SEVERE, ex.getMessage(), ex);
        }
        return VirtualMachineVO.from(vm);
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
    @Override
    public VirtualMachineVO onVirtualMachineGuestOSReadySecondPart(final String vmId, final String vmName,
        final String vmProviderId) {
        VirtualMachine vm = this.em.find(VirtualMachine.class, Integer.parseInt(vmId));

        VirtualMachineMXBean vmBean = this.clusterManager.getVirtualMachineMBean(vmProviderId);
        try {
            vm.setProviderId(vmProviderId);
            vm.setMac(vmBean.getMacAddress());
            GuestInfo guestInfo = vmBean.getGuestInfo();
            if (guestInfo != null && guestInfo.getIpAddresses() != null && guestInfo.getIpAddresses().size() > 0) {
                String vmIpAddress = guestInfo.getIpAddresses().get(0);
                vm.setIp(vmIpAddress);
                InetAddress addr = null;
                try {
                    addr = InetAddress.getByName(vmIpAddress);
                } catch (UnknownHostException e) {
                }
                if (addr != null) {
                    vm.setDnsName(addr.getHostName());
                }
            }
            vm.setMachine(vmBean.getHostMBean().getObjectName().toString());
            CloudAdminBean.log.info("VM " + vmName + " ready, IP=" + vm.getIp() + " hostname=" + vm.getDnsName());

        } catch (VMMException ex) {
            CloudAdminBean.log.log(Level.SEVERE, ex.getMessage(), ex);
        }

        vm.setCreateDate(new Date());
        vm.setStatus(VirtualMachine.Status.RUNNING);

        try {
            this.em.merge(vm);
            this.em.flush();
        } catch (Exception ex) {
            CloudAdminBean.log.log(Level.SEVERE, ex.getMessage(), ex);
        }
        return VirtualMachineVO.from(vm);
    }

    @Override
    public void setPerfMetricInfosToVM(final String vmId, final PerfMetricInfo[] perfMetricInfos) {
        VirtualMachine vm = this.em.find(VirtualMachine.class, Integer.parseInt(vmId));
        if (vm != null) {
            Collection<org.ow2.sirocco.cloudmanager.service.monitoring.impl.PerfMetricInfo> result = new HashSet<org.ow2.sirocco.cloudmanager.service.monitoring.impl.PerfMetricInfo>();
            for (PerfMetricInfo pmi : perfMetricInfos) {
                org.ow2.sirocco.cloudmanager.service.monitoring.impl.PerfMetricInfo pm = new org.ow2.sirocco.cloudmanager.service.monitoring.impl.PerfMetricInfo();
                pm.setName(pmi.getName());
                pm.setDescription(pmi.getDescription());
                pm.setIntervalSample(pmi.getInterval());
                pm.setTypeSample(pmi.getType());
                pm.setUnit(pmi.getUnit().value());
                result.add(pm);
            }
            vm.setPerfMetricInfos(result);
            this.em.merge(vm);
        }
    }
}
