/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: CloudAdminMDB.java 232 2011-08-05 09:22:44Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.ejb.ActivationConfigProperty;
import javax.ejb.MessageDriven;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.Query;

import org.ow2.sirocco.cloudmanager.admin.event.DelVirtualMachineEvent;
import org.ow2.sirocco.cloudmanager.admin.event.VirtualMachineStatusChangeEvent;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineVO;
import org.ow2.sirocco.cloudmanager.business.SystemInstance;
import org.ow2.sirocco.cloudmanager.business.SystemInstance.SystemInstanceStatus;
import org.ow2.sirocco.cloudmanager.business.VirtualMachine;

@MessageDriven(activationConfig = {
    @ActivationConfigProperty(propertyName = "destinationType", propertyValue = "javax.jms.Topic"),
    @ActivationConfigProperty(propertyName = "destination", propertyValue = "CloudAdmin")})
public class CloudAdminMDB implements MessageListener {

    private Logger log = Logger.getLogger(CloudAdminMDB.class.getName());

    @PersistenceContext(unitName = "persistence-unit/main", type = PersistenceContextType.TRANSACTION)
    protected EntityManager em;

    public void onMessage(final Message msg) {
        if (msg instanceof ObjectMessage) {
            ObjectMessage objectMessage = (ObjectMessage) msg;
            try {
                Object payload = objectMessage.getObject();
                this.log.info("RECEIVED CloudAdmin TOPIC EVENT WITH OBJ=" + payload + " user="
                    + msg.getStringProperty("username"));
                if (payload instanceof VirtualMachineStatusChangeEvent) {
                    VirtualMachineStatusChangeEvent event = (VirtualMachineStatusChangeEvent) payload;
                    this.onVirtualMachineStatusChangeEvent(event.getServer());
                } else if (payload instanceof DelVirtualMachineEvent) {
                    DelVirtualMachineEvent event = (DelVirtualMachineEvent) payload;
                    this.onDelVirtualMachineEvent(event);
                } else {
                    // nothing to do.
                }
            } catch (JMSException ex) {
                this.log.log(Level.SEVERE, "Failed to extract from JMS message", ex);
                return;
            }
        }
    }

    private void onVirtualMachineStatusChangeEvent(final VirtualMachineVO vMVO) {
        VirtualMachine vM = null;

        Query vM_list_query = this.em.createQuery(
            "FROM " + VirtualMachine.class.getSimpleName() + " v WHERE v.providerId=:providerId").setParameter("providerId",
            vMVO.getProviderId());
        if (vM_list_query == null) {
            // nothing to do.
        } else {
            if (vM_list_query.getResultList().isEmpty()) {
                // nothing to do.
            } else {
                vM = (VirtualMachine) vM_list_query.getResultList().iterator().next();
            }
        }
        if (vM == null) {
            // nothing to do.
        } else {
            // Check if all the VM that belong to the vM's SystemInstance
            // are in the STOPPED state (apart vM itself).
            SystemInstance sI = vM.getSystemInstance();
            if (sI == null) {
                // nothing to do.
            } else {
                if (SystemInstanceStatus.CREATING.equals(sI.getStatus())) {
                    Iterator<VirtualMachine> vm_temp_ITERATOR = sI.getVirtualMachines().iterator();
                    boolean trueIfAllTheOtherVMAreInTheStoppedState = true;
                    while ((trueIfAllTheOtherVMAreInTheStoppedState) && (vm_temp_ITERATOR.hasNext())) {
                        VirtualMachine vm_temp = vm_temp_ITERATOR.next();
                        if (vM.getId().equals(vm_temp.getId())) {
                            // nothing to do.
                        } else {
                            // Ideally, the following lines should be used.
                            // if (VirtualMachine.Status.STOPPED.
                            // equals(vm_temp.getStatus())) {
                            // // nothing to do.
                            // } else { trueIfAllTheOtherVMAreInTheStoppedState
                            // =
                            // false; }
                            if (VirtualMachine.Status.IN_CONFIGURATION.equals(vm_temp.getStatus())) {
                                trueIfAllTheOtherVMAreInTheStoppedState = false;
                            } else {
                                // nothing to do.
                            }
                        }
                    }
                    if (trueIfAllTheOtherVMAreInTheStoppedState) {
                        sI.setStatus(SystemInstanceStatus.CREATED);
                        this.em.persist(sI);
                        this.em.flush();
                    } else {
                        // nothing to do.
                    }
                } else if (SystemInstanceStatus.STARTING.equals(sI.getStatus())) {
                    Iterator<VirtualMachine> vm_temp_ITERATOR = sI.getVirtualMachines().iterator();
                    boolean trueIfAllTheOtherVMAreInTheRunningState = true;
                    while ((trueIfAllTheOtherVMAreInTheRunningState) && (vm_temp_ITERATOR.hasNext())) {
                        VirtualMachine vm_temp = vm_temp_ITERATOR.next();
                        if (vM.getId().equals(vm_temp.getId())) {
                            // nothing to do.
                        } else {
                            // Ideally, the following lines should be used.
                            // if (VirtualMachine.Status.STOPPED.
                            // equals(vm_temp.getStatus())) {
                            // // nothing to do.
                            // } else { trueIfAllTheOtherVMAreInTheStoppedState
                            // =
                            // false; }
                            if (VirtualMachine.Status.RUNNING.equals(vm_temp.getStatus())) {
                                // nothing to do.
                            } else {
                                trueIfAllTheOtherVMAreInTheRunningState = false;
                            }
                        }
                    }
                    if (trueIfAllTheOtherVMAreInTheRunningState) {
                        sI.setStatus(SystemInstanceStatus.STARTED);
                        this.em.persist(sI);
                        this.em.flush();
                    } else {
                        // nothing to do.
                    }
                } else if (SystemInstanceStatus.STOPPING.equals(sI.getStatus())) {
                    Iterator<VirtualMachine> vm_temp_ITERATOR = sI.getVirtualMachines().iterator();
                    boolean trueIfAllTheOtherVMAreInTheStoppedState = true;
                    while ((trueIfAllTheOtherVMAreInTheStoppedState) && (vm_temp_ITERATOR.hasNext())) {
                        VirtualMachine vm_temp = vm_temp_ITERATOR.next();
                        if (vM.getId().equals(vm_temp.getId())) {
                            // nothing to do.
                        } else {
                            if (VirtualMachine.Status.STOPPED.equals(vm_temp.getStatus())) {
                                // nothing to do.
                            } else {
                                trueIfAllTheOtherVMAreInTheStoppedState = false;
                            }
                        }
                    }
                    if (trueIfAllTheOtherVMAreInTheStoppedState) {
                        sI.setStatus(SystemInstanceStatus.STOPPED);
                        this.em.persist(sI);
                        this.em.flush();
                    } else {
                        // nothing to do.
                    }
                } else {
                    // nothing to do.
                }
            }
        }
    }

    private void onDelVirtualMachineEvent(final DelVirtualMachineEvent event) {
        String vmId = event.getVmId();

        VirtualMachine vm = this.em.find(VirtualMachine.class, Integer.parseInt(vmId));
        SystemInstance si = vm.getSystemInstance();

        if (si == null) {
            // nothing to do.
        } else {
            SystemInstanceStatus siStatus = si.getStatus();
            if (SystemInstanceStatus.DESTROYING.equals(siStatus)) {
                Iterator<VirtualMachine> vmTemp_ITERATOR = si.getVirtualMachines().iterator();
                boolean trueIfAllTheOtherVMAreInTheDestroyedState = true;
                while ((trueIfAllTheOtherVMAreInTheDestroyedState) && (vmTemp_ITERATOR.hasNext())) {
                    VirtualMachine vmTemp = vmTemp_ITERATOR.next();
                    if (vm.getId().equals(vmTemp.getId())) {
                        // nothing to do.
                    } else {
                        if (VirtualMachine.Status.DESTROYED.equals(vmTemp.getStatus())) {
                            // nothing to do.
                        } else {
                            trueIfAllTheOtherVMAreInTheDestroyedState = false;
                        }
                    }
                }
                if (trueIfAllTheOtherVMAreInTheDestroyedState) {
                    si.setStatus(SystemInstanceStatus.DESTROYED);
                    this.em.merge(si);
                    this.em.flush();
                } else {
                    // nothing to do.
                }
            } else {
                // nothing to do.
            }
        }
    }
}
