/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: ClusterManagerProxyBean.java 220 2011-08-01 13:13:33Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.util.Date;
import java.util.List;

import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;

import org.ow2.easybeans.osgi.annotation.OSGiResource;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidHostIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidPerfMetricIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVirtualMachineIdException;
import org.ow2.sirocco.cloudmanager.business.VirtualMachine;
import org.ow2.sirocco.cloudmanager.clustermanager.api.Domain;
import org.ow2.sirocco.cloudmanager.clustermanager.api.IClusterManager;
import org.ow2.sirocco.cloudmanager.clustermanager.api.PerfMetricBase;
import org.ow2.sirocco.cloudmanager.clustermanager.api.PerfMetricInfo;
import org.ow2.sirocco.vmm.api.VMMException;

@Stateless(name = "ClusterManagerProxyBean", mappedName = "ClusterManagerProxyBean")
@Remote(IRemoteClusterManagerProxy.class)
@Local(IClusterManagerProxy.class)
public class ClusterManagerProxyBean implements IClusterManagerProxy {

    // private static Logger log =
    // Logger.getLogger(ClusterManagerProxyBean.class.getName());

    @OSGiResource
    private IClusterManager clusterManager;

    @PersistenceContext(unitName = "persistence-unit/main", type = PersistenceContextType.TRANSACTION)
    protected EntityManager em;

    public VirtualMachine getVirtualMachineByVmId(final String vmId) {
        @SuppressWarnings("unchecked")
        List<VirtualMachine> list = this.em.createQuery("FROM VirtualMachine v WHERE v.id=:vmId AND v.status<>'DESTROYED'")
            .setParameter("vmId", Integer.parseInt(vmId)).getResultList();
        if (list.size() > 0) {
            return list.get(0);
        } else {
            return null;
        }
    }

    public List<String> getDataCenterLocations() {
        return this.clusterManager.getLocations();
    }

    public Domain getDataCenterResourceTree() {
        return this.clusterManager.getResourceTree();
    }

    public PerfMetricInfo[] getHostPerfMetricInfos() {
        return this.clusterManager.getHostPerfMetricInfos();
    }

    public PerfMetricBase[] getHostPerfMetrics(final String hostId, final String metricId, final Date startTime,
        final Date endTime) throws InvalidHostIdException, InvalidPerfMetricIdException {
        try {
            return this.clusterManager.getHostPerfMetrics(hostId, metricId, startTime, endTime);
        } catch (VMMException ex) {
            if (ex.getMessage().equals("InvalidPerfMetricIdException")) {
                throw new InvalidPerfMetricIdException();
            }
            if (ex.getMessage().equals("InvalidHostIdException")) {
                throw new InvalidHostIdException();
            }
            return null;
        }
    }

    public PerfMetricInfo[] getVirtualMachinePerfMetricInfos() {
        return this.clusterManager.getVirtualMachinePerfMetricInfos();
    }

    public PerfMetricBase[] getVirtualMachinePerfMetrics(final String vmId, final String metricId, final Date startTime,
        final Date endTime) throws InvalidVirtualMachineIdException, InvalidPerfMetricIdException {
        VirtualMachine vs = this.getVirtualMachineByVmId(vmId);
        if (vs == null) {
            throw new InvalidVirtualMachineIdException("vmId " + vmId + " unknown");
        }
        try {
            return this.clusterManager.getVirtualMachinePerfMetrics(vs.getName(), metricId, startTime, endTime);
        } catch (VMMException ex) {
            throw new InvalidPerfMetricIdException();
        }
    }

}
