/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: DestroyExpiredVMTimerSessionBean.java 220 2011-08-01 13:13:33Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import javax.annotation.Resource;
import javax.ejb.Local;
import javax.ejb.Stateless;
import javax.ejb.Timeout;
import javax.ejb.Timer;
import javax.ejb.TimerService;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.Query;

import org.ow2.easybeans.osgi.annotation.OSGiResource;
import org.ow2.sirocco.cloudmanager.business.VirtualMachine;
import org.ow2.sirocco.cloudmanager.clustermanager.api.IClusterManager;
import org.ow2.sirocco.cloudmanager.common.DbParameters;
import org.ow2.sirocco.cloudmanager.utils.VirtUtils;
import org.ow2.sirocco.vmm.api.VirtualMachineMXBean;

@Stateless(name = "DestroyExpiredVMTimerSessionBean", mappedName = "DestroyExpiredVMTimerSessionBean")
@Local(IDestroyExpiredVMTimer.class)
public class DestroyExpiredVMTimerSessionBean implements IDestroyExpiredVMTimer {

    @Resource
    private TimerService timerService;

    @PersistenceContext(type = PersistenceContextType.TRANSACTION)
    protected EntityManager em;

    @OSGiResource
    private IClusterManager clusterManager;

    private Logger log = Logger.getLogger(DestroyExpiredVMTimerSessionBean.class.getName());

    public void startTimer() {
        this.timerService.createTimer(10000, 86400000, null);
        this.log.info("DestroyExpiredVMTimerSessionBean set");
    }

    @Timeout
    public void handleTimeout(final Timer timer) {

        // query database for expired VM
        Query query = this.em.createNativeQuery(
            "SELECT * FROM VirtualMachine v WHERE v.expirationDate < CURRENT_DATE() AND v.status<>'DESTROYED'",
            VirtualMachine.class);

        @SuppressWarnings("unchecked")
        List<VirtualMachine> result = query.getResultList();
        if (result.size() > 0) {
            for (Object obj : result) {
                VirtualMachine vs = (VirtualMachine) obj;
                @SuppressWarnings("unused")
                VirtualMachineMXBean vmMBean = null;
                try {
                    vmMBean = this.clusterManager.getVirtualMachineMBean(vs.getProviderId());
                    String to = vs.getUser().getEmail();

                    // used to set $ variables in DbParameters mail
                    final Map<String, String> varMap = new HashMap<String, String>();
                    varMap.put("vm", vs.getName());
                    varMap.put("url", DbParameters.getInstance().SERVICE_URL);
                    varMap.put("user", vs.getUser().getEmail());

                    VirtUtils.sendMail(DbParameters.getInstance().ADMIN_MAIL, to, DbParameters.getInstance().SERVICE_NAME,
                        VirtUtils.replace(DbParameters.getInstance().VM_EXPIRED_MAIL1, varMap));
                    VirtUtils.sendMail(DbParameters.getInstance().ADMIN_MAIL, DbParameters.getInstance().ADMIN_MAIL,
                        DbParameters.getInstance().SERVICE_NAME,
                        VirtUtils.replace(DbParameters.getInstance().VM_EXPIRED_MAIL2, varMap));
                } catch (Exception ex) {
                    this.log.info("Error while sending mail");
                }
            }
        }
    }

    public void checkStatus() {
        @SuppressWarnings("unchecked")
        Collection<Timer> timers = this.timerService.getTimers();
        for (Timer timer : timers) {
            this.log.info("Timer will expire after " + timer.getTimeRemaining() + " milliseconds.\n");
        }
    }

    public void stopTimers() {
        @SuppressWarnings("unchecked")
        Collection<Timer> timers = this.timerService.getTimers();
        for (Timer timer : timers) {
            timer.cancel();
        }
        this.log.info("DestroyExpiredVMTimerSessionBean stopped");
    }
}
