/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: EventManagerBean.java 214 2011-07-28 08:58:50Z otterley $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;

import org.ow2.sirocco.cloudmanager.admin.vo.EventVO;
import org.ow2.sirocco.cloudmanager.business.Event;

@Stateless(mappedName = "EventManagerBean")
@Remote(IRemoteEventManager.class)
@Local(IEventManager.class)
public class EventManagerBean implements IEventManager {

    @PersistenceContext(type = PersistenceContextType.TRANSACTION)
    private EntityManager em;

    public List<EventVO> findEventsVo(Date startTime, Date endTime, final int pageNumber, final int pageSize) {
        if (startTime == null) {
            startTime = new Date(0);
        }
        if (endTime == null) {
            endTime = new Date();
        }
        @SuppressWarnings("unchecked")
        List<Event> list = this.em.createNamedQuery("findEvents").setParameter("startTime", startTime)
            .setParameter("endTime", endTime).setMaxResults(pageSize).setFirstResult(pageNumber * pageSize).getResultList();

        List<EventVO> result = new ArrayList<EventVO>();
        for (Event e : list) {
            result.add(EventVO.from(e));
        }
        return result;
    }

    public int countEvents(Date startTime, Date endTime) {
        if (startTime == null) {
            startTime = new Date(0);
        }
        if (endTime == null) {
            endTime = new Date();
        }
        return ((Long) this.em.createNamedQuery("countEvents").setParameter("startTime", startTime)
            .setParameter("endTime", endTime).getSingleResult()).intValue();
    }

    public List<EventVO> findEventsVoByProject(final String projectId, Date startTime, Date endTime, final int pageNumber,
        final int pageSize) {
        if (startTime == null) {
            startTime = new Date(0);
        }
        if (endTime == null) {
            endTime = new Date();
        }
        @SuppressWarnings("unchecked")
        List<Event> list = this.em.createNamedQuery("findEventsVoByProject").setParameter("projectId", projectId)
            .setParameter("startTime", startTime).setParameter("endTime", endTime).setMaxResults(pageSize)
            .setFirstResult(pageNumber * pageSize).getResultList();

        List<EventVO> result = new ArrayList<EventVO>();
        for (Event e : list) {
            result.add(EventVO.from(e));
        }
        return result;
    }

    public int countEventsByProject(final String projectId, Date startTime, Date endTime) {
        if (startTime == null) {
            startTime = new Date(0);
        }
        if (endTime == null) {
            endTime = new Date();
        }
        return ((Long) this.em.createNamedQuery("countEventsByProject").setParameter("projectId", projectId)
            .setParameter("startTime", startTime).setParameter("endTime", endTime).getSingleResult()).intValue();
    }

    public Event newEvent(final Event.Level level, final String description, final String projectId, final String objectId,
        final String type, final String detail) {
        Event event = new Event();
        event.setLevel(level);
        event.setDescription(description);
        event.setProjectId(projectId);
        event.setObjectId(objectId);
        event.setType(type);
        event.setDetail(detail);
        event.setTime(new Date());
        this.em.persist(event);
        this.em.flush();
        return event;
    }

}
