/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: IOVFImporter.java 219 2011-08-01 12:33:08Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.MalformedURLException;

import javax.xml.bind.JAXBException;

import org.ow2.sirocco.cloudmanager.admin.exception.DuplicateNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidArgumentException;
import org.ow2.sirocco.cloudmanager.admin.exception.OVFImporterException;
import org.ow2.sirocco.cloudmanager.admin.exception.PermissionDeniedException;
import org.ow2.sirocco.cloudmanager.admin.exception.VMImageWithTheGivenURLAlreadyExistsException;
import org.ow2.sirocco.vmm.api.VMMException;

/**
 * This interface provides public methods for importing an OVF appliance from a
 * file to a local (temporary) file, parsing it, and creating the corresponding
 * SystemTemplate. Note that OVF Archives (i.e., OVA) will be imported (in the
 * local server) by the VirtualMachineImage Manager module.
 */
public interface IOVFImporter {

    /**
     * Imports the given OVF Appliance file on the local server.
     * 
     * @param userName
     * @param oVFApplianceLocation : e.g., http://aaa.bbb.ccc/ddd/eee.ovf
     * @return the local file location.
     * @throws OVFImporterException
     * @throws MalformedURLException
     * @throws IOException
     * @throws InvalidArgumentException
     * @throws PermissionDeniedException
     */
    String oVFApplianceFileImportation(final String userName, final String oVFApplianceLocation) throws OVFImporterException,
        MalformedURLException, IOException, InvalidArgumentException, PermissionDeniedException;

    /**
     * Parses the OVF Appliance file available at the given location (on the
     * local server), then creates the corresponding SystemTemplate, and
     * persists it.
     * 
     * @param userName
     * @param oVFApplianceLocationOnTheLocalFileSystem
     * @throws OVFImporterException
     * @throws FileNotFoundException
     * @throws JAXBException
     * @throws VMMException
     * @throws VMImageWithTheGivenURLAlreadyExistsException
     * @throws InvalidArgumentException
     * @throws DuplicateNameException
     * @throws PermissionDeniedException
     */
    void oVFApplianceParsingAndSystemTemplateCreation(final String userName,
        final String oVFApplianceLocationOnTheLocalFileSystem) throws OVFImporterException, FileNotFoundException,
        JAXBException, VMMException, VMImageWithTheGivenURLAlreadyExistsException, InvalidArgumentException,
        DuplicateNameException, PermissionDeniedException;

}
