/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: IProjectManager.java 233 2011-08-08 10:08:06Z otterley $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.util.List;

import org.ow2.sirocco.cloudmanager.admin.exception.InvalidProjectIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidProjectNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidUsernameException;
import org.ow2.sirocco.cloudmanager.admin.exception.UserAlreadyInProjectException;
import org.ow2.sirocco.cloudmanager.admin.exception.VMImageInUseException;
import org.ow2.sirocco.cloudmanager.admin.exception.VMInUseException;
import org.ow2.sirocco.cloudmanager.admin.exception.VolumeInUseException;
import org.ow2.sirocco.cloudmanager.admin.vo.ProjectVO;
import org.ow2.sirocco.cloudmanager.admin.vo.ResourceInfoVO;
import org.ow2.sirocco.cloudmanager.admin.vo.ResourceQuotaVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VMImageVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VolumeVO;
import org.ow2.sirocco.cloudmanager.business.Project;

public interface IProjectManager {

    public boolean checkIfUsernameInProjectId(final String username, final String projectId);

    public String getRightsForUsernameInProjectId(final String username, final String projectId);

    public Integer countCPUsByProject(final String projectId);

    public Integer countDiskByProject(final String projectId);

    public Integer countRAMByProject(final String projectId);

    public Integer countVMByProject(final String projectId);

    public Project createProject(final String projectId, final String projectName, final String desc, final String owner)
        throws UserAlreadyInProjectException;

    public List<ProjectVO> getAllProjectsVo();

    public ProjectVO getDefaultProjectVoByUsername(final String username) throws InvalidUsernameException;

    public List<Project> getProjectsByUsername(final String username) throws InvalidUsernameException;

    public List<ProjectVO> getProjectsVoByUsername(final String username) throws InvalidUsernameException;

    public void addUserToProject(final String username, final String projectId, final String rights, String fromUsername)
        throws UserAlreadyInProjectException;

    public void delUserFromProject(final String username, final String projectId, String fromUsername)
        throws InvalidProjectIdException, VMInUseException, VMImageInUseException;

    public void deleteProject(final String projectId) throws InvalidProjectIdException, VMInUseException,
        VMImageInUseException, VolumeInUseException;

    public Project getProjectByName(final String projectName) throws InvalidProjectNameException;

    public Project getProjectByProjectId(final String projectId) throws InvalidProjectIdException;

    public List<VirtualMachineVO> getVirtualMachinesVoByProjectId(final String projectId) throws InvalidProjectIdException;

    List<VolumeVO> getVolumeVoByProjectId(String projectId) throws InvalidProjectIdException;

    public ResourceInfoVO getResourceInfoVoByProjectId(final String projectId) throws InvalidProjectIdException;

    /**
     * Returns the resource quota of a given project
     * 
     * @param projectId the projectId
     * @return a resouce quota value object
     * @throws InvalidProjectIdException if the provided project is invalid
     */
    public ResourceQuotaVO getProjectResourceQuotaVo(String projectId) throws InvalidProjectIdException;

    /**
     * Updates the resource quota of a given project
     * 
     * @param projectId of the project
     * @param quota the new resource quota to assign to the project
     * @throws InvalidProjectIdException if the provided project is invalid
     */
    void updateProjectResourceQuota(final String projectId, final ResourceQuotaVO quota) throws InvalidProjectIdException;

    /**
     * Returns the VM images belonging to a given user in a project projectId
     * 
     * @param projectId
     * @return a list of VM image value objects
     * @throws InvalidProjectIdException
     */
    public abstract List<VMImageVO> getVMImagesVoByProjectId(String projectId) throws InvalidProjectIdException;

}
