/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: ISystemTemplateManager.java 223 2011-08-02 14:16:57Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.util.List;

import org.ow2.sirocco.cloudmanager.admin.exception.InvalidArgumentException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidSystemInstanceStatusException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidSystemTemplateStatusException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidUsernameException;
import org.ow2.sirocco.cloudmanager.admin.exception.PermissionDeniedException;
import org.ow2.sirocco.cloudmanager.admin.vo.SystemTemplateVO;

/**
 * This interface provides public methods for managing System Instances (i.e.,
 * instances of SystemTemplates).
 */
public interface ISystemTemplateManager {

    /**
     * @return
     */
    List<SystemTemplateVO> listSystemTemplates();

    /**
     * Destroys the SystemTemplate specified by the given systemTemplateId. This
     * method destroys the related/associated objects(s), and the SystemInstance
     * object itself (i.e. move object(s) to DESTROYED state).
     * 
     * @param userName
     * @param systemTemplateId
     * @throws InvalidUsernameException
     * @throws InvalidArgumentException
     * @throws PermissionDeniedException
     * @throws InvalidSystemInstanceStatusException
     * @throws InvalidSystemTemplateStatusException
     */
    void destroySystemTemplate(final String userName, final Integer systemTemplateId) throws InvalidUsernameException,
        InvalidArgumentException, PermissionDeniedException, InvalidSystemInstanceStatusException,
        InvalidSystemTemplateStatusException;

    /**
     * Remove SystemTemplate(s) in the Destroyed state from the DB (remove also
     * SystemTemplates' associated destroyed object(s)/line(s)).
     * 
     * @throws InvalidSystemInstanceStatusException
     */
    void cleanUpDestroyedSystemTemplates() throws InvalidSystemInstanceStatusException;

}
