/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: IVMImageManager.java 111 2011-07-06 13:24:49Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.util.List;

import org.ow2.sirocco.cloudmanager.admin.exception.InvalidArgumentException;
import org.ow2.sirocco.cloudmanager.admin.exception.VMImageWithTheGivenURLAlreadyExistsException;
import org.ow2.sirocco.cloudmanager.admin.vo.VMImageVO;
import org.ow2.sirocco.vmm.api.VMMException;

/**
 * This interface provides public methods for managing VMImage files (i.e.,
 * Virtual Machine Image files).
 */
public interface IVMImageManager {

    /**
     * Uploads the given VMImage file into Sirocco (more precisely, into the
     * Sirocco's cloudProvider identified by the given cloudProviderId).
     * 
     * @param uRL
     * @param format
     * @param name
     * @param description
     * @param checkSum
     * @param oVFSizeInMB
     * @param cloudProviderId
     * @param creationViaASystemTemplate
     * @throws VMMException
     * @throws VMImageWithTheGivenURLAlreadyExistsException
     * @throws InvalidArgumentException
     */
    void uploadVirtualMachineImageFile(String uRL, String format, String name, String description, String checkSum,
        Integer oVFSizeInMB, String cloudProviderId, Boolean creationViaASystemTemplate) throws VMMException,
        VMImageWithTheGivenURLAlreadyExistsException, InvalidArgumentException;

    /**
     * @return a list containing (all) the VirtualMachineImages known by
     *         Sirocco.
     */
    List<VMImageVO> getVMImageFiles();

}
