/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: IVolumeManager.java 208 2011-07-27 10:17:19Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.util.List;

import org.ow2.sirocco.cloudmanager.admin.exception.CloudAdminException;
import org.ow2.sirocco.cloudmanager.admin.exception.DuplicateNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidProjectIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidUsernameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVirtualMachineIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVolumeIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.PermissionDeniedException;
import org.ow2.sirocco.cloudmanager.admin.exception.ResourceQuotaExceededException;
import org.ow2.sirocco.cloudmanager.admin.exception.VolumeInUseException;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VolumeVO;
import org.ow2.sirocco.cloudmanager.business.Volume;
import org.ow2.sirocco.cloudmanager.clustermanager.api.VolumeSpec;

public interface IVolumeManager {
    // Volume createVolumeRecord(final String projectId, final String username,
    // VolumeSpec volumeSpec)
    // throws InvalidUsernameException, InvalidNameException,
    // ResourceQuotaExceededException, DuplicateNameException,
    // InvalidProjectIdException, PermissionDeniedException,
    // CloudAdminException;

    VolumeVO createVolume(final String projectId, final String username, VolumeSpec volumeSpec)
        throws InvalidUsernameException, InvalidNameException, ResourceQuotaExceededException, DuplicateNameException,
        InvalidProjectIdException, PermissionDeniedException, CloudAdminException;

    void destroyVolume(final String username, String volumeId) throws InvalidUsernameException, InvalidProjectIdException,
        InvalidVolumeIdException, VolumeInUseException, PermissionDeniedException, CloudAdminException;

    void attachVolumeToVirtualMachine(String projectId, final String username, String volumeId, String vmId)
        throws InvalidUsernameException, InvalidProjectIdException, InvalidVolumeIdException, InvalidVirtualMachineIdException,
        VolumeInUseException, PermissionDeniedException, CloudAdminException;

    void detachVolumeFromVirtualMachine(String projectId, final String username, String volumeId, String vmId)
        throws InvalidUsernameException, InvalidProjectIdException, InvalidVolumeIdException, InvalidVirtualMachineIdException,
        PermissionDeniedException, CloudAdminException;

    List<VirtualMachineVO> getVirtualMachineUsingVolume(String volumeId) throws InvalidVolumeIdException;

    Volume getVolumeByName(final String name);

    Volume getVolumeById(final int id);

}
