/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: ProjectManagerBean.java 275 2011-08-24 14:03:23Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.UUID;

import javax.ejb.EJB;
import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.Query;

import org.ow2.sirocco.cloudmanager.admin.event.DelProjectEvent;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidProjectIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidProjectNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidUsernameException;
import org.ow2.sirocco.cloudmanager.admin.exception.UserAlreadyInProjectException;
import org.ow2.sirocco.cloudmanager.admin.exception.VMImageInUseException;
import org.ow2.sirocco.cloudmanager.admin.exception.VMInUseException;
import org.ow2.sirocco.cloudmanager.admin.exception.VolumeInUseException;
import org.ow2.sirocco.cloudmanager.admin.vo.ProjectVO;
import org.ow2.sirocco.cloudmanager.admin.vo.ResourceInfoVO;
import org.ow2.sirocco.cloudmanager.admin.vo.ResourceQuotaVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VMImageVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VolumeVO;
import org.ow2.sirocco.cloudmanager.business.Project;
import org.ow2.sirocco.cloudmanager.business.ResourceQuota;
import org.ow2.sirocco.cloudmanager.business.RoleAssignment;
import org.ow2.sirocco.cloudmanager.business.RoleAssignment.Rights;
import org.ow2.sirocco.cloudmanager.business.User;
import org.ow2.sirocco.cloudmanager.business.VMImage;
import org.ow2.sirocco.cloudmanager.business.VirtualMachine;
import org.ow2.sirocco.cloudmanager.business.Volume;
import org.ow2.sirocco.cloudmanager.common.DbParameters;

@Stateless(name = "ProjectManagerBean", mappedName = "ProjectManagerBean")
@Remote(IRemoteProjectManager.class)
@Local(IProjectManager.class)
public class ProjectManagerBean implements IProjectManager {

    // private static Logger log =
    // Logger.getLogger(ProjectManagerBean.class.getName());

    @EJB(mappedName = "CloudAdminBean")
    private IRemoteCloudAdmin cloudAdmin;

    @EJB(mappedName = "UserManagerBean")
    private IUserManager userManager;

    @PersistenceContext(unitName = "persistence-unit/main", type = PersistenceContextType.TRANSACTION)
    protected EntityManager em;

    public Integer countCPUsByProject(final String projectId) {
        BigDecimal nb = new BigDecimal(0);
        Query q = this.em
            .createNativeQuery(
                "SELECT SUM(nbCpus) FROM VirtualMachine v, Project p WHERE p.id=v.project_id AND p.projectId=:projectId AND v.status<>'DESTROYED' GROUP BY p.projectId=:projectId")
            .setParameter("projectId", projectId);
        if (q.getResultList().size() > 0) {
            BigDecimal resultQ = null;
            resultQ = (BigDecimal) q.getResultList().get(0);
            nb = resultQ;
            return nb.intValue();
        } else {
            return 0;
        }
    }

    public Integer countDiskByProject(final String projectId) {
        BigDecimal totalDisk = new BigDecimal(0);
        Query q = this.em
            .createNativeQuery(
                "SELECT SUM(diskSizeMB) FROM VirtualMachine v, Project p WHERE p.id=v.project_id AND p.projectId=:projectId AND v.status<>'DESTROYED' GROUP BY p.projectId=:projectId")
            .setParameter("projectId", projectId);
        if (q.getResultList().size() > 0) {
            BigDecimal resultQ = null;
            resultQ = (BigDecimal) q.getResultList().get(0);
            totalDisk = resultQ;

        }

        q = this.em
            .createNativeQuery(
                "SELECT SUM(capacityInMB) FROM Volume v, Project p WHERE p.id=v.project_id AND p.projectId=:projectId AND v.status<>'DESTROYED' GROUP BY p.projectId=:projectId")
            .setParameter("projectId", projectId);
        if (q.getResultList().size() > 0) {
            BigDecimal resultQ = null;
            resultQ = (BigDecimal) q.getResultList().get(0);
            totalDisk = resultQ;
        }

        q = this.em
            .createNativeQuery(
                "SELECT SUM(diskSizeMB) FROM VMImage v, Project p WHERE p.id=v.project_id AND p.projectId=:projectId AND v.status<>'DESTROYED' GROUP BY p.projectId=:projectId")
            .setParameter("projectId", projectId);
        if (q.getResultList().size() > 0) {
            BigDecimal resultQ = null;
            resultQ = (BigDecimal) q.getResultList().get(0);
            totalDisk = resultQ;
        }

        return totalDisk.intValue();
    }

    public Integer countRAMByProject(final String projectId) {
        BigDecimal totalMem = new BigDecimal(0);
        Query q = this.em
            .createNativeQuery(
                "SELECT SUM(ramSizeMB) FROM VirtualMachine v, Project p WHERE p.id=v.project_id AND p.projectId=:projectId AND v.status<>'DESTROYED' GROUP BY p.projectId=:projectId")
            .setParameter("projectId", projectId);
        if (q.getResultList().size() > 0) {
            BigDecimal resultQ = null;
            resultQ = (BigDecimal) q.getResultList().get(0); // O is the first
            totalMem = resultQ;
            return totalMem.intValue();
        } else {
            return 0;
        }
    }

    public Integer countVMByProject(final String projectId) {
        @SuppressWarnings("unchecked")
        List<VirtualMachine> list = this.em
            .createQuery("FROM VirtualMachine v WHERE v.project.projectId=:projectId AND v.status<>'DESTROYED'")
            .setParameter("projectId", projectId).getResultList();
        return list.size();
    }

    public Project createProject(final String projectId, final String projectName, final String desc, final String owner)
        throws UserAlreadyInProjectException {

        String projId = projectId;
        Date todayDate = new Date();
        ResourceQuota quota = new ResourceQuota();
        quota.setCpuQuota(DbParameters.getInstance().DEFAULT_CPU_QUOTA);
        quota.setDiskQuotaInMB(DbParameters.getInstance().DEFAULT_DISK_QUOTA_IN_MB);
        quota.setRamQuotaInMB(DbParameters.getInstance().DEFAULT_RAM_QUOTA_IN_MB);
        quota.setVmQuota(DbParameters.getInstance().DEFAULT_VM_QUOTA);

        Project project = new Project();
        project.setCreateDate(todayDate);
        project.setDescription(desc);
        project.setName(projectName);
        project.setResourceQuota(quota);
        project.setOwner(this.userManager.getUserByUsername(owner));
        if (projId == null) {
            projId = "proj-" + UUID.randomUUID();
        }
        project.setProjectId(projId);

        this.em.persist(project);

        this.addUserToProject(owner, projId, Rights.ADMIN.toString(), null);

        return project;
    }

    public List<ProjectVO> getProjectsVoByUsername(final String username) throws InvalidUsernameException {
        List<Project> assignList = this.getProjectsByUsername(username);
        List<ProjectVO> resultVo = new ArrayList<ProjectVO>();
        for (Project proj : assignList) {
            resultVo.add(ProjectVO.from(proj, true));
        }
        return resultVo;
    }

    public List<Project> getProjectsByUsername(final String username) throws InvalidUsernameException {
        @SuppressWarnings("unchecked")
        List<Project> assignList = this.em
            .createQuery("SELECT r.project FROM RoleAssignment r WHERE r.user.username=:username")
            .setParameter("username", username).getResultList();

        if (assignList == null) {
            throw new InvalidUsernameException("username " + username + " unknown");
        }

        return assignList;
    }

    public List<VirtualMachineVO> getVirtualMachinesVoByProjectId(final String projectId) throws InvalidProjectIdException {
        Project project = this.getProjectByProjectId(projectId);
        if (project == null) {
            throw new InvalidProjectIdException("projectId " + projectId + " unknown");
        }

        Collection<VirtualMachine> vmsList = project.getVirtualMachines();

        List<VirtualMachineVO> resultVo = new ArrayList<VirtualMachineVO>();
        for (Object obj : vmsList) {
            VirtualMachine vm = (VirtualMachine) obj;
            if (!vm.getStatus().equals(VirtualMachine.Status.DESTROYED)) {
                resultVo.add(VirtualMachineVO.from(vm));
            }
        }
        return resultVo;
    }

    @Override
    public List<VolumeVO> getVolumeVoByProjectId(final String projectId) throws InvalidProjectIdException {
        Project project = this.getProjectByProjectId(projectId);
        if (project == null) {
            throw new InvalidProjectIdException("projectId " + projectId + " unknown");
        }

        Collection<Volume> volList = project.getVolumes();

        @SuppressWarnings("unchecked")
        Collection<Volume> listPublicVolumes = this.em.createQuery(
            "FROM Volume v WHERE v.visibility='PUBLIC' AND v.status='AVAILABLE'").getResultList();

        volList.addAll(listPublicVolumes);

        List<VolumeVO> resultVo = new ArrayList<VolumeVO>();
        for (Object obj : volList) {
            Volume vol = (Volume) obj;
            if (!vol.getStatus().equals(Volume.Status.DESTROYED)) {
                resultVo.add(VolumeVO.from(vol));
            }
        }
        return resultVo;
    }

    public List<VMImageVO> getVMImagesVoByProjectId(final String projectId) throws InvalidProjectIdException {

        Project proj = null;
        try {
            proj = this.getProjectByProjectId(projectId);
        } catch (InvalidProjectIdException e) {
            throw new InvalidProjectIdException("projectId " + projectId + " unknown");
        }

        @SuppressWarnings("unchecked")
        Collection<VMImage> listVmImages = this.em.createQuery(
            "FROM VMImage v WHERE v.visibility='PUBLIC' AND v.status='AVAILABLE'").getResultList();

        Collection<VMImage> listVmImages2 = proj.getVmImages();

        listVmImages.addAll(listVmImages2);

        List<VMImageVO> listVmImagesVo = new ArrayList<VMImageVO>();
        for (VMImage vmImage : listVmImages) {
            VMImageVO vmiVo = VMImageVO.from(vmImage);
            if (!vmiVo.getStatus().equals(VMImage.Status.DESTROYED)) {
                listVmImagesVo.add(vmiVo);
            }
        }
        return listVmImagesVo;
    }

    public ProjectVO getDefaultProjectVoByUsername(final String username) throws InvalidUsernameException {
        Project proj = (Project) this.em
            .createQuery("SELECT r.project FROM RoleAssignment r WHERE r.user.username=:username AND r.project.name=:username")
            .setParameter("username", username).getSingleResult();

        if (proj == null) {
            throw new InvalidUsernameException("username " + username + " unknown");
        }

        return ProjectVO.from(proj, true);
    }

    public boolean checkIfUsernameInProjectId(final String username, final String projectId) {
        @SuppressWarnings("unchecked")
        List<RoleAssignment> list = this.em
            .createQuery("FROM RoleAssignment r WHERE r.user.username=:username AND r.project.projectId=:projectId")
            .setParameter("username", username).setParameter("projectId", projectId).getResultList();
        if (null == list || list.size() == 0) {
            return false; // don't exist
        }
        return true; // exists
    }

    public String getRightsForUsernameInProjectId(final String username, final String projectId) {
        Rights result = (Rights) this.em
            .createQuery(
                "SELECT r.rights FROM RoleAssignment r WHERE r.user.username=:username AND r.project.projectId=:projectId")
            .setParameter("username", username).setParameter("projectId", projectId).getSingleResult();
        return result.toString();
    }

    @SuppressWarnings("unchecked")
    private Collection<Project> getAllProjects() {
        Query query = this.em.createQuery("SELECT p FROM Project p");
        return query.getResultList();
    }

    public List<ProjectVO> getAllProjectsVo() {
        Collection<Project> list = this.getAllProjects();
        List<ProjectVO> resultVo = new ArrayList<ProjectVO>();
        for (Project proj : list) {
            resultVo.add(ProjectVO.from(proj, true));
        }
        return resultVo;
    }

    public void addUserToProject(final String username, final String projectId, final String rights, final String fromUsername)
        throws UserAlreadyInProjectException {
        User user = this.userManager.getUserByUsername(username);
        Project proj = null;
        try {
            proj = this.getProjectByProjectId(projectId);
        } catch (InvalidProjectIdException e) {
            e.printStackTrace();
        }

        if (this.checkIfUsernameInProjectId(username, projectId)) {
            throw new UserAlreadyInProjectException("User already member of this project!");
        }

        RoleAssignment assign = new RoleAssignment(user, proj, Rights.valueOf(rights));
        this.em.persist(assign);
        user.getAssignments().add(assign);
        proj.getAssignments().add(assign);
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
    public void delUserFromProject(final String username, final String projectId, final String fromUsername)
        throws InvalidProjectIdException, VMInUseException, VMImageInUseException {

        User user = this.userManager.getUserByUsername(username);
        Project proj = this.getProjectByProjectId(projectId);

        Query queryVm = this.em
            .createQuery("FROM VirtualMachine v WHERE v.project=:proj AND v.user=:user AND v.status<>'DESTROYED'")
            .setParameter("proj", proj).setParameter("user", user);
        if (queryVm.getResultList().size() > 0) {
            throw new VMInUseException("User " + username + " has still VM in used in project " + proj.getName());
        }

        Query queryVmi = this.em.createQuery("FROM VMImage v WHERE v.project=:proj AND v.user=:user AND v.status<>'DESTROYED'")
            .setParameter("proj", proj).setParameter("user", user);
        if (queryVmi.getResultList().size() > 0) {
            throw new VMImageInUseException("User " + username + " has still VM image in project " + proj.getName());
        }

        long userid = user.getId();
        long projectid = proj.getId();
        RoleAssignment role = (RoleAssignment) this.em
            .createQuery("FROM RoleAssignment r WHERE r.userId=:userid AND r.projId=:projectid")
            .setParameter("projectid", projectid).setParameter("userid", userid).getSingleResult();
        this.em.remove(role);
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
    public void deleteProject(final String projectId) throws InvalidProjectIdException, VMInUseException,
        VMImageInUseException, VolumeInUseException {
        Project project = null;
        try {
            project = (Project) this.em.createQuery("FROM Project WHERE projectId=:projectId")
                .setParameter("projectId", projectId).getSingleResult();
        } catch (NoResultException e) {
            throw new InvalidProjectIdException("project " + projectId + " unknown");
        }

        Query queryVm = this.em.createQuery("FROM VirtualMachine v WHERE v.project=:proj AND v.status<>'DESTROYED'")
            .setParameter("proj", project);
        if (queryVm.getResultList().size() > 0) {
            throw new VMInUseException("VM is used in project " + project.getName());
        }

        Query queryVmi = this.em.createQuery("FROM VMImage v WHERE v.project=:proj AND v.status<>'DESTROYED'").setParameter(
            "proj", project);
        if (queryVmi.getResultList().size() > 0) {
            throw new VMImageInUseException("VM image is used in project " + project.getName());
        }

        Query queryVol = this.em.createQuery("FROM Volume v WHERE v.project=:proj AND v.status<>'DESTROYED'").setParameter(
            "proj", project);
        if (queryVol.getResultList().size() > 0) {
            throw new VolumeInUseException("Volume is used in project " + project.getName());
        }

        Query queryDelVmi = this.em.createQuery("DELETE VMImage v WHERE v.project = :proj");
        queryDelVmi.setParameter("proj", project).executeUpdate();

        Query queryDelVol = this.em.createQuery("DELETE Volume v WHERE v.project = :proj");
        queryDelVol.setParameter("proj", project).executeUpdate();

        Query queryVm2 = this.em.createQuery("FROM VirtualMachine v WHERE v.project = :proj");
        @SuppressWarnings("unchecked")
        List<VirtualMachine> vms = queryVm2.setParameter("proj", project).getResultList();

        for (VirtualMachine v : vms) {
            v.getAdditionalProperties().remove(0);
            this.em.remove(v);
        }

        // Remove users from project
        List<User> listUser = this.userManager.getUsersByProjectId(projectId);
        for (User u : listUser) {
            this.delUserFromProject(u.getUsername(), projectId, null);
        }

        this.em.remove(project);
        this.cloudAdmin.emitTopicMessage(new DelProjectEvent(projectId), "admin", "true");
    }

    public Project getProjectByName(final String projectName) throws InvalidProjectNameException {
        Project project = null;
        try {
            project = (Project) this.em.createQuery("FROM Project WHERE name = :name").setParameter("name", projectName)
                .getSingleResult();
        } catch (NoResultException ex) {
            throw new InvalidProjectNameException("project " + projectName + " unknown");
        }
        return project;
    }

    public Project getProjectByProjectId(final String projectId) throws InvalidProjectIdException {
        Project project = null;
        try {
            project = (Project) this.em.createQuery("FROM " + Project.class.getSimpleName() + " WHERE projectId = :projectId")
                .setParameter("projectId", projectId).getSingleResult();
        } catch (NoResultException ex) {
            throw new InvalidProjectIdException("project " + projectId + " unknown");
        }
        return project;
    }

    public ResourceInfoVO getResourceInfoVoByProjectId(final String projectId) throws InvalidProjectIdException {
        if (this.getProjectByProjectId(projectId) == null) {
            throw new InvalidProjectIdException("project " + projectId + " unknown");
        }
        ResourceInfoVO resourceInfo = new ResourceInfoVO();
        resourceInfo.setMemoryMB(this.countRAMByProject(projectId));
        resourceInfo.setStorageMB(this.countDiskByProject(projectId));
        resourceInfo.setNumCPUs(this.countCPUsByProject(projectId));
        resourceInfo.setNumVMs(this.countVMByProject(projectId));
        return resourceInfo;
    }

    public ResourceQuotaVO getProjectResourceQuotaVo(final String projectId) throws InvalidProjectIdException {
        Project project = this.getProjectByProjectId(projectId);
        if (project == null) {
            throw new InvalidProjectIdException("project " + projectId + " unknown");
        }
        ResourceQuotaVO quotaVo = new ResourceQuotaVO();
        quotaVo.setCpuQuota(project.getResourceQuota().getCpuQuota());
        quotaVo.setDiskQuotaInMB(project.getResourceQuota().getDiskQuotaInMB());
        quotaVo.setRamQuotaInMB(project.getResourceQuota().getRamQuotaInMB());
        quotaVo.setMaxNumberOfVMs(project.getResourceQuota().getVmQuota());
        return quotaVo;
    }

    public void updateProjectResourceQuota(final String projectId, final ResourceQuotaVO quota)
        throws InvalidProjectIdException {
        Project project = this.getProjectByProjectId(projectId);
        if (project == null) {
            throw new InvalidProjectIdException("project " + projectId + " unknown");
        }
        project.getResourceQuota().setCpuQuota(quota.getCpuQuota());
        project.getResourceQuota().setDiskQuotaInMB(quota.getDiskQuotaInMB());
        project.getResourceQuota().setRamQuotaInMB(quota.getRamQuotaInMB());
        project.getResourceQuota().setVmQuota(quota.getMaxNumberOfVMs());
        this.em.merge(project);
        this.em.flush();
    }

}
