/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: SystemInstanceManagerBean.java 373 2011-10-03 08:45:56Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Logger;

import javax.ejb.EJB;
import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.Query;

import org.ow2.sirocco.cloudmanager.admin.exception.CloudAdminException;
import org.ow2.sirocco.cloudmanager.admin.exception.InternalErrorException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidArgumentException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidProjectIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidSystemInstanceStatusException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidSystemTemplateStatusException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidUsernameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVolumeIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.PermissionDeniedException;
import org.ow2.sirocco.cloudmanager.admin.exception.ResourceQuotaExceededException;
import org.ow2.sirocco.cloudmanager.admin.exception.VolumeInUseException;
import org.ow2.sirocco.cloudmanager.admin.vo.FromNonVOToVO;
import org.ow2.sirocco.cloudmanager.admin.vo.SystemInstanceVO;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineSpec;
import org.ow2.sirocco.cloudmanager.admin.vo.VolumeVO;
import org.ow2.sirocco.cloudmanager.business.NetworkInstance;
import org.ow2.sirocco.cloudmanager.business.NetworkInstance.NetworkInstanceStatus;
import org.ow2.sirocco.cloudmanager.business.NetworkTemplate;
import org.ow2.sirocco.cloudmanager.business.Project;
import org.ow2.sirocco.cloudmanager.business.SystemInstance;
import org.ow2.sirocco.cloudmanager.business.SystemInstance.SystemInstanceStatus;
import org.ow2.sirocco.cloudmanager.business.SystemTemplate;
import org.ow2.sirocco.cloudmanager.business.SystemTemplate.SystemTemplateStatus;
import org.ow2.sirocco.cloudmanager.business.User;
import org.ow2.sirocco.cloudmanager.business.VirtualMachine;
import org.ow2.sirocco.cloudmanager.business.VirtualMachineTemplate;
import org.ow2.sirocco.cloudmanager.business.Volume;
import org.ow2.sirocco.cloudmanager.business.VolumeTemplate;
import org.ow2.sirocco.cloudmanager.clustermanager.api.VolumeSpec;
import org.ow2.sirocco.cloudmanager.utils.PermissionChecker;

@Stateless(name = "SystemInstanceManagerBean", mappedName = "SystemInstanceManagerBean")
@Remote(IRemoteSystemInstanceManager.class)
@Local(ISystemInstanceManager.class)
public class SystemInstanceManagerBean implements ISystemInstanceManager, IRemoteSystemInstanceManager {

    private static Logger log = Logger.getLogger(SystemInstanceManagerBean.class.getName());

    @PersistenceContext(unitName = "persistence-unit/main", type = PersistenceContextType.TRANSACTION)
    private EntityManager em = null;

    @EJB(mappedName = "VolumeManagerBean")
    private IVolumeManager volumeManagerBean;

    @EJB(mappedName = "CloudAdminBean")
    private ICloudAdmin cloudAdminBean;

    @EJB(mappedName = "UserManagerBean")
    private IUserManager userManagerBean;

    @EJB(mappedName = "ProjectManagerBean")
    private IProjectManager projectManagerBean;

    @EJB(mappedName = "SystemInstanceManagerBean")
    private ISystemInstanceManager systemInstanceManagerBean;

    private static final int RANDOM_MULTIPLIER_100 = 100;

    @Override
    public List<SystemInstanceVO> listSystemInstances() {
        SystemInstanceManagerBean.log.info("Enters in: listSystemInstance()");
        List<SystemInstanceVO> result;
        Query systemInstanceList = this.em.createQuery("FROM " + SystemInstance.class.getSimpleName() + " s");
        if (systemInstanceList == null) {
            result = null;
        } else {
            result = new ArrayList<SystemInstanceVO>();
            @SuppressWarnings("unchecked")
            List<SystemInstance> siList = systemInstanceList.getResultList();
            for (SystemInstance si : siList) {
                result.add(FromNonVOToVO.fromSystemInstanceToSystemInstanceVO(si));
            }
        }
        SystemInstanceManagerBean.log.finest("Leaves: listSystemInstance()");
        return result;
    }

    @Override
    public Integer createSystemInstance(final String userName, final String projectId, final String systemTemplateName,
        final Properties properties) throws ResourceQuotaExceededException, PermissionDeniedException, CloudAdminException {
        SystemInstanceManagerBean.log.info("Enters in: createSystemInstance(...)");
        Integer result = null;

        SystemTemplate sT = null;

        // Check if a user corresponding to userName exists.
        User user = null;
        // Query user_temp_list_query = this.em
        // .createQuery("FROM " + User.class.getSimpleName() +
        // " u WHERE u.username=:userName").setParameter("userName",
        // userName);
        // if (user_temp_list_query == null) {
        // throw new InvalidArgumentException("The given userName: " + userName
        // + " is unknown.");
        // } else {
        // if (user_temp_list_query.getResultList().isEmpty()) {
        // throw new InvalidArgumentException("The given userName: " + userName
        // + " is unknown.");
        // } else {
        // user = (User) user_temp_list_query.getResultList().iterator().next();
        // }
        // }
        user = this.userManagerBean.getUserByUsername(userName);
        if (user == null) {
            throw new InvalidArgumentException("The given userName: " + userName + " is unknown.");
        } else {
            // nothing to do.
        }

        // Check if a project corresponding to projectId exists.
        Project project = null;
        // Query project_temp_list_query = this.em.createQuery(
        // "FROM " + Project.class.getSimpleName() +
        // " p WHERE p.projectId=:projectId").setParameter("projectId",
        // projectId);
        // if (project_temp_list_query == null) {
        // throw new InvalidArgumentException("The given projectId: " +
        // projectId + " is unknown.");
        // } else {
        // if (project_temp_list_query.getResultList().isEmpty()) {
        // throw new InvalidArgumentException("The given projectId: " +
        // projectId + " is unknown.");
        // } else {
        // project = (Project)
        // project_temp_list_query.getResultList().iterator().next();
        // }
        // }
        project = this.projectManagerBean.getProjectByProjectId(projectId);
        if (project == null) {
            throw new InvalidArgumentException("The given projectId: " + projectId + " is unknown.");
        } else {
            // nothing to do.
        }

        // Check user's permission.
        if (new PermissionChecker().canManageSystemInstances(user)) {
            // nothing to do.
        } else {
            throw new PermissionDeniedException("Permission denied.");
        }

        // Check if a SystemTemplate corresponding to systemTemplateId exists.
        Query systemTemplate_temp_list_query = this.em.createQuery(
            "FROM " + SystemTemplate.class.getSimpleName() + " s WHERE s.name=:name").setParameter("name", systemTemplateName);
        if (systemTemplate_temp_list_query == null) {
            throw new InvalidArgumentException("The given systemTemplateName: " + systemTemplateName + " is unknown.");
        } else {
            if (systemTemplate_temp_list_query.getResultList().isEmpty()) {
                throw new InvalidArgumentException("The given systemTemplateName: " + systemTemplateName + " is unknown.");
            } else {
                sT = (SystemTemplate) systemTemplate_temp_list_query.getResultList().iterator().next();
            }
        }

        // Check if the SystemTemplate is in the available state.
        if (SystemTemplateStatus.AVAILABLE.equals(sT.getStatus())) {
            // nothing to do.
        } else {
            throw new InvalidSystemTemplateStatusException("The given systemTemplateName: " + systemTemplateName
                + " is NOT in the " + SystemTemplateStatus.AVAILABLE + " state.");
        }

        // If a systemTemplate is available, it means that all its children are
        // "available" too (i.e., VolumeTemplate, VirtualMachineTemplate,
        // VMImage, ...). So the following check isn't nedeed.

        // // Check if the VMImage(s) of the SystemTemplate's VolumeTemplate(s)
        // are
        // // in the available state.
        // Iterator<VolumeTemplate> vT_temp_ITERATOR =
        // sT.getVolumeTemplates().iterator();
        // while (vT_temp_ITERATOR.hasNext()) {
        // VolumeTemplate vT_temp = vT_temp_ITERATOR.next();
        // VMImage vMImage = vT_temp.getVirtualMachineImage();
        // if (vMImage == null) {
        // // nothing to do.
        // } else if (VMImage.Status.AVAILABLE.equals(vMImage.getStatus())) {
        // // nothing to do.
        // } else {
        // throw new InvalidVMImageStatusException("The vMImage with id: " +
        // vMImage.getId() + ", and with name: "
        // + vMImage.getName() + " is NOT in the " + VMImage.Status.AVAILABLE +
        // " state.");
        // }
        // }

        // Instanciate the SystemTemplate :
        // - create SystemInstance
        SystemInstance sI = new SystemInstance();
        sI.setStatus(SystemInstanceStatus.CREATING);
        sI.setUser(user);
        user.getSystemInstances().add(sI);
        sI.setProject(project);
        project.getSystemInstances().add(sI);
        sI.setSystemTemplate(sT);
        sT.getSystemInstances().add(sI);

        // - create Volume(s), and Volume(s) for real (PENDING,
        // and then, AVAILABLE)
        Iterator<VolumeTemplate> vT_temp_ITERATOR = sT.getVolumeTemplates().iterator();
        Hashtable<Integer, Integer> volumeId_volumeTemplateId_hT = new Hashtable<Integer, Integer>();
        while (vT_temp_ITERATOR.hasNext()) {
            VolumeTemplate vT_temp = vT_temp_ITERATOR.next();

            // Volume volume = new Volume();
            // volume.setBootableVolume(bootableVolume);
            // volume.setName(name);
            // volume.setCapacityInMB(vT_temp.getCapacityInMB());
            // volume.setDescription(description);
            // volume.setProviderId(providerId);
            // volume.setStatus(Volume.Status.PENDING);

            Map<String, String> constraints = new HashMap<String, String>();
            // constraints.put("hypervisor", "dummy");
            constraints.put("hypervisor", vT_temp.getVolumeFormat());
            // constraints.put("location", "Grenoble");

            VolumeSpec volumeSpec = new VolumeSpec();
            volumeSpec.setCapacityInMB(vT_temp.getCapacityInMB());
            volumeSpec.setName(vT_temp.getName() + "_" + System.nanoTime() + "_"
                + new Double(Math.random() * SystemInstanceManagerBean.RANDOM_MULTIPLIER_100).intValue());
            volumeSpec.setConstraints(constraints);

            VolumeVO volumeVO = this.volumeManagerBean.createVolume(projectId, userName, volumeSpec);
            volumeId_volumeTemplateId_hT.put(Integer.parseInt(volumeVO.getVolumeId()), vT_temp.getId());
        }

        Iterator<Integer> volumeId_temp_ITERATOR = volumeId_volumeTemplateId_hT.keySet().iterator();
        while (volumeId_temp_ITERATOR.hasNext()) {
            Integer volumeId_temp = volumeId_temp_ITERATOR.next();
            Volume volume = null;

            // Check if the Volume corresponding to the given volumeId_temp has
            // already been persisted in DB.
            volume = this.volumeManagerBean.getVolumeById(volumeId_temp);
            if (volume == null) {
                throw new InvalidArgumentException("The given volumeId: " + volumeId_temp + " is unknown.");
            } else {
                // nothing to do.
            }

            volume.getSystemInstances().add(sI);
            sI.getVolumes().add(volume);

            Integer volumeTemplateId = volumeId_volumeTemplateId_hT.get(volumeId_temp);

            vT_temp_ITERATOR = sT.getVolumeTemplates().iterator();
            while (vT_temp_ITERATOR.hasNext()) {
                VolumeTemplate vT_temp = vT_temp_ITERATOR.next();
                if (vT_temp.getId().equals(volumeTemplateId)) {
                    volume.setVolumeTemplate(vT_temp);
                    vT_temp.getVolumes().add(volume);
                } else {
                    // nothing to do.
                }
            }

        }

        // - create VirtualMachine(s), and creation of the VM(s) for real via
        // CloudAdminBean.
        // Warning, VMs creation requires that the VMs' main Volumes are in the
        // AVAILABLE state/status.
        Iterator<VirtualMachineTemplate> vMTemplate_temp_ITERATOR = sT.getVirtualMachineTemplates().iterator();
        while (vMTemplate_temp_ITERATOR.hasNext()) {
            VirtualMachineTemplate vMTemplate_temp = vMTemplate_temp_ITERATOR.next();

            // Creation, and filling vMSpec. (use vMTemplate_temp's data).
            VirtualMachineSpec vMSpec = new VirtualMachineSpec();
            vMSpec.setVmName(vMTemplate_temp.getName() + "_" + System.nanoTime() + "_"
                + new Double(Math.random() * 100).intValue());
            vMSpec.setNumCPUs(vMTemplate_temp.getNbCpus());
            vMSpec.setMemorySizeMB(vMTemplate_temp.getRamSizeMB());
            vMSpec.setUserData(new HashMap<String, String>());

            List<Integer> volumeId_aL = new ArrayList<Integer>();
            Iterator<Volume> volume_temp_ITERATOR = vMTemplate_temp.getVolumeTemplates().iterator().next().getVolumes()
                .iterator();
            while (volume_temp_ITERATOR.hasNext()) {
                volumeId_aL.add(volume_temp_ITERATOR.next().getId());
            }

            String vMId = this.cloudAdminBean.createVirtualMachineWithVolumes(projectId, userName, vMSpec, volumeId_aL);

            VirtualMachine vM = this.cloudAdminBean.getVirtualMachineById(new Integer(vMId));

            vM.setVirtualMachineTemplate(vMTemplate_temp);
            vMTemplate_temp.getVirtualMachines().add(vM);

            vM.setSystemInstance(sI);
            sI.getVirtualMachines().add(vM);

            volume_temp_ITERATOR = sI.getVolumes().iterator();
            while (volume_temp_ITERATOR.hasNext()) {
                Volume volume_temp = volume_temp_ITERATOR.next();

                VolumeTemplate vT = volume_temp.getVolumeTemplate();

                Iterator<VirtualMachineTemplate> vMT_temp_ITERATOR = vT.getVirtualMachineTemplates().iterator();
                while (vMT_temp_ITERATOR.hasNext()) {
                    VirtualMachineTemplate vMT_temp = vMT_temp_ITERATOR.next();
                    if (vMT_temp.getId().equals(vM.getId())) {
                        volume_temp.getVirtualMachines().add(vM);
                        vM.getVolumes().add(volume_temp);
                    } else {
                        // nothing to do.
                    }
                }
            }
        }

        // - create Network(s)
        Iterator<NetworkTemplate> nI_temp_ITERATOR = sT.getNetworkTemplates().iterator();
        while (nI_temp_ITERATOR.hasNext()) {
            NetworkTemplate nT_temp = nI_temp_ITERATOR.next();
            NetworkInstance ni = new NetworkInstance();
            ni.setStatus(NetworkInstanceStatus.CREATED);
            ni.setNetworkTemplate(nT_temp);
            nT_temp.getNetworks().add(ni);
            ni.setSystemInstance(sI);
            sI.getNetworks().add(ni);
            this.em.persist(sI);
        }

        // Persist everything.
        this.em.persist(sI);
        this.em.flush();

        SystemInstanceManagerBean.log.finest("Leaves: createSystemInstance(...)");
        return result;
    }

    @Override
    public void startSystemInstance(final String userName, final Integer systemInstanceId) throws InvalidArgumentException,
        InvalidSystemInstanceStatusException, PermissionDeniedException, CloudAdminException {
        SystemInstanceManagerBean.log.info("Enters in: startSystemInstance(...)");
        // Check if a user corresponding to userName exists.
        User user = null;
        Query user_temp_list_query = this.em
            .createQuery("FROM " + User.class.getSimpleName() + " u WHERE u.username=:userName").setParameter("userName",
                userName);
        if (user_temp_list_query == null) {
            throw new InvalidArgumentException("The given userName: " + userName + " is unknown.");
        } else {
            if (user_temp_list_query.getResultList().isEmpty()) {
                throw new InvalidArgumentException("The given userName: " + userName + " is unknown.");
            } else {
                user = (User) user_temp_list_query.getResultList().iterator().next();
            }
        }

        // Check user's permission.
        if (new PermissionChecker().canManageSystemInstances(user)) {
            // nothing to do.
        } else {
            throw new PermissionDeniedException("Permission denied.");
        }

        // Check if a SystemInstance corresponding to systemInstanceId exists.
        SystemInstance sI = null;
        Query systemInstance_temp_list_query = this.em.createQuery(
            "FROM " + SystemInstance.class.getSimpleName() + " s WHERE s.id=:id").setParameter("id", systemInstanceId);
        if (systemInstance_temp_list_query == null) {
            throw new InvalidArgumentException("The given systemInstance: " + systemInstanceId + " is unknown.");
        } else {
            if (systemInstance_temp_list_query.getResultList().isEmpty()) {
                throw new InvalidArgumentException("The given systemInstance: " + systemInstanceId + " is unknown.");
            } else {
                sI = (SystemInstance) systemInstance_temp_list_query.getResultList().iterator().next();
            }
        }

        // Check that it is in the CREATED or STOPPED state.
        if (SystemInstanceStatus.CREATED.equals(sI.getStatus())) {
            // nothing to do.
        } else if (SystemInstanceStatus.STOPPED.equals(sI.getStatus())) {
            // nothing to do.
        } else {
            throw new InvalidSystemInstanceStatusException("The given systemInstance: " + systemInstanceId + " is NOT in the "
                + SystemInstanceStatus.CREATED + ", or " + SystemInstanceStatus.STOPPED + " state.");
        }

        // Move its state to STARTING.
        sI.setStatus(SystemInstanceStatus.STARTING);
        this.em.persist(sI);
        this.em.flush();

        // Use cloudAdminBean to start the VirtualMachines (in the right order),
        // via clusterManager.startVirtualMachine(vmRef);
        List<VirtualMachineTemplate> vMT_list = sI.getSystemTemplate().getVirtualMachineTemplates();
        List<VirtualMachineTemplate> vMT_list_CLONE = new ArrayList<VirtualMachineTemplate>();
        Iterator<VirtualMachineTemplate> vMT_temp_ITERATOR = vMT_list.iterator();
        while (vMT_temp_ITERATOR.hasNext()) {
            VirtualMachineTemplate vMT_temp = vMT_temp_ITERATOR.next();
            vMT_list_CLONE.add(vMT_temp);
        }

        int startupLevel = 0;
        while (!(vMT_list_CLONE.isEmpty())) {
            Iterator<VirtualMachineTemplate> vMT_temp_CLONE_ITERATOR = vMT_list_CLONE.iterator();
            List<VirtualMachineTemplate> removeMe = new ArrayList<VirtualMachineTemplate>();
            while (vMT_temp_CLONE_ITERATOR.hasNext()) {
                VirtualMachineTemplate vMT_temp_CLONE = vMT_temp_CLONE_ITERATOR.next();
                if (startupLevel == vMT_temp_CLONE.getoVF_order()) {
                    // Find the corresponding VM.
                    Iterator<VirtualMachine> vMFromSI_temp_ITERATOR = sI.getVirtualMachines().iterator();
                    while (vMFromSI_temp_ITERATOR.hasNext()) {
                        VirtualMachine vMFromSI_temp = vMFromSI_temp_ITERATOR.next();
                        if (vMT_temp_CLONE.getId().equals(vMFromSI_temp.getVirtualMachineTemplate().getId())) {
                            // Start the VM (cloudAdminBean.startVirtualMachine
                            // self-changes the VM state/status to STARTING).
                            this.cloudAdminBean.startVirtualMachine(vMFromSI_temp.getId().toString());
                        } else {
                            // nothing to do.
                        }
                    }
                    removeMe.add(vMT_temp_CLONE);
                } else {
                    // nothing to do.
                }
            }

            // Remove the just "starting" VM from vMT_list_CLONE.
            Iterator<VirtualMachineTemplate> removeME_temp_ITERATOR = removeMe.iterator();
            while (removeME_temp_ITERATOR.hasNext()) {
                vMT_list_CLONE.remove(removeME_temp_ITERATOR.next());
            }

            startupLevel = startupLevel + 1;
        }

        this.em.persist(sI);
        this.em.flush();

        SystemInstanceManagerBean.log.finest("Leaves: startSystemInstance(...)");
    }

    @Override
    public void stopSystemInstance(final String userName, final Integer systemInstanceId) throws InvalidArgumentException,
        InvalidSystemInstanceStatusException, PermissionDeniedException, CloudAdminException {
        SystemInstanceManagerBean.log.info("Enters in: stopSystemInstance(...)");
        // Check if a user corresponding to userName exists.
        User user = null;
        Query user_temp_list_query = this.em
            .createQuery("FROM " + User.class.getSimpleName() + " u WHERE u.username=:userName").setParameter("userName",
                userName);
        if (user_temp_list_query == null) {
            throw new InvalidArgumentException("The given userName: " + userName + " is unknown.");
        } else {
            if (user_temp_list_query.getResultList().isEmpty()) {
                throw new InvalidArgumentException("The given userName: " + userName + " is unknown.");
            } else {
                user = (User) user_temp_list_query.getResultList().iterator().next();
            }
        }

        // Check user's permission.
        if (new PermissionChecker().canManageSystemInstances(user)) {
            // nothing to do.
        } else {
            throw new PermissionDeniedException("Permission denied.");
        }

        // Check if a SystemInstance corresponding to systemInstanceId exists.
        SystemInstance sI = null;
        Query systemInstance_temp_list_query = this.em.createQuery(
            "FROM " + SystemInstance.class.getSimpleName() + " s WHERE s.id=:id").setParameter("id", systemInstanceId);
        if (systemInstance_temp_list_query == null) {
            throw new InvalidArgumentException("The given systemInstance: " + systemInstanceId + " is unknown.");
        } else {
            if (systemInstance_temp_list_query.getResultList().isEmpty()) {
                throw new InvalidArgumentException("The given systemInstance: " + systemInstanceId + " is unknown.");
            } else {
                sI = (SystemInstance) systemInstance_temp_list_query.getResultList().iterator().next();
            }
        }

        // Check that it is in the STARTED state.
        if (SystemInstanceStatus.STARTED.equals(sI.getStatus())) {
            // nothing to do.
        } else {
            throw new InvalidSystemInstanceStatusException("The given systemInstance: " + systemInstanceId + " is NOT in the "
                + SystemInstanceStatus.STARTED + " state.");
        }

        // Move its state to STOPPING.
        sI.setStatus(SystemInstanceStatus.STOPPING);
        this.em.persist(sI);
        this.em.flush();

        // Use cloudAdminBean to stop the VirtualMachines (in the correct
        // order), via stopVirtualMachine(...);
        List<VirtualMachineTemplate> vMT_list = sI.getSystemTemplate().getVirtualMachineTemplates();
        List<VirtualMachineTemplate> vMT_list_CLONE = new ArrayList<VirtualMachineTemplate>();
        Iterator<VirtualMachineTemplate> vMT_temp_ITERATOR = vMT_list.iterator();
        while (vMT_temp_ITERATOR.hasNext()) {
            VirtualMachineTemplate vMT_temp = vMT_temp_ITERATOR.next();
            vMT_list_CLONE.add(vMT_temp);
        }

        // Let's find the max startupLevel (i.e., oVF_order).
        Integer startupLevel = null;
        Iterator<VirtualMachineTemplate> vMT_temp_CLONE_ITERATOR = vMT_list_CLONE.iterator();
        while (vMT_temp_CLONE_ITERATOR.hasNext()) {
            VirtualMachineTemplate vMT_temp_CLONE = vMT_temp_CLONE_ITERATOR.next();
            if (startupLevel == null) {
                startupLevel = vMT_temp_CLONE.getoVF_order();
            } else if (vMT_temp_CLONE.getoVF_order() > startupLevel) {
                startupLevel = vMT_temp_CLONE.getoVF_order();
            } else {
                // nothing to do.
            }
        }

        // Stop the VM(s), beginning from the startupLevel equals max oVF_order,
        // and finishing at startupLevel equals 0.
        while (startupLevel >= 0) {
            Iterator<VirtualMachineTemplate> vMT_temp_CLONE_ITERATOR_2 = vMT_list_CLONE.iterator();
            List<VirtualMachineTemplate> removeMe = new ArrayList<VirtualMachineTemplate>();
            while (vMT_temp_CLONE_ITERATOR_2.hasNext()) {
                VirtualMachineTemplate vMT_temp_CLONE = vMT_temp_CLONE_ITERATOR_2.next();
                if (startupLevel.equals(vMT_temp_CLONE.getoVF_order())) {
                    // Find the corresponding VM.
                    Iterator<VirtualMachine> vMFromSI_temp_ITERATOR = sI.getVirtualMachines().iterator();
                    while (vMFromSI_temp_ITERATOR.hasNext()) {
                        VirtualMachine vMFromSI_temp = vMFromSI_temp_ITERATOR.next();
                        if (vMT_temp_CLONE.getId().equals(vMFromSI_temp.getVirtualMachineTemplate().getId())) {
                            // Start the VM (cloudAdminBean.startVirtualMachine
                            // self-changes the VM state/status to STARTING).
                            this.cloudAdminBean.stopVirtualMachine(vMFromSI_temp.getId().toString(), userName);
                        } else {
                            // nothing to do.
                        }
                    }
                    removeMe.add(vMT_temp_CLONE);
                } else {
                    // nothing to do.
                }
            }

            // Remove the just "stopping" VM from vMT_list_CLONE.
            Iterator<VirtualMachineTemplate> removeME_temp_ITERATOR = removeMe.iterator();
            while (removeME_temp_ITERATOR.hasNext()) {
                vMT_list_CLONE.remove(removeME_temp_ITERATOR.next());
            }

            startupLevel = startupLevel - 1;
        }

        if (vMT_list_CLONE.isEmpty()) {
            // nothing to do.
        } else {
            try {
                throw new InternalErrorException("vMT_list_CLONE is NOT empty: " + vMT_list_CLONE.size() + ".");
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        this.em.persist(sI);
        this.em.flush();

        SystemInstanceManagerBean.log.finest("Leaves: stopSystemInstance(...)");
    }

    @Override
    public void destroySystemInstance(final String userName, final Integer systemInstanceId) throws InvalidUsernameException,
        InvalidArgumentException, PermissionDeniedException, InvalidSystemInstanceStatusException, CloudAdminException {
        SystemInstanceManagerBean.log.info("Enters in: destroySystemInstance(...)");
        // Check if a user corresponding to userName exists.
        User user = null;
        Query userList = this.em.createQuery("FROM " + User.class.getSimpleName() + " u WHERE u.username=:userName")
            .setParameter("userName", userName);
        if (userList == null) {
            throw new InvalidArgumentException("The given userName: " + userName + " is unknown.");
        } else {
            if (userList.getResultList().isEmpty()) {
                throw new InvalidArgumentException("The given userName: " + userName + " is unknown.");
            } else {
                user = (User) userList.getResultList().iterator().next();
            }
        }

        // Check user's permission.
        if (new PermissionChecker().canManageSystemInstances(user)) {
            // nothing to do.
        } else {
            throw new PermissionDeniedException("Permission denied.");
        }

        // Check if a SystemInstance corresponding to systemInstanceId exists.
        SystemInstance si = null;
        Query systemInstanceList = this.em.createQuery("FROM " + SystemInstance.class.getSimpleName() + " s WHERE s.id=:id")
            .setParameter("id", systemInstanceId);
        if (systemInstanceList == null) {
            throw new InvalidArgumentException("The given systemInstance: " + systemInstanceId + " is unknown.");
        } else {
            if (systemInstanceList.getResultList().isEmpty()) {
                throw new InvalidArgumentException("The given systemInstance: " + systemInstanceId + " is unknown.");
            } else {
                si = (SystemInstance) systemInstanceList.getResultList().iterator().next();
            }
        }

        // Check that it is in the STOPPED state.
        if (SystemInstanceStatus.CREATED.equals(si.getStatus())) {
            // nothing to do.
        } else if (SystemInstanceStatus.STOPPED.equals(si.getStatus())) {
            // nothing to do.
        } else {
            throw new InvalidSystemInstanceStatusException("The given systemInstance: " + systemInstanceId + " is NOT in the "
                + SystemInstanceStatus.CREATED + ", or " + SystemInstanceStatus.STOPPED + " state.");
        }

        // Move its state to DESTROYING.
        this.systemInstanceManagerBean.moveStatusOfSiToDESTROYING(si.getId());

        // "Destroy" the SystemInstance's related objects, and the
        // SystemInstance itself :
        // - destroy Network(s)
        for (NetworkInstance niTemp : si.getNetworks()) {
            NetworkInstance ni = null;
            ni = this.em.find(NetworkInstance.class, niTemp.getId());
            ni.setStatus(NetworkInstanceStatus.DESTROYED);
            this.em.merge(ni);
            this.em.flush();
        }

        // - destroy the real VirtualMachine(s), then move, if needed,
        // their representation in DB to DESTROYED status.
        for (VirtualMachine vmTemp : si.getVirtualMachines()) {
            VirtualMachine vm = null;
            vm = this.em.find(VirtualMachine.class, vmTemp.getId());
            this.cloudAdminBean.destroyVirtualMachine(vm.getId().toString(), userName);
            // VMs' state is automatically updated by the system.
        }

        List<Integer> volumesIdToBeDeleted = new ArrayList<Integer>();
        for (Volume volumeTemp : si.getVolumes()) {
            volumesIdToBeDeleted.add(volumeTemp.getId());
        }

        this.systemInstanceManagerBean.destroyVolume(userName, volumesIdToBeDeleted);

        // SystemInstance's state is automatically moved to DESTROYED (see
        // CloudAdminMDB).

        SystemInstanceManagerBean.log.finest("Leaves: destroySystemInstance(...)");
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
    public Integer moveStatusOfSiToDESTROYING(final Integer siId) {
        Integer result;
        SystemInstance si = this.em.find(SystemInstance.class, siId);
        si.setStatus(SystemInstanceStatus.DESTROYING);
        this.em.merge(si);
        this.em.flush();
        result = si.getId();
        return result;
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
    public boolean destroyVolume(final String userName, final List<Integer> volumesIdToBeDeleted)
        throws InvalidUsernameException, InvalidProjectIdException, InvalidVolumeIdException, VolumeInUseException,
        PermissionDeniedException, CloudAdminException {
        boolean result;
        // - destroy Volume(s), and Volume(s) for real (AVAILABLE, and then,
        // DESTROYED)
        for (Integer volumeId : volumesIdToBeDeleted) {
            Volume volume = null;
            volume = this.em.find(Volume.class, volumeId);
            this.volumeManagerBean.destroyVolume(userName, volume.getId().toString());
            // Volume's state is automatically updated by
            // volumeManagerBean.destroyVolume(...);
        }

        result = true;
        return result;
    }

    @Override
    public void cleanUpDestroyedSystemInstances() {
        SystemInstanceManagerBean.log.info("Enters in: cleanUpDestroyedSystemInstances(...)");

        // Retrieve all the destroyed SystemInstance(s).
        Query destroyedSystemInstanceQuery = this.em.createQuery(
            "FROM " + SystemInstance.class.getSimpleName() + " s WHERE s.status=:status").setParameter("status",
            SystemInstanceStatus.DESTROYED);
        if (destroyedSystemInstanceQuery == null) {
            // nothing to do.
        } else {
            @SuppressWarnings("unchecked")
            List<SystemInstance> destroyedSystemInstanceList = destroyedSystemInstanceQuery.getResultList();

            for (SystemInstance siTemp : destroyedSystemInstanceList) {

                // Clean up (i.e., delete) this SystemInstance's
                // NetworkInstance(s)
                ArrayList<Integer> niIdToDelete = new ArrayList<Integer>();
                for (NetworkInstance ni : siTemp.getNetworks()) {
                    niIdToDelete.add(ni.getId());
                }

                for (Integer niId : niIdToDelete) {
                    NetworkInstance ni = null;
                    ni = this.em.find(NetworkInstance.class, niId);

                    NetworkTemplate nt = ni.getNetworkTemplate();
                    nt.getNetworks().remove(ni);
                    ni.setNetworkTemplate(null);

                    siTemp.getNetworks().remove(ni);
                    ni.setSystemInstance(null);

                    this.em.remove(ni);
                    this.em.flush();
                }

                // Clean up (i.e., delete) this SystemInstance's Volume(s)
                ArrayList<Integer> volumeIdToDelete = new ArrayList<Integer>();
                for (Volume volume : siTemp.getVolumes()) {
                    volumeIdToDelete.add(volume.getId());
                }

                for (Integer volumeId : volumeIdToDelete) {
                    Volume volume = null;
                    volume = this.em.find(Volume.class, volumeId);

                    VolumeTemplate vt = volume.getVolumeTemplate();
                    vt.getVolumes().remove(volume);
                    volume.setVolumeTemplate(null);

                    for (VirtualMachine vmTemp : volume.getVirtualMachines()) {
                        if (siTemp.getId().equals(vmTemp.getSystemInstance())) {
                            vmTemp.getVolumes().remove(volume);
                            volume.getVirtualMachines().remove(vmTemp);
                        } else {
                            // nothing to do.
                        }
                    }

                    siTemp.getVolumes().remove(volume);
                    volume.getSystemInstances().remove(siTemp);

                    if ((volume.getSystemInstances().isEmpty()) && (volume.getVirtualMachines().isEmpty())) {
                        this.em.remove(volume);
                    } else {
                        this.em.merge(volume);
                    }
                    this.em.flush();
                }

                // Clean up (i.e., delete) this SystemInstance's
                // VirtualMachine(s)
                ArrayList<Integer> vmIdToDelete = new ArrayList<Integer>();
                for (VirtualMachine vm : siTemp.getVirtualMachines()) {
                    vmIdToDelete.add(vm.getId());
                }

                for (Integer vmId : vmIdToDelete) {
                    VirtualMachine vm = null;
                    vm = this.em.find(VirtualMachine.class, vmId);

                    VirtualMachineTemplate vmTemplate = vm.getVirtualMachineTemplate();
                    vmTemplate.getVirtualMachines().remove(vm);
                    vm.setVirtualMachineTemplate(null);

                    siTemp.getVirtualMachines().remove(vm);
                    vm.setSystemInstance(null);

                    this.em.remove(vm);
                    this.em.flush();
                }

                // Clean up (i.e., delete) this SystemInstance itself.
                this.em.remove(siTemp);
                this.em.flush();
            }
        }

        SystemInstanceManagerBean.log.finest("Leaves: cleanUpDestroyedSystemInstances(...)");
    }
}