/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: VMImageManagerBean.java 372 2011-09-30 14:49:32Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.service;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.Query;

import org.ow2.easybeans.osgi.annotation.OSGiResource;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidArgumentException;
import org.ow2.sirocco.cloudmanager.admin.exception.VMImageWithTheGivenURLAlreadyExistsException;
import org.ow2.sirocco.cloudmanager.admin.vo.VMImageVO;
import org.ow2.sirocco.cloudmanager.business.VMImage;
import org.ow2.sirocco.cloudmanager.clustermanager.api.IClusterManager;
import org.ow2.sirocco.vmm.api.VMMException;

@Stateless(name = "VMImageManagerBean", mappedName = "VMImageManagerBean")
@Remote(IRemoteVMImageManager.class)
@Local(IVMImageManager.class)
public class VMImageManagerBean implements IVMImageManager {

    private static Logger log = Logger.getLogger(VMImageManagerBean.class.getName());

    @PersistenceContext(unitName = "persistence-unit/main", type = PersistenceContextType.TRANSACTION)
    protected EntityManager em;

    // XXX Use cloudProvider (instead of clusterManager) when released.
    @OSGiResource
    private IClusterManager clusterManager;

    @Override
    public void uploadVirtualMachineImageFile(final String uRL, final String format, final String name,
        final String description, final String checkSum, final Integer oVFSizeInMB, final String cloudProviderId,
        final Boolean creationViaASystemTemplate) throws VMMException, VMImageWithTheGivenURLAlreadyExistsException,
        InvalidArgumentException {
        VMImageManagerBean.log.finest("Enters in: uploadVirtualMachineImageFile(uRL=" + uRL + ", format=" + format + ", name="
            + name + ", description=" + description + ", checkSum=" + checkSum + ", cloudProviderId=" + cloudProviderId + ")");

        // XXX we reject the upload from an already known URL, but do
        // we also reject the upload of an already known name, etc. ?

        if (creationViaASystemTemplate == null) {
            throw new InvalidArgumentException("The given creationViaASystemTemplate is " + creationViaASystemTemplate + ".");
        } else if (creationViaASystemTemplate) {
            // nothing to do.
        } else {
            // Check if a vmimage with the given uRL is already known.
            Query vMImage_temp_list_query = this.em.createQuery(
                "FROM " + VMImage.class.getSimpleName() + " v WHERE v.uploadedFromURL=:uploadedFromURL").setParameter(
                "uploadedFromURL", uRL);
            if (vMImage_temp_list_query == null) {
                // nothing to do.
            } else {
                if (vMImage_temp_list_query.getResultList().isEmpty()) {
                    // nothing to do.
                } else {
                    throw new VMImageWithTheGivenURLAlreadyExistsException("A VMImage with " + uRL + " as URL already exists.");
                }
            }

            // Creation of the corresponding VMImage via vMImage_temp variable.
            VMImage vMImage_temp = new VMImage();
            vMImage_temp.setUploadedFromURL(uRL);
            vMImage_temp.setStatus(VMImage.Status.IN_CONSTRUCTION);
            vMImage_temp.setDescription(description);
            vMImage_temp.setVMImageFormat(format);
            vMImage_temp.setName(name);
            vMImage_temp.setImageSizeMB(oVFSizeInMB);
            // XXX setDiskSizeMB is used like in CloudAdminBean
            // : createAndStartVirtualMachine.
            vMImage_temp.setDiskSizeMB(oVFSizeInMB);

            // TODO vMImage_temp.setUser(this.
            // userManager.getUserByUsername(fromUsername));
            // vMImage_temp.setProject(proj);

            // XXX vMImage_temp.setHypervisor(vs.
            // getVmImage().getHypervisor());
            // vMImage_temp.setArchitecture(vs.getVmImage().getArchitecture());
            // vMImage_temp.setOsType(vs.getVmImage().getOsType());
            // vMImage_temp.setVisibility(visibility);
            // vMImage_temp.setIsBootable(isBootable);

            // The following line also persists volumeTemplate_temp.
            this.em.persist(vMImage_temp);
            this.em.flush();
        }

        // XXX cloudProviderId should be used in order to select the
        // targeted cloud provider.

        // Note that: description, and checksum arguments aren't currently used
        // by clusterManager.uploadVMImage(...);. Upload returns a String that
        // must be used as providerId when creating a (entityBean) VMImage.

        // XXX For testing purpose, use dummy for format; others parameters
        // aren't important.
        this.clusterManager.uploadVMImage(uRL, format, name, description, checkSum);

        VMImageManagerBean.log.finest("Leaves: uploadVirtualMachineImageFile(...)");
    }

    @Override
    public List<VMImageVO> getVMImageFiles() {
        List<VMImageVO> result = new ArrayList<VMImageVO>();
        @SuppressWarnings("unchecked")
        List<VMImage> list = this.em.createQuery("FROM " + VMImage.class.getSimpleName() + " v").getResultList();
        Iterator<VMImage> vMImage_temp_ITERATOR = list.iterator();
        while (vMImage_temp_ITERATOR.hasNext()) {
            VMImage vMImage_temp = vMImage_temp_ITERATOR.next();
            VMImageVO vMImageVO = VMImageVO.from(vMImage_temp);
            result.add(vMImageVO);
        }
        return result;
    }

}
