/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id$
 *
 */

package org.ow2.sirocco.cloudmanager.service.monitoring.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.ejb.EJB;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;

import org.ow2.easybeans.osgi.annotation.OSGiResource;
import org.ow2.sirocco.cloudmanager.business.Project;
import org.ow2.sirocco.cloudmanager.business.User;
import org.ow2.sirocco.cloudmanager.business.VirtualMachine;
import org.ow2.sirocco.cloudmanager.clustermanager.api.IClusterManager;
import org.ow2.sirocco.cloudmanager.service.ICloudAdmin;
import org.ow2.sirocco.cloudmanager.service.IUserManager;
import org.ow2.sirocco.cloudmanager.service.monitoring.api.IMonitoringManager;
import org.ow2.sirocco.vmm.api.monitoring.PerfMetricInfo.Unit;

@Stateless(name = "MonitoringManagerBean", mappedName = "MonitoringManagerBean")
@Remote(IMonitoringManager.class)
public class MonitoringManagerBean implements IMonitoringManager {

    @OSGiResource
    private IClusterManager clusterManager;

    @EJB(mappedName = "CloudAdminBean")
    ICloudAdmin cloudAdmin;

    @EJB(mappedName = "UserManagerBean")
    private IUserManager userManager;

    @PersistenceContext(unitName = "persistence-unit/main", type = PersistenceContextType.TRANSACTION)
    private EntityManager em = null;

    @Override
    public List<PerfMetricInfoVO> listPerfMetricInfos(final Entity type, final String id) {
        List<PerfMetricInfoVO> result = new ArrayList<PerfMetricInfoVO>();
        org.ow2.sirocco.cloudmanager.clustermanager.api.PerfMetricInfo[] result_tmp = null;
        if (type.equals(IMonitoringManager.Entity.HOST)) {
            result_tmp = this.clusterManager.getHostPerfMetricInfos();
        } else if (type.equals(IMonitoringManager.Entity.SERVERPOOL)) {
            result_tmp = this.clusterManager.getServerPoolPerfMetricInfos();
        } else if (type.equals(IMonitoringManager.Entity.DOMAIN)) {
            result_tmp = this.clusterManager.getDomainPerfMetricInfos();
        } else if (type.equals(IMonitoringManager.Entity.VIRTUALMACHINE)) {
            @SuppressWarnings("unchecked")
            VirtualMachine v = this.em.find(VirtualMachine.class, Integer.valueOf(id));
            ArrayList<PerfMetricInfo> parray = new ArrayList<PerfMetricInfo>(v.getPerfMetricInfos());
            if (parray != null) {
                for (PerfMetricInfo pmi : parray) {
                    result.add(this.fromPerfMetricInfoToPerfMetricInfoValueObject(pmi));
                }
            }
            return result;
        } else if (type.equals(IMonitoringManager.Entity.USER)) {
            result_tmp = this.clusterManager.getUserPerfMetricInfos(id);
        } else if (type.equals(IMonitoringManager.Entity.PROJECT)) {
            result_tmp = this.clusterManager.getProjectPerfMetricInfos(id);
        }
        if (result_tmp != null) {
            for (org.ow2.sirocco.cloudmanager.clustermanager.api.PerfMetricInfo pmi : result_tmp) {
                PerfMetricInfoVO pmitmp = new PerfMetricInfoVO();
                pmitmp.setId(pmi.getId());
                pmitmp.setName(pmi.getName());
                pmitmp.setUnit(pmi.getUnit().value());
                pmitmp.setDescription(pmi.getDescription());
                pmitmp.setTypeSample(pmi.getType());
                pmitmp.setIntervalSample(pmi.getInterval());
                result.add(pmitmp);
            }
        }
        return result;
    }

    private PerfMetricInfoVO fromPerfMetricInfoToPerfMetricInfoValueObject(final PerfMetricInfo p) {
        PerfMetricInfoVO result = new PerfMetricInfoVO();
        result.setId(p.getId());
        result.setDescription(p.getDescription());
        result.setIntervalSample(p.getIntervalSample());
        result.setName(p.getName());
        result.setTypeSample(p.getTypeSample());
        result.setUnit(p.getUnit());
        return result;
    }

    private PerfMetricVO fromPerfMetricToPerfMetricValueObject(final PerfMetric p) {
        PerfMetricVO result = new PerfMetricVO();
        result.setId(p.getId());
        result.setTimeStamp(p.getTimeStamp());
        result.setValue(p.getValue());
        return result;
    }

    @Override
    public PerfMetricVO getPerfMetric(final Entity type, final String id, final PerfMetricInfoVO info) {
        org.ow2.sirocco.vmm.api.monitoring.PerfMetric result = null;
        org.ow2.sirocco.cloudmanager.clustermanager.api.PerfMetricInfo pmitmp = new org.ow2.sirocco.cloudmanager.clustermanager.api.PerfMetricInfo();

        pmitmp.setName(info.getName());
        pmitmp.setUnit(Unit.fromValue(info.getUnit()));
        pmitmp.setDescription(info.getDescription());
        pmitmp.setType(info.getTypeSample());
        pmitmp.setInterval(info.getIntervalSample());

        if (type.equals(IMonitoringManager.Entity.HOST)) {
            result = this.clusterManager.getHostPerfMetric(id, pmitmp);
        } else if (type.equals(IMonitoringManager.Entity.SERVERPOOL)) {
            result = this.clusterManager.getServerPoolPerfMetric(id, pmitmp);
        } else if (type.equals(IMonitoringManager.Entity.DOMAIN)) {
            result = this.clusterManager.getDomainPerfMetric(id, pmitmp);
        } else if (type.equals(IMonitoringManager.Entity.VIRTUALMACHINE)) {
            VirtualMachine v = this.em.find(VirtualMachine.class, Integer.valueOf(id));
            result = this.clusterManager.getVirtualMachinePerfMetric(v.getProviderId(), pmitmp);
        } else if (type.equals(IMonitoringManager.Entity.USER)) {
            User u = this.userManager.getUserByUsername(id);
            @SuppressWarnings("unchecked")
            List<VirtualMachine> vms = this.em
                .createQuery(
                    "FROM VirtualMachine v WHERE v.user=:user_id AND v.status<>'DESTROYED' AND v.status<>'IN_CONFIGURATION'")
                .setParameter("user_id", u).getResultList();
            if (vms != null) {
                float res_value = 0;
                for (VirtualMachine vm : vms) {
                    result = this.clusterManager.getVirtualMachinePerfMetric(vm.getProviderId(), pmitmp);
                    if (result != null) {
                        res_value += result.getValue();
                    }
                }
                if (pmitmp.getUnit().equals(Unit.PERCENT) && vms.size() > 0) {
                    res_value = res_value / vms.size();
                }
                if (result != null) {
                    result.setValue(res_value);
                }
            }
        } else if (type.equals(IMonitoringManager.Entity.PROJECT)) {
            Project p = this.em.find(Project.class, Integer.valueOf(id));
            @SuppressWarnings("unchecked")
            List<VirtualMachine> vms = this.em
                .createQuery(
                    "FROM VirtualMachine v WHERE v.project=:project_id AND v.status<>'DESTROYED' AND v.status<>'IN_CONFIGURATION'")
                .setParameter("project_id", p).getResultList();
            if (vms != null) {
                float res_value = 0;
                for (VirtualMachine vm : vms) {
                    result = this.clusterManager.getVirtualMachinePerfMetric(vm.getProviderId(), pmitmp);
                    if (result != null) {
                        res_value += result.getValue();
                    }
                }
                if (pmitmp.getUnit().equals(Unit.PERCENT) && vms.size() > 0) {
                    res_value = res_value / vms.size();
                }
                if (result != null) {
                    result.setValue(res_value);
                }
            }
        }
        PerfMetricVO pmtmp = new PerfMetricVO();
        if (result != null) {
            pmtmp.setValue(result.getValue());
            pmtmp.setTimeStamp(new Date(result.getTime()));
        }
        return pmtmp;
    }

    public List<PerfMetricVO> getPerfMetrics(final Entity type, final String id, final PerfMetricInfoVO info,
        final Date startTime, final Date endTime) {
        List<PerfMetricVO> res = new ArrayList<PerfMetricVO>();
        org.ow2.sirocco.vmm.api.monitoring.PerfMetric[] result = null;
        org.ow2.sirocco.cloudmanager.clustermanager.api.PerfMetricInfo pmitmp = new org.ow2.sirocco.cloudmanager.clustermanager.api.PerfMetricInfo();

        pmitmp.setName(info.getName());
        pmitmp.setUnit(Unit.fromValue(info.getUnit()));
        pmitmp.setDescription(info.getDescription());
        pmitmp.setType(info.getTypeSample());
        pmitmp.setInterval(info.getIntervalSample());

        if (type.equals(IMonitoringManager.Entity.HOST)) {
            result = this.clusterManager.getHostPerfMetrics(id, pmitmp, startTime, endTime);
        } else if (type.equals(IMonitoringManager.Entity.SERVERPOOL)) {
            result = this.clusterManager.getServerPoolPerfMetrics(id, pmitmp, startTime, endTime);
        } else if (type.equals(IMonitoringManager.Entity.DOMAIN)) {
            result = this.clusterManager.getDomainPerfMetrics(id, pmitmp, startTime, endTime);
        } else if (type.equals(IMonitoringManager.Entity.VIRTUALMACHINE)) {

            VirtualMachine v = this.em.find(VirtualMachine.class, Integer.valueOf(id));

            ArrayList<PerfMetricInfo> infos = new ArrayList<PerfMetricInfo>(v.getPerfMetricInfos());

            if (infos != null) {
                for (PerfMetricInfo infotmp : infos) {
                    if (infotmp.getName().equals(info.getName()) && infotmp.getTypeSample().equals(info.getTypeSample())
                        && infotmp.getIntervalSample().equals(info.getIntervalSample())) {
                        ArrayList<PerfMetric> pms = new ArrayList<PerfMetric>(infotmp.getMetricsbetween(startTime, endTime));
                        for (PerfMetric pm : pms) {
                            res.add(this.fromPerfMetricToPerfMetricValueObject(pm));
                        }
                    }
                }
                return res;
            }
        } else if (type.equals(IMonitoringManager.Entity.USER)) {
            if (info != null) {
                User u = this.userManager.getUserByUsername(id);

                @SuppressWarnings("unchecked")
                List<VirtualMachine> vms = this.em
                    .createQuery("FROM VirtualMachine v WHERE v.user=:user_id AND v.status='RUNNING'")
                    .setParameter("user_id", u).getResultList();
                if (vms != null) {

                    int cpt = 0;
                    int cptj = 0;

                    List<PerfMetricVO> tmp = new ArrayList<PerfMetricVO>();
                    res = this.getPerfMetrics(Entity.VIRTUALMACHINE, vms.get(0).getId().toString(), info, startTime, endTime);

                    // on parcourt ts les vms
                    for (cpt = 1; cpt < vms.size(); cpt++) {
                        tmp = this.getPerfMetrics(Entity.VIRTUALMACHINE, vms.get(cpt).getId().toString(), info, startTime,
                            endTime);
                        // on parcourt ts les pms
                        for (PerfMetricVO pVO : tmp) {
                            for (int j = cptj; j < res.size(); j++) {
                                if (res.get(j).getTimeStamp().equals(pVO)) {
                                    res.get(j).setValue(pVO.getValue() + res.get(j).getValue());
                                    if (info.getUnit().equals("%")) {
                                        res.get(j).setValue(res.get(j).getValue() / 2);
                                    }
                                    cptj = j + 1;
                                    break;
                                }
                            }
                        }
                    }
                    return res;
                }

            }
            // return res;
        } else if (type.equals(IMonitoringManager.Entity.PROJECT)) {
            Project p = this.em.find(Project.class, Integer.valueOf(id));

            if (p != null) {

                List<VirtualMachine> vms = new ArrayList<VirtualMachine>(p.getVirtualMachines());

                int cpt = 0;
                int cptj = 0;

                List<PerfMetricVO> tmp = new ArrayList<PerfMetricVO>();
                res = this.getPerfMetrics(Entity.VIRTUALMACHINE, vms.get(0).getId().toString(), info, startTime, endTime);

                for (cpt = 1; cpt < vms.size(); cpt++) {
                    tmp = this.getPerfMetrics(Entity.VIRTUALMACHINE, vms.get(cpt).getId().toString(), info, startTime, endTime);
                    for (PerfMetricVO pVO : tmp) {
                        for (int j = cptj; j < res.size(); j++) {
                            if (res.get(j).getTimeStamp().equals(pVO)) {
                                res.get(j).setValue(pVO.getValue() + res.get(j).getValue());
                                if (info.getUnit().equals("%")) {
                                    res.get(j).setValue(res.get(j).getValue() / 2);
                                }
                                cptj = j + 1;
                                break;
                            }
                        }
                    }
                }
                return res;
            }
        }

        if (result != null) {
            for (org.ow2.sirocco.vmm.api.monitoring.PerfMetric pm : result) {
                PerfMetricVO pmtmp = new PerfMetricVO();
                pmtmp.setTimeStamp(new Date(pm.getTime()));
                pmtmp.setValue(pm.getValue());
                res.add(pmtmp);
            }
        }
        return res;
    }

    @Override
    public PerfMetricInfoVO getPerfMetricInfoById(final String id, final Entity type) {
        if (type.equals(IMonitoringManager.Entity.VIRTUALMACHINE)) {
            PerfMetricInfo info = this.em.find(PerfMetricInfo.class, Integer.valueOf(id));
            if (info != null) {
                return this.fromPerfMetricInfoToPerfMetricInfoValueObject(info);
            }
        } else {
            org.ow2.sirocco.cloudmanager.clustermanager.api.PerfMetricInfo info = this.clusterManager.getPerfMetricInfoById(
                Integer.valueOf(id), type.toString());
            PerfMetricInfoVO pmiVO = new PerfMetricInfoVO();
            pmiVO.setId(info.getId());
            pmiVO.setName(info.getName());
            pmiVO.setUnit(info.getUnit().value());
            pmiVO.setDescription(info.getDescription());
            pmiVO.setTypeSample(info.getType());
            pmiVO.setIntervalSample(info.getInterval());
            return pmiVO;
        }
        return null;
    }
}
