/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id$
 *
 */

package org.ow2.sirocco.cloudmanager.service.monitoring.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;

@Entity
public class PerfMetricInfo implements Serializable {

    /**
     * 
     */
    private static final long serialVersionUID = 1L;

    private Integer id;

    private String name;

    private String description;

    private String unit;

    private String typeSample; // value's type (average, max, min)

    private Long intervalSample;

    private Collection<PerfMetric> metrics;

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    public Integer getId() {
        return this.id;
    }

    public void setId(final Integer id) {
        this.id = id;
    }

    public String getName() {
        return this.name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    public String getDescription() {
        return this.description;
    }

    public void setDescription(final String description) {
        this.description = description;
    }

    public String getUnit() {
        return this.unit;
    }

    public void setUnit(final String unit) {
        this.unit = unit;
    }

    public static long getSerialversionuid() {
        return PerfMetricInfo.serialVersionUID;
    }

    public void setMetrics(final Collection<PerfMetric> metrics) {
        this.metrics = metrics;
    }

    @OneToMany(mappedBy = "info", cascade = CascadeType.ALL)
    public Collection<PerfMetric> getMetrics() {
        return this.metrics;
    }

    public Collection<PerfMetric> getMetricsbetween(final Date startTime, final Date endTime) {
        Collection<PerfMetric> res = new ArrayList<PerfMetric>();
        for (PerfMetric p : this.metrics) {
            if (p.getTimeStamp().after(startTime) && p.getTimeStamp().before(endTime)) {
                res.add(p);
            }
        }
        return res;
    }

    public boolean addPerfMetric(final PerfMetric pm) {
        // add the metric only if the timestamp is after the timestamp of the
        // last metric saved
        ArrayList<PerfMetric> arrays = new ArrayList<PerfMetric>(this.metrics);
        if (arrays == null) {
            System.out.println("PMI: array Null");
            return false;
        }
        if (arrays.size() == 0) {
            System.out.println("PMI: taille=0, save OK");
            this.metrics.add(pm);
            return true;
        }
        if ((arrays.get(this.metrics.size() - 1)).getTimeStamp().compareTo(pm.getTimeStamp()) >= 0) {
            System.out.println("PMI: <=");
            return false;
        }
        System.out.println("PMI: save OK");
        this.metrics.add(pm);
        return true;
    }

    public void setIntervalSample(final Long intervalSample) {
        this.intervalSample = intervalSample;
    }

    public Long getIntervalSample() {
        return this.intervalSample;
    }

    public void setTypeSample(final String typeSample) {
        this.typeSample = typeSample;
    }

    public String getTypeSample() {
        return this.typeSample;
    }

}
