/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: ResourcesPoolOverallPlanning.java 87 2011-06-30 15:07:34Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.service.reservation.impl;

import java.util.Hashtable;

import org.ow2.sirocco.vmm.api.ResourcePartitionMXBean;

public class ResourcesPoolOverallPlanning {

    private ResourcePartitionMXBean resourcePartitionMXBean;

    private String resourcePoolHypervisorType = null;

    private String resourcePoolLocation = null;

    private Long resourcesPoolTotalCPUCapacity = null;

    private Long resourcesPoolTotalMemoryCapacity = null;

    private Long resourcesPoolTotalStorageCapacity = null;

    /**
     * Contains the timeUnit set. Each timeUnit is expressed w.r.t. the
     * TimeUnitGranularityType defined for the module. Each timeUnit is
     * associated to several information (the available resources in the pool,
     * the projects' id having a reservation for this timeUnit, the
     * number/amount of resources available in the project, and the
     * reservationsId of the reservation taken for/in this timeUnit).
     */
    private Hashtable<Long, ResourcesPoolPlanningByTimeUnit> resourcesReservationPlanning = null;

    /**
     * @param resourcePartitionMXBean
     * @param resourcePoolHypervisorType
     * @param resourcePoolLocation
     */
    public ResourcesPoolOverallPlanning(final ResourcePartitionMXBean resourcePartitionMXBean,
        final String resourcePoolHypervisorType, final String resourcePoolLocation) {
        this.resourcePartitionMXBean = resourcePartitionMXBean;
        this.resourcePoolHypervisorType = resourcePoolHypervisorType;
        this.resourcePoolLocation = resourcePoolLocation;
        this.resourcesPoolTotalCPUCapacity = this.resourcePartitionMXBean.getCpuCoreCapacity();
        this.resourcesPoolTotalMemoryCapacity = this.resourcePartitionMXBean.getMemoryCapacityMB();
        this.resourcesPoolTotalStorageCapacity = this.resourcePartitionMXBean.getStorageCapacityMB();
        this.resourcesReservationPlanning = new Hashtable<Long, ResourcesPoolPlanningByTimeUnit>();
    }

    public ResourcePartitionMXBean getResourcePartitionMXBean() {
        return this.resourcePartitionMXBean;
    }

    // public void setResourcePartitionMXBean(final ResourcePartitionMXBean
    // resourcePartitionMXBean) {
    // this.resourcePartitionMXBean = resourcePartitionMXBean;
    // }

    public final String getResourcePoolHypervisorType() {
        return this.resourcePoolHypervisorType;
    }

    // public final void setResourcePoolHypervisorType(String
    // resourcePoolHypervisorType) {
    // this.resourcePoolHypervisorType = resourcePoolHypervisorType;
    // }

    public final String getResourcePoolLocation() {
        return this.resourcePoolLocation;
    }

    // public final void setResourcePoolLocation(String resourcePoolLocation) {
    // this.resourcePoolLocation = resourcePoolLocation;
    // }

    public final Long getResourcesPoolTotalCPUCapacity() {
        return this.resourcesPoolTotalCPUCapacity;
    }

    public final Long getResourcesPoolTotalMemoryCapacity() {
        return this.resourcesPoolTotalMemoryCapacity;
    }

    public final Long getResourcesPoolTotalStorageCapacity() {
        return this.resourcesPoolTotalStorageCapacity;
    }

    public final Hashtable<Long, ResourcesPoolPlanningByTimeUnit> getResourcesReservationPlanning() {
        return this.resourcesReservationPlanning;
    }

    @Override
    public String toString() {
        return this.getClass().getName() + " [resourcePartitionMXBean=" + this.getResourcePartitionMXBean()
            + " resourcePoolHypervisorType=" + this.getResourcePoolHypervisorType() + " getResourcePoolLocation="
            + this.getResourcePoolLocation() + " resourcesPoolTotalCPUCapacity=" + this.getResourcesPoolTotalCPUCapacity()
            + " resourcesPoolTotalMemoryCapacity=" + this.getResourcesPoolTotalMemoryCapacity()
            + " resourcesPoolTotalStorageCapacity=" + this.getResourcesPoolTotalStorageCapacity()
            + " resourcesReservationPlanning=" + this.getResourcesReservationPlanning() + "]";
    }

}
