/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: ResourcesReservation.java 220 2011-08-01 13:13:33Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.service.reservation.impl;

import java.io.Serializable;
import java.util.Calendar;

import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;

import org.ow2.sirocco.cloudmanager.service.reservation.api.ResourcesReservationState;

@SuppressWarnings("serial")
@Entity
public class ResourcesReservation implements Serializable {

    // Warning "key" can't be an attribut --> this is a reserved attribut !

    /**
     * C'est le reservation Id
     */
    private int id;

    private ResourcesReservationState resourcesReservationState;

    private String projectId;

    /**
     * startTime is a real Calendar object. The one given by the user.
     */
    private Calendar startTime;

    /**
     * endTime is a real Calendar object. The one given by the user.
     */
    private Calendar endTime;

    /**
     * Number of CPU core(s) reserved.
     */
    private Integer reservedNumberOfCPUCore;

    /**
     * Memeory size in MB/Mo reserved.
     */
    private Integer reservedMemorySizeInMB;

    /**
     * Storage size in MB/Mo reserved.
     */
    private Integer reservedStorageSizeInMB;

    /**
     * Remaining number of CPU Cores.
     */
    private Integer freeNumberOfCpuCores;

    /**
     * Remaining memory in MB.
     */
    private Integer freeMemorySizeInMB;

    /**
     * Remaining storage in MB.
     */
    private Integer freeStorageSizeInMB;

    private String hypervisorType;

    private String location;

    public ResourcesReservation() {
        // This contructor is required by JPA when using non-empty
        // contructor(s).
    }

    /**
     * this.id is automatically generated.
     */
    public ResourcesReservation(final ResourcesReservationState resourcesReservationState, final String projectId,
        final Calendar startTime, final Calendar endTime, final Integer reservedNumberOfCPUCore,
        final Integer reservedMemorySizeInMB, final Integer reservedStorageSizeInMB, final Integer freeNumberOfCpuCores,
        final Integer freeMemorySizeInMB, final Integer freeStorageSizeInMB, final String hypervisorType, final String location) {
        super();

        // this.id is automatically generated.

        this.setResourcesReservationState(resourcesReservationState);
        this.setProjectId(projectId);
        this.setStartTime(startTime);
        this.setEndTime(endTime);
        this.setReservedNumberOfCPUCore(reservedNumberOfCPUCore);
        this.setReservedMemorySizeInMB(reservedMemorySizeInMB);
        this.setReservedStorageSizeInMB(reservedStorageSizeInMB);

        this.setFreeNumberOfCpuCores(freeNumberOfCpuCores);
        this.setFreeMemorySizeInMB(freeMemorySizeInMB);
        this.setFreeStorageSizeInMB(freeStorageSizeInMB);

        this.setHypervisorType(hypervisorType);
        this.setLocation(location);
    }

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    public int getId() {
        return this.id;
    }

    public void setId(final int id) {
        this.id = id;
    }

    @Enumerated(EnumType.STRING)
    public ResourcesReservationState getResourcesReservationState() {
        return this.resourcesReservationState;
    }

    public void setResourcesReservationState(final ResourcesReservationState resourcesReservationState) {
        this.resourcesReservationState = resourcesReservationState;
    }

    public String getProjectId() {
        return this.projectId;
    }

    public void setProjectId(final String projectId) {
        this.projectId = projectId;
    }

    public Calendar getStartTime() {
        return this.startTime;
    }

    public void setStartTime(final Calendar startTime) {
        this.startTime = startTime;
    }

    public Calendar getEndTime() {
        return this.endTime;
    }

    public void setEndTime(final Calendar endTime) {
        this.endTime = endTime;
    }

    public Integer getReservedNumberOfCPUCore() {
        return this.reservedNumberOfCPUCore;
    }

    public void setReservedNumberOfCPUCore(final Integer reservedNumberOfCPUCore) {
        this.reservedNumberOfCPUCore = reservedNumberOfCPUCore;
    }

    public Integer getReservedMemorySizeInMB() {
        return this.reservedMemorySizeInMB;
    }

    public void setReservedMemorySizeInMB(final Integer reservedMemorySizeInMB) {
        this.reservedMemorySizeInMB = reservedMemorySizeInMB;
    }

    public Integer getReservedStorageSizeInMB() {
        return this.reservedStorageSizeInMB;
    }

    public void setReservedStorageSizeInMB(final Integer reservedStorageSizeInMB) {
        this.reservedStorageSizeInMB = reservedStorageSizeInMB;
    }

    public Integer getFreeNumberOfCpuCores() {
        return this.freeNumberOfCpuCores;
    }

    public void setFreeNumberOfCpuCores(final Integer freeNumberOfCpuCores) {
        this.freeNumberOfCpuCores = freeNumberOfCpuCores;
    }

    public Integer getFreeMemorySizeInMB() {
        return this.freeMemorySizeInMB;
    }

    public void setFreeMemorySizeInMB(final Integer freeMemorySizeInMB) {
        this.freeMemorySizeInMB = freeMemorySizeInMB;
    }

    public Integer getFreeStorageSizeInMB() {
        return this.freeStorageSizeInMB;
    }

    public void setFreeStorageSizeInMB(final Integer freeStorageSizeInMB) {
        this.freeStorageSizeInMB = freeStorageSizeInMB;
    }

    public String getHypervisorType() {
        return this.hypervisorType;
    }

    public void setHypervisorType(final String hypervisorType) {
        this.hypervisorType = hypervisorType;
    }

    public String getLocation() {
        return this.location;
    }

    public void setLocation(final String location) {
        this.location = location;
    }

}
