/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: ResourcesReservationVO.java 87 2011-06-30 15:07:34Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.service.reservation.impl;

import java.io.Serializable;
import java.util.Calendar;

import org.ow2.sirocco.cloudmanager.service.reservation.api.ResourcesReservationState;

@SuppressWarnings("serial")
public class ResourcesReservationVO implements Serializable {

    /**
     * This is the reservation Id.
     */
    private int id;

    private ResourcesReservationState resourcesReservationState;

    private String projectId;

    /**
     * startTime is a real Calendar object. The one given by the user.
     */
    private Calendar startTime;

    /**
     * endTime is a real Calendar object. The one given by the user.
     */
    private Calendar endTime;

    /**
     * Number of CPU core(s) reserved.
     */
    private Integer reservedNumberOfCPUCore;

    /**
     * Memeory size in MB/Mo reserved.
     */
    private Integer reservedMemorySizeInMB;

    /**
     * Storage size in MB/Mo reserved.
     */
    private Integer reservedStorageSizeInMB;

    /**
     * Remaining number of CPU Cores.
     */
    private Integer freeNumberOfCpuCores;

    /**
     * Remaining memory in MB.
     */
    private Integer freeMemorySizeInMB;

    /**
     * Remaining storage in MB.
     */
    private Integer freeStorageSizeInMB;

    private String hypervisorType;

    private String location;

    public ResourcesReservationVO(final int id, final ResourcesReservationState resourcesReservationState,
        final String projectId, final Calendar startTime, final Calendar endTime, final Integer reservedNumberOfCPUCore,
        final Integer reservedMemorySizeInMB, final Integer reservedStorageSizeInMB, final Integer freeNumberOfCpuCores,
        final Integer freeMemorySizeInMB, final Integer freeStorageSizeInMB, final String hypervisorType, final String location) {
        this.id = id;
        this.resourcesReservationState = resourcesReservationState;
        this.projectId = projectId;
        this.startTime = startTime;
        this.endTime = endTime;
        this.reservedNumberOfCPUCore = reservedNumberOfCPUCore;
        this.reservedMemorySizeInMB = reservedMemorySizeInMB;
        this.reservedStorageSizeInMB = reservedStorageSizeInMB;
        this.freeNumberOfCpuCores = freeNumberOfCpuCores;
        this.freeMemorySizeInMB = freeMemorySizeInMB;
        this.freeStorageSizeInMB = freeStorageSizeInMB;
        this.hypervisorType = hypervisorType;
        this.location = location;
    }

    public final int getId() {
        return this.id;
    }

    public final ResourcesReservationState getResourcesReservationState() {
        return this.resourcesReservationState;
    }

    public final String getProjectId() {
        return this.projectId;
    }

    public final Calendar getStartTime() {
        return this.startTime;
    }

    public final Calendar getEndTime() {
        return this.endTime;
    }

    public final Integer getReservedNumberOfCPUCore() {
        return this.reservedNumberOfCPUCore;
    }

    public final Integer getReservedMemorySizeInMB() {
        return this.reservedMemorySizeInMB;
    }

    public final Integer getReservedStorageSizeInMB() {
        return this.reservedStorageSizeInMB;
    }

    public final Integer getFreeNumberOfCpuCores() {
        return this.freeNumberOfCpuCores;
    }

    public final Integer getFreeMemorySizeInMB() {
        return this.freeMemorySizeInMB;
    }

    public final Integer getFreeStorageSizeInMB() {
        return this.freeStorageSizeInMB;
    }

    public final String getHypervisorType() {
        return this.hypervisorType;
    }

    public final String getLocation() {
        return this.location;
    }

    @Override
    public String toString() {
        return this.getClass().getName() + " [id=" + this.getId() + " resourcesReservationState="
            + this.getResourcesReservationState() + " projectId=" + this.getProjectId() + " startTime=" + this.getStartTime()
            + " endTime=" + this.getEndTime() + " reservedNumberOfCPUCore=" + this.getReservedNumberOfCPUCore()
            + " reservedMemorySizeInMB=" + this.getReservedMemorySizeInMB() + " reservedStorageSizeInMB="
            + this.getReservedStorageSizeInMB() + " freeNumberOfCpuCores=" + this.getFreeNumberOfCpuCores()
            + " freeMemorySizeInMB=" + this.getFreeMemorySizeInMB() + " freeStorageSizeInMB=" + this.getFreeStorageSizeInMB()
            + " hypervisorType=" + this.getHypervisorType() + " location=" + this.getLocation() + "]";
    }
}
