/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: PermissionChecker.java 318 2011-09-09 12:20:59Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.utils;

import org.ow2.sirocco.cloudmanager.business.Project;
import org.ow2.sirocco.cloudmanager.business.RoleAssignment.Rights;
import org.ow2.sirocco.cloudmanager.business.User;
import org.ow2.sirocco.cloudmanager.business.VMImage;
import org.ow2.sirocco.cloudmanager.business.VirtualMachine;
import org.ow2.sirocco.cloudmanager.business.Volume;
import org.ow2.sirocco.cloudmanager.common.jndilocator.ServiceLocator;
import org.ow2.sirocco.cloudmanager.service.IRemoteProjectManager;

public class PermissionChecker {

    public PermissionChecker() {
        this.initialize();
    }

    private transient IRemoteProjectManager projectManager;

    private void initialize() {
        this.projectManager = (IRemoteProjectManager) ServiceLocator.getInstance().getRemoteObject("ProjectManagerBean");
    }

    public boolean canCreateVM(final User user, final Project project) {
        if (user.getAdmin()) {
            return true;
        } else if (this.projectManager.checkIfUsernameInProjectId(user.getUsername(), project.getProjectId())) {
            return true;
        } else {
            return false;
        }
    }

    public boolean canStartVm(final User user, final Project project) {
        if (user.getAdmin()) {
            return true;
        } else if (this.projectManager.checkIfUsernameInProjectId(user.getUsername(), project.getProjectId())) {
            return true;
        } else {
            return false;
        }
    }

    public boolean canCreateVolume(final User user, final Project project) {
        if (user.getAdmin()) {
            return true;
        } else if (this.projectManager.checkIfUsernameInProjectId(user.getUsername(), project.getProjectId())) {
            return true;
        } else {
            return false;
        }
    }

    public boolean canStopVm(final User user, final VirtualMachine vm) {
        if (user.getAdmin()) {
            return true;
        }
        if (user.equals(vm.getUser())) {
            return true;
        }
        if (this.projectManager.getRightsForUsernameInProjectId(user.getUsername(), vm.getProject().getProjectId()).equals(
            Rights.ADMIN.toString())) {
            return true;
        }
        return false;
    }

    public boolean canPauseVM(final User fromUser, final VirtualMachine vm) {
        if (fromUser.getAdmin()) {
            return true;
        }
        if (fromUser.equals(vm.getUser())) {
            return true;
        }
        if (this.projectManager.getRightsForUsernameInProjectId(fromUser.getUsername(), vm.getProject().getProjectId()).equals(
            Rights.ADMIN.toString())) {
            return true;
        }
        return false;
    }

    public boolean canDestroyVM(final User fromUser, final VirtualMachine vm) {
        if (fromUser.getAdmin()) {
            return true;
        }
        if (fromUser.equals(vm.getUser())) {
            return true;
        }
        if (this.projectManager.getRightsForUsernameInProjectId(fromUser.getUsername(), vm.getProject().getProjectId()).equals(
            Rights.ADMIN.toString())) {
            return true;
        }
        return false;
    }

    public boolean canDestroyVolume(final User fromUser, final Volume vol) {
        if (fromUser.getAdmin()) {
            return true;
        }
        if (fromUser.equals(vol.getUser())) {
            return true;
        }
        if (this.projectManager.getRightsForUsernameInProjectId(fromUser.getUsername(), vol.getProject().getProjectId())
            .equals(Rights.ADMIN.toString())) {
            return true;
        }
        return false;
    }

    public boolean canRebootVM(final User fromUser, final VirtualMachine vm) {
        if (fromUser.getAdmin()) {
            return true;
        }
        if (fromUser.equals(vm.getUser())) {
            return true;
        }
        if (this.projectManager.getRightsForUsernameInProjectId(fromUser.getUsername(), vm.getProject().getProjectId()).equals(
            Rights.ADMIN.toString())) {
            return true;
        }
        return false;
    }

    public boolean canUpdateVirtualMachineExpirationDate(final User fromUser, final VirtualMachine vm) {
        if (fromUser.getAdmin()) {
            return true;
        }
        if (fromUser.equals(vm.getUser())) {
            return true;
        }
        if (this.projectManager.getRightsForUsernameInProjectId(fromUser.getUsername(), vm.getProject().getProjectId()).equals(
            Rights.ADMIN.toString())) {
            return true;
        }
        return false;
    }

    public boolean canDestroyVMImage(final User fromUser, final VMImage vmi) {
        if (fromUser.getAdmin()) {
            return true;
        }
        if (fromUser.equals(vmi.getUser())) {
            return true;
        }
        if (this.projectManager.getRightsForUsernameInProjectId(fromUser.getUsername(), vmi.getProject().getProjectId())
            .equals(Rights.ADMIN.toString())) {
            return true;
        }
        return false;
    }

    public boolean canDestroyProject(final User fromUser, final Project project) {
        if (fromUser.getAdmin()) {
            return true;
        }
        return false;
    }

    public boolean canDestroyUser(final User fromUser) {
        if (fromUser.getAdmin()) {
            return true;
        }
        return false;
    }

    public boolean canImportOVF(final User user) {
        // XXX Can everyone import OVF files ?
        if (user.getAdmin()) {
            return true;
        } else {
            return true;
        }
    }

    public boolean canManageSystemInstances(final User user) {
        // XXX Can everyone manage systeminstances ?
        if (user.getAdmin()) {
            return true;
        } else {
            return true;
        }
    }

    public boolean canManageSystemTemplates(final User user) {
        // XXX Can everyone manage systeminstances ?
        if (user.getAdmin()) {
            return true;
        } else {
            return true;
        }
    }

}
