/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: SshCommand.java 36 2011-06-20 14:38:41Z dangtran $
 *
 */

package org.ow2.sirocco.cloudmanager.utils;

import java.io.InputStream;
import java.util.logging.Logger;

import com.jcraft.jsch.ChannelExec;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;
import com.jcraft.jsch.UserInfo;

public class SshCommand {

    private static Logger log = Logger.getLogger(SshCommand.class.getName());

    static public Result command(String host, String user, final String password, String command) throws JSchException {

        try {
            JSch jsch = new JSch();
            Session session = jsch.getSession(user, host, 22);

            session.setUserInfo(new UserInfo() {
                public String getPassphrase() {
                    return null;
                }

                public String getPassword() {
                    return password;
                }

                public boolean promptPassphrase(String arg0) {
                    return true;
                }

                public boolean promptPassword(String arg0) {
                    return true;
                }

                public boolean promptYesNo(String arg0) {
                    return true;
                }

                public void showMessage(String arg0) {
                }
            });

            session.connect();
            ChannelExec channel = (ChannelExec) session.openChannel("exec");
            ((ChannelExec) channel).setCommand(command);
            // channel.setInputStream(null);
            // ((ChannelExec) channel).setErrStream(System.err);
            InputStream in = channel.getInputStream();
            InputStream err = channel.getErrStream();
            channel.connect();

            Result result = new Result();
            StringBuffer output = new StringBuffer();
            StringBuffer error = new StringBuffer();
            byte[] tmp = new byte[1024];

            while (true) {
                while (in.available() > 0) {
                    int i = in.read(tmp, 0, 1024);
                    if (i < 0)
                        break;
                    String s = new String(tmp, 0, i);
                    output.append(s);
                }
                if (channel.isClosed()) {
                    result.exitCode = channel.getExitStatus();
                    break;
                }
                try {
                    Thread.sleep(100);
                } catch (Exception ee) {
                }
            }

            while (true) {
                while (err.available() > 0) {
                    int i = err.read(tmp, 0, 1024);
                    if (i < 0)
                        break;
                    String s = new String(tmp, 0, i);
                    error.append(s);
                }
                if (channel.isClosed()) {
                    result.exitCode = channel.getExitStatus();
                    break;
                }
                try {
                    Thread.sleep(100);
                } catch (Exception ee) {
                    ee.printStackTrace();
                }
            }

            channel.disconnect();
            session.disconnect();
            result.output = new String(output);
            result.error = new String(error);

            return result;

        } catch (Exception ex) {
            log.info("SSH failure: user=[" + user + "] password=[" + password + "]");
            JSchException e = new JSchException();
            e.initCause(ex);
            throw e;
        }
    }
}
