/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: VirtUtils.java 87 2011-06-30 15:07:34Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.utils;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Map;
import java.util.Properties;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.ow2.sirocco.cloudmanager.common.DbParameters;

/**
 * This class provides some utils for the application
 * 
 * @author Serge MONTET
 */
public class VirtUtils {

    /**
     * This method return a random UUID key
     * 
     * @author Serge MONTET
     * @return
     */
    public static String CreateUniqueKey() {
        return Math.abs(UUID.randomUUID().getMostSignificantBits()) + "";
    }

    /**
     * This method the ip adress from a hostname
     * 
     * @author Serge MONTET
     * @param hostname
     * @return
     */
    public static String getIPAdressFromHostName(final String hostname) {

        try {
            InetAddress addr = InetAddress.getByName(hostname);
            byte[] ipAddr = addr.getAddress();

            // Convert to dot representation
            String ipAddrStr = "";
            for (int i = 0; i < ipAddr.length; i++) {
                if (i > 0) {
                    ipAddrStr += ".";
                }
                ipAddrStr += ipAddr[i] & 0xFF;
            }

            return ipAddrStr;

        } catch (UnknownHostException e) {
            return null;
        }
    }

    /**
     * This method return a MegaByte number (more lisible for Memory charge etc
     * ..)
     * 
     * @author Serge MONTET
     * @param byteNumber
     * @return
     */
    public static Long fromByteToMegaByte(final Long byteNumber) {

        Long megaByteNumber = byteNumber / 1024;
        megaByteNumber = megaByteNumber / 1024;
        return megaByteNumber;
    }

    /**
     * This method return the Formated uptime (DD:HH:MM)
     * 
     * @author Serge MONTET
     * @param uptimeInSecond
     * @return
     */
    public static String getFullUptimeInfo(final long uptimeInSecond) {

        long min = uptimeInSecond / 60;
        long hour = min / 60;
        long day = hour / 24;

        String stringtime = day % 24 + ":" + hour % 24 + ":" + min % 60;// +second%60+" Seconds ...";

        return stringtime;
    }

    /**
     * Rounding of a double with n decimal place (after the .).
     * 
     * @author Serge MONTET
     * @param a , the value to be convert.
     * @param n , the total number of decimal to be kept.
     * @return The rounding value with n decimal place.
     */
    public static Double floor(final double a, final int n) {
        Double p = Math.pow(10.0, n);
        return Math.floor((a * p) + 0.5) / p;
    }

    /**
     * Send a email
     * 
     * @throws MessagingException
     */
    public static void sendMail(final String from, final String to, final String subject, final String msg) throws Exception {

        Properties props = System.getProperties(); // Get system properties
        props.put("mail.smtp.host", DbParameters.getInstance().MAIL_HOST);
        props.put("mail.smtp.port", DbParameters.getInstance().MAIL_PORT);
        props.put("mail.smtp.timeout", "30000");
        Session session = Session.getInstance(props, null);

        // Define message
        MimeMessage message = new MimeMessage(session);

        // Set the from address
        try {
            message.setFrom(new InternetAddress(from));
        } catch (AddressException e) {
            throw e;
        } catch (MessagingException e) {
            throw e;
        }

        // Set the to address
        try {
            message.addRecipient(Message.RecipientType.TO, new InternetAddress(to));
        } catch (AddressException e) {
            throw e;
        } catch (MessagingException e) {
            throw e;
        }

        // Set the subject
        try {
            message.setSubject(subject);
            message.setText(msg);
            Transport.send(message);
        } catch (MessagingException e) {
            throw e;
        }

    }

    public static String replace(final String template, final Map<String, String> map) {

        final StringBuilder list = new StringBuilder("\\$(");

        for (final String key : map.keySet()) {
            list.append(key);
            list.append("|");
        }
        list.append("[^\\s\\S])");
        Pattern pattern = Pattern.compile(list.toString());
        Matcher matcher = pattern.matcher(template);
        final StringBuffer stringBuffer = new StringBuffer();
        while (matcher.find()) {
            final String string = matcher.group(1);
            matcher.appendReplacement(stringBuffer, map.get(string));
        }
        matcher.appendTail(stringBuffer);
        return stringBuffer.toString();
    }
}
